unit MAIN;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ImgList, StdActns,
  ActnList, ToolWin, Messages, z2m, Spin;

Const
  WM_POSTINITFORM = (WM_USER+1);
  UM_DLL_NOTIFY = (WM_USER+2);

type
  TfmMain = class(TForm)
    grCurRdInfo: TGroupBox;
    grReaders: TGroupBox;
    lvReaders: TListView;
    labFindRdStatus: TLabel;
    Panel2: TPanel;
    Label4: TLabel;
    edtRdComPort: TEdit;
    Label5: TLabel;
    edtRdSN: TEdit;
    Label6: TLabel;
    edtInfo: TMemo;
    grCards: TGroupBox;
    labFindCdStatus: TLabel;
    Panel3: TPanel;
    btnSearchCd: TButton;
    btnEditCd: TButton;
    lvCards: TListView;
    btnInputKeys: TButton;
    btnIndicators: TButton;
    Panel1: TPanel;
    btnCloseRd: TButton;
    btnOpenRd: TButton;
    btnSearchRd: TButton;
    btnConfig: TButton;
    edtRdVer: TEdit;
    Label2: TLabel;
    btnPowerKey: TButton;
    btnRdTime: TButton;
    btnRdControl: TButton;
    btnTestM3Net: TButton;
    btnSetRdPassw: TButton;
    procedure lvCardsKeyPress(Sender: TObject; var Key: Char);
    procedure lvCardsDblClick(Sender: TObject);
    procedure lvReadersKeyPress(Sender: TObject; var Key: Char);
    procedure lvReadersDblClick(Sender: TObject);
    procedure btnInputKeysClick(Sender: TObject);
    procedure lvCardsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure lvReadersAdvancedCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; Stage: TCustomDrawStage;
      var DefaultDraw: Boolean);
    procedure btnEditCdClick(Sender: TObject);
    procedure btnSearchCdClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnCloseRdClick(Sender: TObject);
    procedure btnOpenRdClick(Sender: TObject);
    procedure btnSearchRdClick(Sender: TObject);
    procedure btnIndicatorsClick(Sender: TObject);
    procedure btnConfigClick(Sender: TObject);
    procedure btnPowerKeyClick(Sender: TObject);
    procedure btnRdControlClick(Sender: TObject);
    procedure btnRdTimeClick(Sender: TObject);
    procedure btnTestM3NetClick(Sender: TObject);
    procedure btnSetRdPasswClick(Sender: TObject);
  private
    { Private declarations }
    procedure PostInitForm(var msg: TMessage); message WM_POSTINITFORM;
    procedure ReaderConnect(Const APort: string);
    procedure ReaderDisconnect(Const APort: string);
    procedure AddCardEvent(Const APort: string; Const ACdNum: Tz2mCardNum; ACdTip: Tz2mCardType);
    procedure RmvCardEvent(Const APort: string);
    procedure UMDllNotify(var msg: TMessage); message UM_DLL_NOTIFY;
  public
    { Public declarations }
    procedure UpdateButtons();

    procedure UpdateRdList();
    procedure ClearRdList();

    procedure UpdateRdInfo();

    procedure UpdateCdList();
    procedure ClearCdList();

    procedure DoOpenRd(AIdx: Integer);
    procedure DoOpenCd(AIdx: Integer);
    procedure DoTestM3Net(AIdx: Integer);
  end;

  TRdOb = class(TObject)
    FPort     : String;
    FSn       : Tz2mReaderSN;
    FType     : Tz2mReaderType;
    FVer      : Cardinal;
  public
    constructor Create(Const APort: String; ASn: Tz2mReaderSN;
        AType: Tz2mReaderType; AVer: Cardinal);
  end;

  TCdOb = class(TObject)
    FTip      : Tz2mCardType;
    FNum      : Tz2mCardNum;
  public
    constructor Create(ATip: Tz2mCardType; Const ANum: Tz2mCardNum);
  end;

var
  fmMain: TfmMain;

  m_hNotifyer: THandle;
  m_hRd: THandle;
  m_sRdPort: String;
  m_nRdSn: Tz2mReaderSN;
  m_nRdType: Tz2mReaderType;
  m_nRdVer: Cardinal;
  m_sRdManuf: String;
  m_nRdPassw: Integer;

implementation

uses
  SysUtils,
  uUL, u1K, u4K, uInpKey, uAppUtils, uAppConst, uIndicators, uConfig,
  uPowerKeyDlg, uRdControlDlg, uRdTimeDlg, uM3NetTest, uRdPassw;

{$R *.dfm}

constructor TRdOb.Create(Const APort: String; ASn: Tz2mReaderSN;
    AType: Tz2mReaderType; AVer: Cardinal);
begin
  inherited Create();
  FPort := APort;
  FSn := ASn;
  FType := AType;
  FVer := AVer;
end;

//==============================================================================

constructor TCdOb.Create(ATip: Tz2mCardType; Const ANum: Tz2mCardNum);
begin
  inherited Create();
  FTip := ATip;
  FNum := ANum;
end;

//==============================================================================

procedure TfmMain.UMDllNotify(var msg: TMessage);
var
  nRet: Integer;
  m: TZ2mNotifyMsg;
  sPort: String;
begin
  ZeroMemory(@m, SizeOf(m));
  repeat
    SetLength(sPort, Z2M_MAX_PORTNAME);
    m.pszRdPort := @sPort[1];
    m.nRdPortMax := Z2M_MAX_PORTNAME;
    nRet := z2mGetNextNotifyMsg(m_hNotifyer, @m);
    if nRet <> Z2M_SUCCESS then break;
    SetLength(sPort, m.nRdPortMax);
    case m.nTip of
      Z2M_M_Reader_Connect:     ReaderConnect(sPort);
      Z2M_M_Reader_Disconnect:  ReaderDisconnect(sPort);
      Z2M_M_Card_Connect:       AddCardEvent(sPort, m.nCdNum, m.nCdTip);
      Z2M_M_Card_Disconnect:    RmvCardEvent(sPort);
    end;
  until False;
end;

procedure TfmMain.ReaderConnect(Const APort: string);
begin
  UpdateRdList();
end;

procedure TfmMain.ReaderDisconnect(Const APort: string);
begin
  UpdateRdList();
end;

procedure TfmMain.AddCardEvent(Const APort: string; Const ACdNum: Tz2mCardNum; ACdTip: Tz2mCardType);
begin
  //    
  UpdateCdList();
  UpdateButtons(); //     
  if (fmUL<>nil) then
    PostMessage(fmUL.Handle, WM_ADDCARD, 0, 0)
  else if (fm1K<>nil) then
    PostMessage(fm1K.Handle, WM_ADDCARD, 0, 0)
  else if (fm4K<>nil) then
    PostMessage(fm4K.Handle, WM_ADDCARD, 0, 0)
  else lvCards.SetFocus;
  if fmM3NetTest.Showing then
    fmM3NetTest.CallAddCard(APort, ACdNum, ACdTip);
end;

procedure TfmMain.RmvCardEvent(Const APort: string);
begin
  //   
  UpdateCdList();
  UpdateButtons(); //     
  if (fmUL<>nil) then
    PostMessage(fmUL.Handle, WM_RMVCARD, 0, 0)
  else if (fm1K<>nil) then
    PostMessage(fm1K.Handle, WM_RMVCARD, 0, 0)
  else if (fm4K<>nil) then
    PostMessage(fm4K.Handle, WM_RMVCARD, 0, 0)
  else lvCards.SetFocus;
  if fmM3NetTest.Showing then
    fmM3NetTest.CallRemoveCard(APort);
end;

procedure TfmMain.btnOpenRdClick(Sender: TObject);
begin
  //   
  if lvReaders.ItemIndex = -1 then exit;
  DoOpenRd(lvReaders.ItemIndex);
end;

procedure TfmMain.btnPowerKeyClick(Sender: TObject);
begin
  fmPowerKeyDlg.FRd := m_hRd;
  fmPowerKeyDlg.ShowModal();
end;

procedure TfmMain.btnRdControlClick(Sender: TObject);
begin
  fmRdControl.FRd := m_hRd;
  fmRdControl.ShowModal();
end;

procedure TfmMain.btnRdTimeClick(Sender: TObject);
begin
  fmRdTime.FRd := m_hRd;
  fmRdTime.ShowModal();
end;

procedure TfmMain.btnCloseRdClick(Sender: TObject);
begin
  if m_hRd <> 0 then
  begin
    z2mCloseReader(m_hRd);
    m_hRd := 0;
  end;
  lvReaders.Repaint();
  UpdateRdInfo();
  lvReaders.Repaint;
  UpdateButtons();
  UpdateCdList();
end;

procedure TfmMain.btnConfigClick(Sender: TObject);
begin
  fmConfigDlg.FRd := m_hRd;
  if fmConfigDlg.ShowModal <> mrOk then exit;
end;

procedure TfmMain.btnEditCdClick(Sender: TObject);
begin
  if lvCards.ItemIndex = -1 then exit;
  DoOpenCd(lvCards.ItemIndex);
end;

procedure TfmMain.btnIndicatorsClick(Sender: TObject);
begin
  fmIndicators.FRd := m_hRd;
  fmIndicators.ShowModal;
end;

procedure TfmMain.btnInputKeysClick(Sender: TObject);
begin
  fmInpKey := TfmInpKey.Create(Self);
  try
    fmInpKey.FRd := m_hRd;
    fmInpKey.FSelectMode := False;
    fmInpKey.ShowModal;  //  "   "
  finally
    FreeAndNil(fmInpKey);
  end;
end;

procedure TfmMain.btnSearchCdClick(Sender: TObject);
begin
  UpdateCdList();
  UpdateButtons(); //     
end;

procedure TfmMain.btnSearchRdClick(Sender: TObject);
begin
  //  
  UpdateRdList();
  UpdateButtons(); //     
end;

procedure TfmMain.DoTestM3Net(AIdx: Integer);
var
  obCd: TCdOb;
begin
  obCd := TCdOb(lvCards.Items[AIdx].Data);
  if not (obCd.FTip in [Z2M_C_1K..Z2M_C_4K]) then
  begin
    MessageDlg(SMSG_CARDNOTSUPPORT, mtInformation, [mbOk], 0);
    exit;
  end;
  fmM3NetTest.FRd := m_hRd;
  fmM3NetTest.FRdPortName := m_sRdPort;
  fmM3NetTest.m_nCdType := obCd.FTip;
  fmM3NetTest.m_nCdNum := obCd.FNum;
  fmM3NetTest.ShowModal();
end;

procedure TfmMain.btnTestM3NetClick(Sender: TObject);
begin
  if lvCards.ItemIndex = -1 then exit;
  DoTestM3Net(lvCards.ItemIndex);
end;

procedure TfmMain.btnSetRdPasswClick(Sender: TObject);
begin
  fmRdPasswDlg.FPassw := m_nRdPassw;
  if fmRdPasswDlg.ShowModal() <> mrOk then exit;
  m_nRdPassw := fmRdPasswDlg.FPassw;
  if m_nRdPassw = Z2M_DEF_M3N_PASSW then
    btnSetRdPassw.Font.Style := []
  else
    btnSetRdPassw.Font.Style := [fsBold];
end;

procedure TfmMain.FormCreate(Sender: TObject);
var
  nDllVer: Cardinal;
  nRet: Integer;
  rNInf: Tz2mNotifyParam;
begin
  fmUL := nil;
  fm1K := nil;
  fm4K := nil;
  fmInpKey := nil;
  labFindRdStatus.Caption := '';
  labFindCdStatus.Caption := '';
  m_nRdPassw := Z2M_DEF_M3N_PASSW;
  //    z2m.dll
  nDllVer := z2mGetVersion();
  if LoByte(nDllVer) <> 2 then
  begin
    MessageDlg(SMSG_VERDLLNOTSUPPORT, mtError, [mbOK], 0);
    halt;
  end;
  nRet := z2mInitialize();
  if nRet <> Z2M_SUCCESS then
  begin
    MessageDlg(SMSG_DLLNOTINIT, mtError, [mbOK], 0);
    halt;
  end;
  ZeroMemory(@rNInf, SizeOf(rNInf));
  with rNInf do
  begin
    hWndHandle := Handle;
    nMsgId := UM_DLL_NOTIFY;
  end;
  nRet := z2mCreateNotifier(@m_hNotifyer, rNInf);
  if nRet <> Z2M_SUCCESS then m_hNotifyer := 0;

  m_hRd := 0;
  UpdateButtons(); //     
  PostMessage(Handle, WM_POSTINITFORM, 0, 0);
end;

procedure TfmMain.PostInitForm(var msg: TMessage);
begin
  UpdateRdList();
  UpdateButtons();
end;

procedure TfmMain.FormDestroy(Sender: TObject);
begin
  if m_hNotifyer <> 0 then
    z2mDestroyNotifier(m_hNotifyer);
  ClearRdList();
  if m_hRd <> 0 then
  begin
    z2mCloseReader(m_hRd);
    m_hRd := 0;
  end;
  z2mFinalize();
end;

procedure TfmMain.lvCardsDblClick(Sender: TObject);
var
  lv: TListView;
  pt: TPoint;
  oLVItem: TListItem;
begin
  lv := (Sender as TListView);
  GetCursorPos(pt);
  pt := lv.ScreenToClient(pt);
  oLVItem := lv.GetItemAt(pt.X, pt.Y);
  if oLVItem=nil then exit;
  DoOpenCd(oLVItem.Index);
end;

procedure TfmMain.lvCardsKeyPress(Sender: TObject; var Key: Char);
var
  lv: TListView;
begin
  if Key<>#13 then exit;
  lv := (Sender as TListView);
  if lv.ItemIndex=-1 then exit;
  DoOpenCd(lv.ItemIndex);
end;

procedure TfmMain.lvCardsSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  UpdateButtons();
end;

procedure TfmMain.lvReadersAdvancedCustomDrawItem(Sender: TCustomListView;
  Item: TListItem; State: TCustomDrawState; Stage: TCustomDrawStage;
  var DefaultDraw: Boolean);
var
  obRd: TRdOb;
begin
  obRd := TRdOb(Item.Data);
  if (m_hRd <> 0) and
    SameText(obRd.FPort, m_sRdPort) and (obRd.FSn = m_nRdSn) then
  begin
    Sender.Canvas.Font.Style := [fsBold];
    DefaultDraw := True;
  end;
end;

procedure TfmMain.lvReadersDblClick(Sender: TObject);
var
  lv: TListView;
  pt: TPoint;
  oLVItem: TListItem;
begin
  lv := (Sender as TListView);
  GetCursorPos(pt);
  pt := lv.ScreenToClient(pt);
  oLVItem := lv.GetItemAt(pt.X, pt.Y);
  if oLVItem=nil then exit;
  DoOpenRd(oLVItem.Index);
end;

procedure TfmMain.lvReadersKeyPress(Sender: TObject; var Key: Char);
var
  lv: TListView;
begin
  if Key <> #13 then exit;
  lv := (Sender as TListView);
  if lv.ItemIndex=-1 then exit;
  DoOpenRd(lv.ItemIndex);
end;

procedure TfmMain.UpdateButtons();
var
  fRdOpened: Boolean;
  nIdx: Integer;
begin
  //     :
  // "Connect", "Disconnect", "Scan card"  "Open card"
  btnOpenRd.Enabled := (lvReaders.ItemIndex <> -1) and (m_hRd=0);
  fRdOpened := (m_hRd <> 0);
  btnCloseRd.Enabled := fRdOpened;
  btnSearchCd.Enabled := fRdOpened;
  btnInputKeys.Enabled := fRdOpened;
  btnIndicators.Enabled := fRdOpened;
  btnConfig.Enabled := fRdOpened and (m_nRdType = Z2M_R_M3N);
  btnPowerKey.Enabled := fRdOpened and (m_nRdType = Z2M_R_M3N);
  btnRdTime.Enabled := fRdOpened and (m_nRdType = Z2M_R_M3N);
  btnRdControl.Enabled := fRdOpened and (m_nRdType = Z2M_R_M3N);
  nIdx := lvCards.ItemIndex;
  btnEditCd.Enabled := (nIdx <> -1) and
      (TCdOb(lvCards.Items[nIdx].Data).FTip in [Z2M_C_UL..Z2M_C_4K]);
//  btnTestM3Net.Enabled := (nIdx <> -1) and
//      (TCdOb(lvCards.Items[nIdx].Data).FTip in [Z2M_C_1K..Z2M_C_4K]);
  btnTestM3Net.Enabled := (nIdx <> -1);
end;

procedure TfmMain.UpdateRdList();
var
  hFind: THandle;
  Info: Tz2mFindReaderInfo;
  obRd: TRdOb;
  nRet: Integer;
  sPort: String;
begin
  labFindRdStatus.Caption := SMSG_RDSSEARCH;
  labFindRdStatus.Font.Color := clRed;
  btnSearchRd.Enabled := False;
  Application.ProcessMessages;
  try
    ClearRdList();
    ZeroMemory(@Info, SizeOf(Info));
    SetLength(sPort, Z2M_MAX_PORTNAME);
    Info.pszPort := @sPort[1];
    Info.nPortMax := Z2M_MAX_PORTNAME;
    nRet := z2mFindFirstReader(@hFind, nil, @Info);
    if nRet = Z2M_SUCCESS then
    begin
      try
        repeat
          SetLength(sPort, Info.nPortMax);
          obRd := TRdOb.Create(sPort, Info.nSN, Info.nTip, Info.nFlashVer);
          try
            with lvReaders.Items.Add do
            begin
              Data := obRd;
              Caption := sPort;
              SubItems.Add(RdTipToStr(Info.nTip));
              SubItems.Add(RdSnToStr(Info.nSN));
            end;
          except
            obRd.Free;
            raise;
          end;
          Application.ProcessMessages;
          SetLength(sPort, Z2M_MAX_PORTNAME);
          Info.pszPort := @sPort[1];
          Info.nPortMax := Z2M_MAX_PORTNAME;
          nRet := z2mFindNextReader(hFind, @Info);
        until nRet <> Z2M_SUCCESS;
      finally
        z2mCloseFindReader(hFind);
      end;
    end;
  finally
    labFindRdStatus.Font.Color := clWindowText;
    btnSearchRd.Enabled := True;
    if lvReaders.Items.Count=0 then
    begin
      labFindRdStatus.Caption := SMSG_RDSNOTFOUND;
      btnSearchRd.SetFocus;
    end
    else
    begin
      labFindRdStatus.Caption := format(SMSG_RDSFOUND,
        [lvReaders.Items.Count]);
      lvReaders.SetFocus;
    end;
  end;
  with lvReaders do
    if (ItemIndex = -1) and (Items.Count > 0) then ItemIndex := 0;
end;

procedure TfmMain.ClearRdList();
var
  obRd: TRdOb;
  i: Integer;
begin
  for i := 0 to lvReaders.Items.Count - 1 do
  begin
    obRd := TRdOb(lvReaders.Items[i].Data);
    obRd.Free;
  end;
  lvReaders.Items.Clear;
end;

procedure TfmMain.UpdateRdInfo();
begin
  if m_hRd = 0 then
  begin
    edtRdComPort.Text := '';
    edtRdSN.Text := '';
    edtRdVer.Text := '';
    edtInfo.Text := '';
  end
  else
  begin
    edtRdComPort.Text := m_sRdPort;
    edtRdSN.Text := RdSnToStr(m_nRdSn);
    edtRdVer.Text := format('%d.%d', [LoByte(m_nRdVer), HiByte(m_nRdVer)]);
    edtInfo.Text := m_sRdManuf;
  end;
end;

procedure TfmMain.UpdateCdList();
var
  Info: Tz2mFindCardInfo;
  obCd: TCdOb;
  nRet: Integer;
begin
  labFindCdStatus.Caption := SMSG_CDSSEARCH;
  labFindCdStatus.Font.Color := clRed;
  btnSearchCd.Enabled := False;
  Application.ProcessMessages;
  try
    ClearCdList();
    if m_hRd <> 0 then
    begin
      nRet := z2mFindCard(m_hRd, @Info);
      if nRet = Z2M_SUCCESS then
      begin
        obCd := TCdOb.Create(Info.nTip, Info.nNum);
        try
          with lvCards.Items.Add do
          begin
            Data := obCd;
            Caption := CdTipToShortStr(Info.nTip);
            SubItems.Add(CdIdToStr(Info.nNum));
          end;
        except
          obCd.Free;
          raise;
        end;
      end;
    end;
  finally
    labFindCdStatus.Font.Color := clWindowText;
    btnSearchCd.Enabled := m_hRd<>0;
    if lvCards.Items.Count=0 then
    begin
      labFindCdStatus.Caption := SMSG_CDSNOTFOUND;
      if btnSearchCd.Enabled then
        btnSearchCd.SetFocus;
    end
    else
    begin
      labFindCdStatus.Caption := format(SMSG_CDSFOUND,
        [lvCards.Items.Count]);
      lvCards.SetFocus;
    end;
  end;
  with lvCards do
    if (ItemIndex = -1) and (Items.Count > 0) then ItemIndex := 0;
end;

procedure TfmMain.ClearCdList();
var
  obCd: TCdOb;
  i: Integer;
begin
  for i := 0 to lvCards.Items.Count - 1 do
  begin
    obCd := TCdOb(lvCards.Items[i].Data);
    obCd.Free;
  end;
  lvCards.Items.Clear;
end;

procedure TfmMain.DoOpenRd(AIdx: Integer);
var
  obRd: TRdOb;
  Info: Tz2mReaderInfo;
  nRet: Integer;
  sManuf: String;
begin
  obRd := TRdOb(lvReaders.Items[lvReaders.ItemIndex].Data);
  nRet := z2mOpenReader(@m_hRd, PChar(obRd.FPort), obRd.FSn, obRd.FType, @m_nRdPassw);
  if nRet <> Z2M_SUCCESS then m_hRd := 0;

  if m_hRd = 0 then
  begin
    MsgBoxZ2mErr(Handle, nRet);
    exit;
  end;
  m_sRdPort := obRd.FPort;
  m_nRdSn := obRd.FSn;
  m_nRdType := obRd.FType;
  m_nRdVer := obRd.FVer;

  ZeroMemory(@Info, SizeOf(Info));
  SetLength(sManuf, Z2M_MAX_MANUFACTOR);
  Info.pszManufactor := @sManuf[1];
  Info.nManufactorMax := Z2M_MAX_MANUFACTOR;
  z2mGetReaderInfo(m_hRd, @Info);
  SetLength(sManuf, Info.nManufactorMax);
  m_sRdManuf := sManuf;
  UpdateRdInfo();
  lvReaders.Repaint;
  UpdateButtons();
  UpdateCdList();
end;

procedure TfmMain.DoOpenCd(AIdx: Integer);
var
  obCd: TCdOb;
begin
  obCd := TCdOb(lvCards.Items[AIdx].Data);
  if not (obCd.FTip in [Z2M_C_UL..Z2M_C_4K]) then
  begin
    MessageDlg(SMSG_CARDNOTSUPPORT, mtInformation, [mbOk], 0);
    exit;
  end;
  case obCd.FTip of
    Z2M_C_UL:
    begin
      fmUL := TfmUL.Create(Self);
      try
        fmUL.FRd := m_hRd;
        fmUL.FCdTip := obCd.FTip;
        fmUL.FCdNum := obCd.FNum;
        fmUL.ShowModal; //  "Mifare UltraLight"
      finally
        FreeAndNil(fmUL);
      end;
    end;
    Z2M_C_1K:
    begin
      fm1K := Tfm1K.Create(Self);
      try
        fm1K.FRd := m_hRd;
        fm1K.FCdTip := obCd.FTip;
        fm1K.FCdId := obCd.FNum;
        fm1K.ShowModal; //  "Mifare 1K"
      finally
        FreeAndNil(fm1K);
      end;
    end;
    Z2M_C_4K:
    begin
      fm4K := Tfm4K.Create(Self);
      try
        fm4K.FRd := m_hRd;
        fm4K.FCdTip := obCd.FTip;
        fm4K.FCdId := obCd.FNum;
        fm4K.ShowModal; //  "Mifare 4K"
      finally
        FreeAndNil(fm4K);
      end;
    end;
  end;
end;

end.
