unit uAppUtils;

interface

uses
  Windows,
  z2m;

type
  TMKKey  = array [0..5] of BYTE;

function RdTipToStr(ATip: Tz2mReaderType): String;

function RdTipToShortStr(ARdTip: Tz2mReaderType): string;

function RdSnToStr(ASn: Tz2mReaderSN): string;
function StrToRdSn(AStr: string; var VSn: Tz2mReaderSN): Boolean;

//      
function CdIdToStr(Const ANum: Tz2mCardNum): string;
//      ()
function CdTipToShortStr(ACdTip: Tz2mCardType): string;

function HexToInt(Const AHex: String): int64;

function IsBit(AVal: Cardinal; AIdx: Integer): Boolean;
procedure SetBit(var AVal: Cardinal; AIdx: Integer; AOn: Boolean=True); overload;

function WinToDos(Const St: string): string;
function DosToWin(Const St: string): string;

function HexToDec(ch: Char): Byte;

procedure MsgBoxZ2mErr(AOwner: HWND; AZ2mErrN: Integer);
function GetZ2mErrorText(AErrN: Integer): String;

function GetRdAuthKeysFilePath(ARdSn: Tz2mReaderSn): String;
function AccessStateToBits(AState: Integer): BYTE;
function AccessBitsToState(ABits: BYTE): Integer;
function GetAppDirPath : string;

function GetAuthKey(ARdSn: Tz2mReaderSn; AKeyN: Integer; var VKeyA, VKeyB: Tz2mAuthKey): Boolean;
function GetAuthKeyCount(ARdSn: Tz2mReaderSn; AKeyTip: Tz2mAKMode): Integer;
procedure WriteAuthKeyToFile(ARdSn: Tz2mReaderSn; AKeyN: Integer;
    AKeyTip: Tz2mAKMode; AKeyVal: Tz2mAuthKey);

implementation

uses
  SysUtils, Classes, Dialogs,
  uAppConst;

function RdTipToStr(ATip: Tz2mReaderType): String;
begin
  case ATip of
    Z2M_R_Z2M:  Result := 'Z2USB-MIFARE R/W';
    Z2M_R_M3N:  Result := 'Matrix 3 Net';
    else        Result := 'Unknown';
  end;
end;

function RdTipToShortStr(ARdTip: Tz2mReaderType): string;
begin
  case ARdTip of
    Z2M_R_Z2M:    Result := 'Z2M';
    Z2M_R_M3N:    Result := 'M3N';
    else          Result := format('[%X]', [ARdTip]);
  end;
end;

function RdSnToStr(ASn: Tz2mReaderSN): string;
begin
  Result := format('%.5d', [ASn]);
end;

function StrToRdSn(AStr: string; var VSn: Tz2mReaderSN): Boolean;
begin
  VSn := StrToInt(AStr);
  Result := True;
end;

function CdIdToStr(Const ANum: Tz2mCardNum): string;
var
  i: Integer;
begin
  for i := 1 to ANum[0] do
    Result := Result + IntToHex(ANum[i], 2);
end;

function CdTipToShortStr(ACdTip: Tz2mCardType): string;
begin
  case ACdTip of
    Z2M_C_UL: Result := 'UL';
    Z2M_C_1K: Result := '1K';
    Z2M_C_4K: Result := '4K';
    else      Result := '';
  end;
end;

function HexToInt(Const AHex: String): int64;
begin
  Result:=StrToInt64('$' + AHex);
end;

function IsBit(AVal: Cardinal; AIdx: Integer): Boolean;
begin
  Result := ((AVal shr (AIdx-1)) and 1) <> 0;
end;

procedure SetBit(var AVal: Cardinal; AIdx: Integer; AOn: Boolean);
begin
  ASSERT(AIdx > 0);
  if AOn then
    AVal := AVal or (1 shl (AIdx-1))
  else
    AVal := AVal and (not (1 shl (AIdx-1)));
end;

function WinToDos(Const St: string): string;
var
  Ch: PAnsiChar;
begin
  Ch := AnsiStrAlloc(Length(St) + 1);
  try
    AnsiToOem(PAnsiChar(AnsiString(St)), Ch);
    Result := String(StrPas(Ch));
  finally
    StrDispose(Ch)
  end;
end;

function DosToWin(Const St: string): string;
var
  Ch: PAnsiChar;
begin
  Ch := AnsiStrAlloc(Length(St) + 1);
  try
    OemToAnsi(PAnsiChar(AnsiString(St)), Ch);
    Result := String(StrPas(Ch));
  finally
    StrDispose(Ch)
  end;
end;

function HexToDec(ch: Char): Byte;
begin
  case ch of
    '0'..'9': Result := Ord(ch)-Ord('0');
    'A'..'F': Result := 10+Ord(ch)-Ord('A');
    'a'..'f': Result := 10+Ord(ch)-Ord('a');
  else        Result := 0;
  end;
end;

function GetZ2mErrorText(AErrN: Integer): String;
var
  nMax: Integer;
begin
  nMax := 255;
  SetLength(Result, nMax);
  z2mGetErrorText(AErrN, PChar(@Result[1]), nMax);
  SetLength(Result, nMax);
end;

procedure MsgBoxZ2mErr(AOwner: HWND; AZ2mErrN: Integer);
var
  s: String;
  nMax: Integer;
begin
  nMax := 255;
  SetLength(s, nMax);
  z2mGetErrorText(AZ2mErrN, PChar(@s[1]), nMax);
  SetLength(s, nMax);
  MessageDlg(s, mtError, [mbOk], 0);
end;

function GetRdAuthKeysFilePath(ARdSn: Tz2mReaderSn): String;
begin
  Result := GetAppDirPath()+'\rd_z2m_krom'+IntToStr(ARdSn)+'.dat';
end;

function AccessStateToBits(AState: Integer): BYTE;
begin
  case AState of
    0: Result := 0;  // 0 0 0
    1: Result := 2;  // 0 1 0
    2: Result := 4;  // 1 0 0
    3: Result := 6;  // 1 1 0
    4: Result := 1;  // 0 0 1
    5: Result := 3;  // 0 1 1
    6: Result := 5;  // 1 0 1
    7: Result := 7;  // 1 1 1
  else Result := 0;
  end;
end;

function AccessBitsToState(ABits: BYTE): Integer;
begin
  case ABits of
    0: Result := 0;  // 0 0 0
    1: Result := 4;  // 0 0 1
    2: Result := 1;  // 0 1 0
    3: Result := 5;  // 0 1 1
    4: Result := 2;  // 1 0 0
    6: Result := 3;  // 1 1 0
    5: Result := 6;  // 1 0 1
    7: Result := 7;  // 1 1 1
  else Result := 0;
  end;
end;

function GetAppDirPath : string;
begin
  Result := ExtractFileDir(ParamStr(0));
end;

function GetAuthKey(ARdSn: Tz2mReaderSn; AKeyN: Integer; var VKeyA, VKeyB: Tz2mAuthKey): Boolean;
var
  oFile: TFileStream;
  sPath: String;
  k: Integer;
  aBuf: packed array[0..6] of BYTE;
begin
  Result := False;
  if AKeyN>16 then exit;
  sPath := GetRdAuthKeysFilePath(ARdSn);
  if FileExists(sPath) then
  begin
    oFile := TFileStream.Create(sPath, fmOpenRead or fmShareExclusive);
    try
      k := (AKeyN-1)*14;
      oFile.Seek(k, soFromBeginning);
      oFile.ReadBuffer(aBuf[0], 7);
      if aBuf[0]=0 then exit;
      Move(aBuf[1], VKeyA, 6);
      oFile.ReadBuffer(aBuf[0], 7);
      if aBuf[0]=0 then exit;
      Move(aBuf[1], VKeyB, 6);
      Result := True;
    finally
      oFile.Free;
    end;
  end;
end;

function GetAuthKeyCount(ARdSn: Tz2mReaderSn; AKeyTip: Tz2mAKMode): Integer;
var
  oFile: TFileStream;
  sPath: String;
  i: Integer;
  aBuf: packed array[0..13] of BYTE;
begin
  Result := 0;
  sPath := GetRdAuthKeysFilePath(ARdSn);
  if FileExists(sPath) then
  begin
    oFile := TFileStream.Create(sPath, fmOpenRead or fmShareExclusive);
    try
      case AKeyTip of
        Z2M_K_KeyA:
        begin
          for i := 0 to 15 do
          begin
            oFile.ReadBuffer(aBuf[0], 14);
            if (aBuf[0]=0) then continue;
            Inc(Result);
          end;
        end;
        Z2M_K_KeyB:
        begin
          for i := 0 to 15 do
          begin
            oFile.ReadBuffer(aBuf[0], 14);
            if (aBuf[7]=0) then continue;
            Inc(Result);
          end;
        end;
        else Assert(False);
      end;
    finally
      oFile.Free;
    end;
  end;
end;

procedure WriteAuthKeyToFile(ARdSn: Tz2mReaderSn; AKeyN: Integer;
    AKeyTip: Tz2mAKMode; AKeyVal: Tz2mAuthKey);
var
  oFile: TFileStream;
  sPath: String;
  nOF: Cardinal;
  m: Integer;
  aBuf: packed array[0..6] of BYTE;
begin
  assert((AKeyN>=0) and (AKeyN <= 15));
  sPath := GetRdAuthKeysFilePath(ARdSn);
  if FileExists(sPath) then nOF := fmOpenWrite else nOF := fmCreate;

  oFile := TFileStream.Create(sPath, nOF or fmShareExclusive);
  try
    if nOF = fmCreate then oFile.Size := 7*16*2;
    if AKeyTip = Z2M_K_KeyB then m :=1  else m := 0;
    oFile.Seek(AKeyN*14+m*7, soFromBeginning);
    aBuf[0] := 1;
    Move(AKeyVal, aBuf[1], 6);
    oFile.WriteBuffer(aBuf[0], 7);
  finally
    oFile.Free;
  end;
end;

end.
