unit uConfig;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Z2m, Spin, Grids;

type
  TfmConfigDlg = class(TForm)
    btnOk: TButton;
    btnCancel: TButton;
    GroupBox2: TGroupBox;
    cbWorkMode: TComboBox;
    Label1: TLabel;
    GroupBox3: TGroupBox;
    Label2: TLabel;
    cbZumm: TComboBox;
    Label3: TLabel;
    cbTM: TComboBox;
    Label4: TLabel;
    cbExit: TComboBox;
    Label5: TLabel;
    cbLock: TComboBox;
    Label6: TLabel;
    cbDoor: TComboBox;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Label7: TLabel;
    cbProt: TComboBox;
    chkImpulse: TCheckBox;
    chkNoCard: TCheckBox;
    chkCardNum: TCheckBox;
    Label8: TLabel;
    cbCardnumFormat: TComboBox;
    GroupBox4: TGroupBox;
    cbSecurity: TComboBox;
    grSecur: TGroupBox;
    Label9: TLabel;
    edtBlockN: TSpinEdit;
    Label10: TLabel;
    btnChooseKeys: TButton;
    labSecurKeyMaskVal: TLabel;
    labKeyType: TLabel;
    rbSecurKeyA: TRadioButton;
    rbSecurKeyB: TRadioButton;
    edtSectN: TSpinEdit;
    labSectN: TLabel;
    procedure FormShow(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure btnChooseKeysClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    m_fConfigChanged: Boolean;
    m_fSecurChanged : Boolean;

    procedure UpdateSecurKeyMaskVal();
  public
    { Public declarations }
    FRd             : THandle;
    FCfg            : Tz2mConfig;
    FSecurSectN     : Integer;
    FSecurBlockN    : Integer;
    FSecurKeyMask   : Integer;
    FSecurKeyB      : Boolean;
  end;

var
  fmConfigDlg: TfmConfigDlg;

implementation

{$R *.dfm}

uses
  uAppUtils, uInpKey;


procedure TfmConfigDlg.btnChooseKeysClick(Sender: TObject);
begin
  fmInpKey := TfmInpKey.Create(Self);
  try
    fmInpKey.FRd := FRd;
    fmInpKey.FSelectMode := True;
    fmInpKey.FKeyMask := FSecurKeyMask;
    if fmInpKey.ShowModal() <> mrOk then exit;
    if fmInpKey.FKeyMask <> FSecurKeyMask then
    begin
      FSecurKeyMask := fmInpKey.FKeyMask;
      m_fSecurChanged := True;
      UpdateSecurKeyMaskVal();
    end;
  finally
    FreeAndNil(fmInpKey);
  end;
end;

procedure TfmConfigDlg.UpdateSecurKeyMaskVal();
begin
  labSecurKeyMaskVal.Caption := format('%Xh', [Cardinal(FSecurKeyMask)]);
end;

procedure TfmConfigDlg.btnOkClick(Sender: TObject);
var
  nRet: Tz2mStatus;
  nIdx, nNewSectN, nNewBlockN: Integer;
  nNewFlags: Cardinal;
  fNewKeyB: Boolean;
begin
  ModalResult := mrNone;

  with FCfg do
  begin
    nIdx := cbWorkMode.ItemIndex;
    if (nIdx <> -1) and (nWorkMode <> nIdx) then
    begin
      nWorkMode := nIdx;
      m_fConfigChanged := True;
    end;

    nIdx := cbZumm.ItemIndex;
    if (nIdx <> -1) and (nOutZumm <> nIdx) then
    begin
      nOutZumm := nIdx;
      m_fConfigChanged := True;
    end;
    nIdx := cbTM.ItemIndex;
    if (nIdx <> -1) and (nOutTM <> nIdx) then
    begin
      nOutTM := nIdx;
      m_fConfigChanged := True;
    end;
    nIdx := cbExit.ItemIndex;
    if (nIdx <> -1) and (nOutExit <> nIdx) then
    begin
      nOutExit := nIdx;
      m_fConfigChanged := True;
    end;
    nIdx := cbLock.ItemIndex;
    if (nIdx <> -1) and (nOutLock <> nIdx) then
    begin
      nOutLock := nIdx;
      m_fConfigChanged := True;
    end;
    nIdx := cbDoor.ItemIndex;
    if (nIdx <> -1) and (nOutDoor <> nIdx) then
    begin
      nOutDoor := nIdx;
      m_fConfigChanged := True;
    end;

    nIdx := cbProt.ItemIndex;
    if (nIdx <> -1) and (nProt <> nIdx) then
    begin
      nProt := nIdx;
      m_fConfigChanged := True;
    end;
    nNewFlags := 0;
    if chkImpulse.Checked then nNewFlags := nNewFlags or 1;
    if chkNoCard.Checked then nNewFlags := nNewFlags or 2;
    if chkCardNum.Checked then nNewFlags := nNewFlags or 4;
    if nFlags <> nNewFlags then
    begin
      nFlags := nNewFlags;
      m_fConfigChanged := True;
    end;
    nIdx := cbCardnumFormat.ItemIndex;
    if (nIdx <> -1) and (nCardFormat <> nIdx) then
    begin
      nCardFormat := nIdx;
      m_fConfigChanged := True;
    end;

    nIdx := cbSecurity.ItemIndex;
    if (nIdx <> -1) and (nSecurityMode <> nIdx) then
    begin
      nSecurityMode := nIdx;
      m_fConfigChanged := True;
    end;
  end;
  fNewKeyB := rbSecurKeyB.Checked;
  if FSecurKeyB <> fNewKeyB then
  begin
    FSecurKeyB := fNewKeyB;
    m_fSecurChanged := True;
  end;
  nNewSectN := edtSectN.Value;
  if nNewSectN <> FSecurSectN then
  begin
    FSecurSectN := nNewSectN;
    m_fSecurChanged := True;
  end;

  nNewBlockN := edtBlockN.Value;
  if nNewBlockN <> FSecurBlockN then
  begin
    FSecurBlockN := nNewBlockN;
    m_fSecurChanged := True;
  end;

  if m_fConfigChanged then
  begin
    nRet := z2mSetConfig(FRd, @FCfg);
    if nRet <> Z2M_SUCCESS then
    begin
      MsgBoxZ2mErr(Handle, nRet);
      exit;
    end;
    m_fConfigChanged := False;
  end;

  if m_fSecurChanged then
  begin
    nRet := z2mSetReaderSecurity(FRd, FSecurSectN, FSecurBlockN, Cardinal(FSecurKeyMask), FSecurKeyB);
    if nRet <> Z2M_SUCCESS then
    begin
      MsgBoxZ2mErr(Handle, nRet);
      exit;
    end;
    m_fSecurChanged := False;
  end;
  ModalResult := mrOk;
end;

procedure TfmConfigDlg.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to PageControl1.PageCount - 1 do
    PageControl1.Pages[i].TabVisible := False;
end;

procedure TfmConfigDlg.FormShow(Sender: TObject);
var
  nRet: Tz2mStatus;
  f: Bool;
begin
  m_fSecurChanged := False;
  m_fConfigChanged := False;
  nRet := z2mGetConfig(FRd, @FCfg);
  if nRet <> Z2M_SUCCESS then
  begin
    MsgBoxZ2mErr(Handle, nRet);
    exit;
  end;
  nRet := z2mGetReaderSecurity(FRd, @FSecurSectN, @FSecurBlockN, @FSecurKeyMask, @f);
  if nRet <> Z2M_SUCCESS then
  begin
    MsgBoxZ2mErr(Handle, nRet);
    exit;
  end;
  FSecurKeyB := f;

  cbWorkMode.ItemIndex := FCfg.nWorkMode;
  cbZumm.ItemIndex := FCfg.nOutZumm;
  cbTM.ItemIndex := FCfg.nOutTM;
  cbExit.ItemIndex := FCfg.nOutExit;
  cbLock.ItemIndex := FCfg.nOutLock;
  cbDoor.ItemIndex := FCfg.nOutDoor;
  cbProt.ItemIndex := FCfg.nProt;
  chkImpulse.Checked := (FCfg.nFlags and 1) <> 0;
  chkNoCard.Checked := (FCfg.nFlags and 2) <> 0;
  chkCardNum.Checked := (FCfg.nFlags and 4) <> 0;
  cbCardnumFormat.ItemIndex := FCfg.nCardFormat;
  cbSecurity.ItemIndex := FCfg.nSecurityMode;

  rbSecurKeyB.Checked := FSecurKeyB;
  edtSectN.Value := FSecurSectN;
  edtBlockN.Value := FSecurBlockN;
  PageControl1.ActivePageIndex := 0;
  UpdateSecurKeyMaskVal();
end;

end.
