unit uInpKey;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ComCtrls, StdCtrls, ExtCtrls,
  z2m;

Const
  WM_POSTINITFORM = WM_USER+1;

type
  TAAuthKeys = array [0..1, 0..15] of Tz2mAuthKey;

  TfmInpKey = class(TForm)
    gridKeys: TStringGrid;
    panInfo: TPanel;
    labRdTitle: TLabel;
    edtRdTitle: TEdit;
    Panel1: TPanel;
    btnApply: TButton;
    btnRefresh: TButton;
    btnOk: TButton;
    btnReset: TButton;
    btnCancel: TButton;
    pbTransfer: TProgressBar;
    procedure btnResetClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnRefreshClick(Sender: TObject);
    procedure gridKeysSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure gridKeysKeyPress(Sender: TObject; var Key: Char);
    procedure gridKeysDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormCreate(Sender: TObject);
    procedure gridKeysClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
  private
    { Private declarations }
    m_nCurCol     : Integer;
    m_nCurRow     : Integer;
    m_fCellEdit   : Boolean;
    m_nCellSubN   : Integer;

    m_aOldAKeys   : TAAuthKeys;
    m_aOldAKValid : array [0..1, 0..15] of Boolean;

    m_aNewAKeys   : TAAuthKeys;
    m_aNewAKValid : array [0..1, 0..15] of Boolean;

    m_fChanged    : Boolean;
    m_fDataInit   : Boolean;
    m_nRdSN       : Tz2mReaderSn;
    m_nRdTip      : Tz2mReaderType;
  protected
    procedure PostInitForm(var msg: TMessage); message WM_POSTINITFORM;
    procedure DoneCellEdit(ACancel: Boolean=False);
    procedure NextEditCell();
    procedure UpdateCell(Const ACol, ARow: Integer);
    procedure SetModifyFlag(AModify: Boolean=True);
    procedure DoRefresh();
    procedure DoApply();
    procedure UpdateButtons();
    function GetRdAKFilePath(): String;
    procedure UpdatePanInfo();
  public
    { Public declarations }
    FRd         : THandle;
    FSelectMode : Boolean;
    FKeyMask    : Integer;
  end;

var
  fmInpKey: TfmInpKey;

implementation

uses
  uAppUtils, StrUtils, uAppConst;

{$R *.dfm}

procedure TfmInpKey.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  // Init data
  FRd := 0;
  m_fChanged := False;
  m_nCurCol := -1;
  m_nCurRow := -1;
  m_fCellEdit := False;
  m_nCellSubN := -1;

  // Init controls
  gridKeys.ColWidths[0] := 45;
  for i := 1 to gridKeys.ColCount - 1 do gridKeys.ColWidths[i] := 113;
  gridKeys.Rows[0][0] := 'Cell#';
  for i := 1 to gridKeys.RowCount - 1 do gridKeys.Rows[i][0] := IntToStr(i);
  gridKeys.Cells[1,0] := 'Key A';
  gridKeys.Cells[2,0] := 'Key B';
  gridKeys.ColWidths[1] := 135;
  gridKeys.ColWidths[2] := 135;
  UpdateButtons();
end;

procedure TfmInpKey.FormShow(Sender: TObject);
begin
  PostMessage(Handle, WM_POSTINITFORM, 0, 0);
  btnCancel.Visible := FSelectMode;
end;

procedure TfmInpKey.PostInitForm(var msg: TMessage);
var
  Info: Tz2mReaderInfo;
  nRet: Integer;
begin
  if FRd = 0 then exit;
  ZeroMemory(@info, SizeOf(info));
  nRet := z2mGetReaderInfo(FRd, @Info);
  if nRet <> Z2M_SUCCESS then
  begin
    MsgBoxZ2mErr(Handle, nRet);
    Close;
    exit;
  end;
  m_nRdSN   := Info.nSN;
  m_nRdTip  := Info.nTip;
//  if m_nRdTip <> Z2M_R_Z2M then
//  begin
//    MessageDlg(SMSG_RDNOTSUPPORT, mtError, [mbOk], 0);
//    Close;
//    exit;
//  end;
  UpdatePanInfo();
  DoRefresh();
end;

procedure TfmInpKey.gridKeysClick(Sender: TObject);
var
  grid: TStringGrid;
  i, y, k: Integer;
  pt: TPoint;
  rc: TRect;
begin
  grid := (Sender as TStringGrid);
  GetCursorPos(pt);
  pt := grid.ScreenToClient(pt);
  if pt.X > (grid.ColWidths[0]) then exit;

  y := 0;
  if pt.Y < y then exit;

  k := -1;
  for i := 1 to grid.RowCount - 1 do
  begin
    rc := grid.CellRect(0, i);
    if PtInRect(rc, pt) then
    begin
      k := i;
      break;
    end;
  end;
  if k <> -1 then
  begin
    SetBit(Cardinal(FKeyMask), k, not IsBit(Cardinal(FKeyMask), k));
    grid.Invalidate();
  end;
end;

procedure TfmInpKey.gridKeysDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  grid: TStringGrid;
  s: string;
  dx, dy: integer;
  a, b, n: Integer;
  rc: TRect;
begin
  grid := (Sender as TStringGrid);
  with grid.Canvas do
  begin
    Font := grid.Font;
    Pen.Color := clBlack;
    if (ACol=0) or (ARow=0) then
    begin
      Brush.Color := grid.FixedColor;
      FillRect(Rect);
      Font.Color := clBlue;
      s := grid.Cells[ACol, ARow];
      dx := TextWidth(s) + 2;
      dy := TextHeight(s) + 2;

      if FSelectMode and (ACol=0) and (ARow > 0) then
      begin
        rc := Rect; rc.Right := rc.Left + 16; rc.Bottom := rc.Top + 16;
        if IsBit(FKeyMask, ARow) then n := DFCS_CHECKED else n := 0;
        DrawFrameControl(Handle, rc, DFC_BUTTON, DFCS_BUTTONCHECK or n);
        TextOut(
          16 + (Rect.Left+Rect.Right-dx-16)div 2,
          (Rect.Top+Rect.Bottom-dy)div 2,
          s);
      end
      else
      begin
        TextOut(
          (Rect.Left+Rect.Right-dx)div 2,
          (Rect.Top+Rect.Bottom-dy)div 2,
          s);
      end;
    end
    else
    begin
      Brush.Color := grid.Color;
      Brush.Style := bsSolid;
      FillRect(Rect);

      s := grid.Cells[ACol, ARow];
      dy := TextHeight(s) + 2;

      if (ACol = m_nCurCol) and (ARow = m_nCurRow) then
      begin
        Pen.Width := 3;
        Brush.Style := bsClear;
        Rectangle(Rect);
      end;

      SetBkMode(Handle, TRANSPARENT);
      TextOut(
        Rect.Left+10,
        (Rect.Top+Rect.Bottom-dy)div 2,
        s);
      if (ACol = m_nCurCol) and (ARow = m_nCurRow)
        and m_fCellEdit and (m_nCellSubN > 0) then
      begin
        a := TextWidth(Copy(s, 1, m_nCellSubN-1));
        b := TextWidth(Copy(s, m_nCellSubN, 1));
        Font.Color := clBlue;
        rc.Left := Rect.Left+ 10+a;
        rc.Right := rc.Left+b;
        rc.Top := (Rect.Top+Rect.Bottom-dy)div 2+dy;
        rc.Bottom := rc.Top;
        MoveTo(rc.Left, rc.Top);
        LineTo(rc.Right, rc.Bottom);
      end;
    end;
  end
end;

procedure TfmInpKey.gridKeysKeyPress(Sender: TObject; var Key: Char);
var
  grid: TStringGrid;
  s: String;
  k1,k2: Integer;
begin
  if (m_nCurCol <= 0) or (m_nCurRow <= 0) then exit;
  grid := (Sender as TStringGrid);
  case Key of
    #27: begin DoneCellEdit(True); exit end;
    #13: begin DoneCellEdit(False); NextEditCell(); exit end;
  end;
  s := grid.Cells[m_nCurCol, m_nCurRow];
  case Key of
    '0'..'9','a'..'f','A'..'F':
    begin
      if (not m_fCellEdit) then m_nCellSubN := 1;

      if Length(s) < m_nCellSubN then
        SetLength(s,m_nCellSubN);
      if CharInSet(Key, ['a'..'f']) then
        s[m_nCellSubN] := Chr(Ord(Key)+Ord('A')-Ord('a'))
      else
        s[m_nCellSubN] := Key;
      Inc(m_nCellSubN);
      if (m_nCellSubN mod 3 = 0) and (m_nCellSubN<17) then // FF.FF.FF.FF.FF.FF
      begin
        if Length(s)<m_nCellSubN then SetLength(s,m_nCellSubN);
        s[m_nCellSubN] := '.';
        Inc(m_nCellSubN);
      end;
      m_fCellEdit := True;
      grid.Cells[m_nCurCol, m_nCurRow] := s;
    end;
    #127:
    begin
      k1 := m_nCurCol-1; k2 := m_nCurRow-1;
      m_aNewAKValid[k1,k2] := m_aOldAKValid[k1,k2];
      m_aNewAKeys[k1,k2] := m_aOldAKeys[k1,k2];
      UpdateCell(m_nCurCol, m_nCurRow);
      DoneCellEdit(False);
    end;
    #8:
    begin
      if GetKeyState(VK_SHIFT) and $80 <> 0 then
      begin
//        grid.Cells[m_nCurCol, m_nCurRow] := '';
//        m_nCellSubN := 17;
//        m_fCellEdit := True;
      end
      else
      begin
        if (not m_fCellEdit) or (m_nCellSubN <= 0) then exit;
        if m_nCellSubN > 1 then Dec(m_nCellSubN);
        if (m_nCellSubN mod 3 = 0) then Dec(m_nCellSubN);
        SetLength(s, m_nCellSubN-1);
        grid.Cells[m_nCurCol, m_nCurRow] := s;
        m_fCellEdit := True;
      end;
    end;
  end;
  if m_fCellEdit and (m_nCellSubN > 17) then
  begin
    DoneCellEdit(False);
    NextEditCell();
  end;
end;

procedure TfmInpKey.gridKeysSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
var
  grid: TStringGrid;
begin
  CanSelect := (ACol >= 1) and (ARow >= 1);
  if CanSelect and ((m_nCurCol <> ACol) or (m_nCurRow <> ARow)) then
  begin
    grid := (Sender as TStringGrid);
    DoneCellEdit();
    m_nCurCol := ACol;
    m_nCurRow := ARow;
    grid.Invalidate;
  end
end;

procedure TfmInpKey.btnApplyClick(Sender: TObject);
begin
  DoApply();
end;

procedure TfmInpKey.btnCancelClick(Sender: TObject);
begin
//  Close;
end;

procedure TfmInpKey.btnOkClick(Sender: TObject);
begin
//  Close;
end;

procedure TfmInpKey.btnRefreshClick(Sender: TObject);
begin
  DoRefresh();
end;

procedure TfmInpKey.btnResetClick(Sender: TObject);
var
  i,j: Integer;
begin
  for i := 0 to 1 do
  for j := 0 to 15 do
  begin
    m_aNewAKValid[i,j] := True;
    m_aNewAKeys[i,j] := $FFFFFFFFFFFF;
    UpdateCell(i+1, j+1);
  end;
  SetModifyFlag();
end;

procedure TfmInpKey.DoneCellEdit(ACancel: Boolean);
var
  s: String;
  nAK: Tz2mAuthKey;
  fValid: Boolean;
begin
  if not m_fCellEdit then exit;
  if ACancel then
    UpdateCell(m_nCurCol, m_nCurRow)
  else
  begin
    s := gridKeys.Cells[m_nCurCol, m_nCurRow];
    if s='' then
    begin
      fValid := False;
      nAK := 0;
    end
    else
    begin
      fValid := True;
      s := AnsiReplaceStr(s, '.', '');
      nAK := HexToInt(s);
      SetModifyFlag(True);
    end;
    m_aNewAKValid[m_nCurCol-1, m_nCurRow-1] := fValid;
    m_aNewAKeys[m_nCurCol-1, m_nCurRow-1] := nAK;
    SetModifyFlag();
  end;
  m_fCellEdit := False;
  m_nCellSubN := 0;
end;

procedure TfmInpKey.UpdateCell(Const ACol, ARow: Integer);
var
  s, s2: String;
  i: Integer;
begin
  assert((ACol>0) and (ARow>0));
  if m_aNewAKValid[ACol-1, ARow-1] then
  begin
    s2 := IntToHex(m_aNewAKeys[ACol-1, ARow-1], 12);
    for i := 0 to 5 do
      s := s + Copy(s2, 1+i*2, 2)+'.';
    Delete(s, Length(s), 1);
  end
  else
    s := '';
  gridKeys.Cells[ACol, ARow] := s;
end;

procedure TfmInpKey.SetModifyFlag(AModify: Boolean);
begin
  if m_fChanged = AModify then exit;
  m_fChanged := AModify;
  UpdatePanInfo();
  UpdateButtons();
end;

procedure TfmInpKey.DoRefresh();
var
  i, j: integer;
  sPath: String;
  oFile: TFileStream;
  aBuf: packed array[0..6] of BYTE;
begin
  pbTransfer.Position := 0;
  pbTransfer.Visible := True;
  try
    sPath := GetRdAKFilePath();
    if FileExists(sPath) then
    begin
      oFile := TFileStream.Create(sPath, fmOpenRead or fmShareExclusive);
      try
        for j := 0 to 15 do
        begin
          for i := 0 to 1 do
          begin
            oFile.ReadBuffer(aBuf[0], 7);
            m_aOldAKValid[i,j] := aBuf[0]<>0;
            Move(aBuf[1], m_aOldAKeys[i,j], 6);
            pbTransfer.StepBy(1);
          end;
        end;
      finally
        oFile.Free;
      end;
    end
    else
    begin
      for i := 0 to 1 do
      begin
        for j := 0 to 15 do
        begin
          m_aOldAKValid[i,j] := False;
          m_aOldAKeys[i,j] := $FFFFFFFFFFFF;
          pbTransfer.StepBy(1);
        end;
      end;
    end;
    Move(m_aOldAKeys[0,0], m_aNewAKeys[0,0], sizeof(m_aNewAKeys));
    Move(m_aOldAKValid[0,0], m_aNewAKValid[0,0], sizeof(m_aNewAKValid));
    for i := 1 to 2 do for j := 1 to 16 do UpdateCell(i, j);
    m_fDataInit := True;
    SetModifyFlag(False);
  finally
    pbTransfer.Visible := False;
  end;
end;

procedure TfmInpKey.DoApply();
var
  i, j, nRet, nMax: integer;
  nAKMode: Tz2mAKMode;
  sPath: String;
  oFile: TFileStream;
  aBuf: packed array[0..6] of BYTE;
  s, s2: string;
begin
  DoneCellEdit(False);
  pbTransfer.Position := 0;
  pbTransfer.Visible := True;
  try
    if (CompareMem(@m_aOldAKeys[0,0], @m_aNewAKeys[0,0],
      sizeof(m_aOldAKeys)) and
      CompareMem(@m_aOldAKValid[0,0], @m_aNewAKValid[0,0],
      sizeof(m_aOldAKValid))) then
    begin
      SetModifyFlag(False);
      exit;
    end;
    sPath := GetRdAKFilePath();
    if not FileExists(sPath) then
    begin
      oFile := TFileStream.Create(sPath, fmCreate);
      try
        oFile.Size := 7*16*2;
      finally
        oFile.Free;
      end;
    end;
    oFile := TFileStream.Create(sPath, fmOpenWrite or fmShareExclusive);
    try
      for i := 0 to 1 do
      begin
        if i=1 then nAKMode := Z2M_K_KeyB else nAKMode := Z2M_K_KeyA;
        for j := 0 to 15 do
        begin
          oFile.Seek(j*14+i*7, soFromBeginning);
          if m_aNewAKValid[i, j] then
          begin
            nRet := z2mSetAuthKey(FRd, nAKMode, j, m_aNewAKeys[i,j]);
            if nRet = Z2M_SUCCESS then
            begin
              Move(m_aNewAKeys[i,j], m_aOldAKeys[i,j], sizeof(Tz2mAuthKey));
              m_aOldAKValid[i,j] := m_aNewAKValid[i,j];
            end
            else
            begin
              nMax := 255;
              SetLength(s, nMax);
              z2mGetErrorText(nRet, PChar(s), nMax);
              if i=1 then s2 := 'B' else s2 := 'A';
              if MessageDlg(format(SMSG_WRITEKEYERR2, [j+1, s2, s]),
                mtError, [mbOk, mbCancel], 0) <> mrOk then
                exit;
            end;
          end
          else
          begin
            m_aOldAKValid[i,j] := m_aNewAKValid[i,j];
          end;
          if m_aOldAKValid[i,j] then aBuf[0] := 1 else aBuf[0] := 0;
          Move(m_aOldAKeys[i,j], aBuf[1], 6);
          oFile.WriteBuffer(aBuf[0], 7);
          pbTransfer.StepBy(1);
        end;
      end;
    finally
      oFile.Free;
    end;
    SetModifyFlag(False);
  finally
    pbTransfer.Visible := False;
  end;
end;

procedure TfmInpKey.UpdateButtons();
begin
  btnApply.Enabled := (FRd<>0) and m_fChanged;
  btnRefresh.Enabled := FRd<>0;
end;

function TfmInpKey.GetRdAKFilePath(): String;
begin
  Result := GetRdAuthKeysFilePath(m_nRdSN);
end;

procedure TfmInpKey.UpdatePanInfo();
var
  s: string;
begin
  if m_fChanged then s := '*' else s := '';
  edtRdTitle.Text := format('%s %s %s',
    [RdSnToStr(m_nRdSN),
      RdTipToShortStr(m_nRdTip),
      s]);
end;

procedure TfmInpKey.NextEditCell();
var
  i, j, i0, j0: Integer;
  fTabOk: Boolean;
begin
  DoneCellEdit(False);
  fTabOk := False;
  i := m_nCurRow;

  if m_nCurCol = -1 then j0 := 1 else j0 := m_nCurCol;
  if m_nCurRow = -1 then i0 := 1 else i0 := m_nCurRow;

  //     
  for j := j0+1 to 2 do
  begin
    fTabOk := True; break;
  end;
  //     
  if not fTabOk then
  begin
    for i := i0+1 to 17 do
    begin
      for j := 1 to 2 do
      begin
        fTabOk := True; break;
      end;
      if fTabOk then break;
    end;
  end;
  //        
  if not fTabOk then
  begin
    for i := 1 to i0-1 do
    begin
      for j := 1 to 2 do
      begin
        fTabOk := True; break;
      end;
      if fTabOk then break;
    end;
  end;
  //        
  if not fTabOk then
  begin
    for j := 1 to j0-1 do
    begin
      fTabOk := True; break;
    end;
  end;
  if fTabOk then
  begin
    gridKeys.Col := j;
    gridKeys.Row := i;
    gridKeys.Invalidate;
  end;
end;

end.
