unit uM3NetTest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Mask, ComCtrls, ExtCtrls, Spin, z2m;

type
  TfmM3NetTest = class(TForm)
    rgTestMode: TRadioGroup;
    btnStartStop: TButton;
    chkCodeAutoInc: TCheckBox;
    edtCodeBlockN: TSpinEdit;
    labCodeBlockN: TLabel;
    labCode: TLabel;
    edtCode: TMaskEdit;
    edtKey: TMaskEdit;
    labKey: TLabel;
    grOpenSectParams: TGroupBox;
    edtOpenSectKey: TMaskEdit;
    chkOpenSectKey: TCheckBox;
    edtSectN: TSpinEdit;
    labSectN: TLabel;
    labStatus: TStaticText;
    labKeyType: TLabel;
    rbKeyA: TRadioButton;
    rbKeyB: TRadioButton;
    labOpenKeyType: TLabel;
    rbOpenKeyA: TRadioButton;
    rbOpenKeyB: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure btnStartStopClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure rgTestModeClick(Sender: TObject);
    procedure chkOpenSectKeyClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FRd             : THandle;
    FRdPortName     : String;

    m_nCdType       : Tz2mCardType;
    m_nCdNum        : Tz2mCardNum;
    m_nOpenKey      : Tz2mAuthKey;
    m_fStarted      : Boolean;

    procedure UpdateControls();

    procedure CallAddCard(Const APort: string; Const ACdNum: Tz2mCardNum; ACdType: Tz2mCardType);
    procedure CallRemoveCard(Const APort: string);

    procedure DoInitCard();
    function KeyToStr(Const AKey: Tz2mAuthKey): String;
    function StrToKey(Const AStr: String): Tz2mAuthKey;
    procedure SetStatus(Const AText: String);
  end;

var
  fmM3NetTest: TfmM3NetTest;

implementation

{$R *.dfm}

uses
  uAppUtils;


procedure TfmM3NetTest.UpdateControls();
var
  nMode: Integer;
  fEditPar, fCode, fOpenKey: Boolean;
begin
  fEditPar := (not m_fStarted);
  nMode := rgTestMode.ItemIndex;
  labKey.Enabled := fEditPar;
  edtKey.Enabled := fEditPar;
  labKeyType.Enabled := fEditPar;
  rbKeyA.Enabled := fEditPar;
  rbKeyB.Enabled := fEditPar;
  labSectN.Enabled := fEditPar;
  edtSectN.Enabled := fEditPar;
  grOpenSectParams.Enabled := fEditPar;
  fCode := (nMode = 1) and fEditPar;
  labCodeBlockN.Enabled := fCode;
  edtCodeBlockN.Enabled := fCode;
  labCode.Enabled := fCode;
  edtCode.Enabled := fCode;
  chkCodeAutoInc.Enabled := fCode;
  rgTestMode.Enabled := fEditPar;
  fOpenKey := chkOpenSectKey.Checked and fEditPar;
  chkOpenSectKey.Enabled := fEditPar;
  edtOpenSectKey.Enabled := fOpenKey;
  labOpenKeyType.Enabled := fEditPar;
  rbOpenKeyA.Enabled := fEditPar;
  rbOpenKeyB.Enabled := fEditPar;
end;

procedure TfmM3NetTest.SetStatus(Const AText: String);
begin
  labStatus.Caption := AText;
end;

procedure TfmM3NetTest.DoInitCard();
var
  nOpenSectN, nTestMode, nTrailerBlockN, nCodeBlockN: Integer;
  nRet: Integer;
  rSectPar: Tz2mMfSectorInfo;
  hOpenCd, hOpenSc: THandle;
  aBuf16: array[0..15] of Byte;
  nKeyAdr: Integer;
  nKey: Tz2mAuthKey;
//  i: Integer;
  nCode: Int64;
begin
  hOpenCd := 0;
  hOpenSc := 0;
  nCodeBlockN := 0;
  try
    nOpenSectN := edtSectN.Value;
    nKey := StrToKey(edtKey.Text);
    nTestMode := rgTestMode.ItemIndex;
    if nTestMode = 1 then
    begin
      nCodeBlockN := edtCodeBlockN.Value;
      nCode := StrToInt64('$' + edtCode.Text);
    end;
    if nOpenSectN > 31 then
      nTrailerBlockN := 15
    else
      nTrailerBlockN := 3;
    nRet := z2mOpenCard(@hOpenCd, FRd, m_nCdType, m_nCdNum);
    if nRet <> Z2M_SUCCESS then
    begin
      SetStatus(format('Init card: open card error (%s).', [GetZ2mErrorText(nRet)]));
      exit;
    end;
    FillChar(rSectPar, SizeOf(rSectPar), 0);
    with rSectPar do
    begin
      hCard := hOpenCd;
      nSectN := nOpenSectN;
      fOpenByMask := False;
      if rbOpenKeyB.Checked then nKeyMode := Z2M_K_KeyB else nKeyMode := Z2M_K_KeyA;
      if chkOpenSectKey.Checked then
        nKey := StrToKey(edtOpenSectKey.Text)
      else
        nKey := z2mDefAuthKey;
    end;
    nRet := z2mOpenMfSector(@hOpenSc, @rSectPar);
    if nRet <> Z2M_SUCCESS then
    begin
      SetStatus(format('Init card: open sector error (%s).', [GetZ2mErrorText(nRet)]));
      exit;
    end;
    if nTestMode = 1 then
    begin
      FillChar(aBuf16[0], SizeOf(aBuf16), 0);
//      for i := 0 to 3 do
//        aBuf16[i] := a[3-i];
      Move(nCode, aBuf16[0], 5);
      nRet := z2mWriteMfBlock(hOpenSc, nCodeBlockN, @aBuf16[0]);
      if nRet <> Z2M_SUCCESS then
      begin
        SetStatus(format('Init card: write code error (%s).', [GetZ2mErrorText(nRet)]));
        exit;
      end;
    end;
    nRet := z2mReadMfBlock(hOpenSc, nTrailerBlockN, @aBuf16[0]);
    if nRet <> Z2M_SUCCESS then
    begin
      SetStatus(format('Init card: read trailer error (%s).', [GetZ2mErrorText(nRet)]));
      exit;
    end;
    if rbKeyB.Checked then nKeyAdr := 10 else nKeyAdr := 0;
    Move(nKey, aBuf16[nKeyAdr], 6);
    nRet := z2mWriteMfBlock(hOpenSc, nTrailerBlockN, @aBuf16[0]);
    if nRet <> Z2M_SUCCESS then
    begin
      SetStatus(format('Init card: write trailer error (%s).', [GetZ2mErrorText(nRet)]));
      exit;
    end;
    SetStatus('Init card: success.');
    if (nTestMode = 1) and chkCodeAutoInc.Checked then
    begin
{$IFOPT Q+}
{$Q-} Inc(nCode); {$Q+}
{$ELSE}
      Inc(nCode);
{$ENDIF}
      edtCode.Text := IntToHex(nCode, 10);
    end;
  finally
    if hOpenCd <> 0 then
      z2mCloseCard(hOpenCd);
  end;
end;

procedure TfmM3NetTest.CallAddCard(Const APort: string; Const ACdNum: Tz2mCardNum; ACdType: Tz2mCardType);
begin
  if FRdPortName <> APort then exit;

  SetStatus(format('Card [%s %s] found.', [CdTipToShortStr(ACdType), CdIdToStr(ACdNum)]));
  m_nCdType := ACdType;
  m_nCdNum := ACdNum;
  if m_fStarted then
    DoInitCard();
end;

procedure TfmM3NetTest.CallRemoveCard(Const APort: string);
begin
  if FRdPortName <> APort then exit;
  SetStatus('Card removed.');
end;

function TfmM3NetTest.KeyToStr(Const AKey: Tz2mAuthKey): String;
//var
//  i: Integer;
begin
  Result := Result + IntToHex(AKey, 12);
end;

function TfmM3NetTest.StrToKey(Const AStr: String): Tz2mAuthKey;
//var
//  i, n: Integer;
//  a: array[0..8] of Byte absolute Result;
begin
//  n := 1;
//  for i := (Length(AStr) div 2)-1 downto 0 do
//  begin
//    a[i] := HexToInt(AStr[n] + AStr[n+1]);
//    Inc(n, 2);
//  end;
  Result := StrToInt64('$'+AStr);
end;

procedure TfmM3NetTest.chkOpenSectKeyClick(Sender: TObject);
var
  fOpenKey: Boolean;
begin
  fOpenKey := chkOpenSectKey.Checked;
  if fOpenKey then
    edtOpenSectKey.Text := KeyToStr(m_nOpenKey)
  else
  begin
    m_nOpenKey := StrToKey(edtOpenSectKey.Text);
    edtOpenSectKey.Text := 'FFFFFFFFFFFF';
  end;
  rbOpenKeyB.Enabled := fOpenKey;
  UpdateControls();
end;

procedure TfmM3NetTest.btnStartStopClick(Sender: TObject);
var
  s: String;
begin
  m_fStarted := not m_fStarted;
  if m_fStarted then
  begin
    if (rgTestMode.ItemIndex = 1) and (edtSectN.Value = 0) and (edtCodeBlockN.Value = 0) then
    begin
      ShowMessage('Incorrent parameters: block #0 in sector #1 is read only.');
      m_fStarted := False;
      exit;
    end;
  end;
  if m_fStarted then s := 'Stop' else s := 'Start';
  btnStartStop.Caption := s;
  UpdateControls();
  labStatus.Caption := '';
  labStatus.Invalidate();
  if m_fStarted then
    DoInitCard();
end;

procedure TfmM3NetTest.FormCreate(Sender: TObject);
begin
  FRd := 0;
  m_fStarted := False;
  m_nOpenKey := z2mDefAuthKey;
end;

procedure TfmM3NetTest.FormShow(Sender: TObject);
begin
  UpdateControls();
end;

procedure TfmM3NetTest.rgTestModeClick(Sender: TObject);
begin
  UpdateControls();
end;

end.
