unit uRdControlDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, z2m, ExtCtrls;

type
  TfmRdControl = class(TForm)
    gridControl: TStringGrid;
    btnCancel: TButton;
    btnOk: TButton;
    Timer1: TTimer;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure gridControlDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure gridControlClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    m_fFlash1       : Boolean;
  public
    { Public declarations }
    FRd: THandle;
    FOutMask        : Integer;
    FOutBits        : Integer;
    FInputMask      : Integer;
  end;

var
  fmRdControl: TfmRdControl;

implementation

{$R *.dfm}

uses
  Math, uAppUtils;

procedure TfmRdControl.btnOkClick(Sender: TObject);
var
  nRet: Tz2mStatus;
begin
  ModalResult := mrNone;
  nRet := z2mOutputControl(FRd, FOutMask, FOutBits);
  if nRet <> Z2M_SUCCESS then
  begin
    MsgBoxZ2mErr(Handle, nRet);
    exit;
  end;
  ModalResult := mrOk;
end;

procedure TfmRdControl.FormCreate(Sender: TObject);
var
  i, cx: Integer;
begin
  with gridControl do
  begin
    Cells[0, 1] := 'ZP_I';
    Cells[0, 2] := 'TBUTT';
    Cells[0, 3] := 'KN.OTKR';
    Cells[0, 4] := 'L';
    Cells[0, 5] := 'DATCH';

    Cells[1, 0] := 'No change';
    Cells[2, 0] := '+5';
    Cells[3, 0] := 'GND';
    Cells[4, 0] := 'Input';

    Canvas.Font := Font;
    cx := 0;
    for i := 1 to 5 do
      cx := max(cx, Canvas.TextWidth(Cells[0, i]) + 8);
    ColWidths[0] := cx;
    for i := 1 to 4 do
      ColWidths[i] := Canvas.TextWidth(Cells[i, 0]) + 8;
  end;
  m_fFlash1 := False;
end;

procedure TfmRdControl.FormShow(Sender: TObject);
var
  nRet: Tz2mStatus;
begin
  nRet := z2mReadInput(FRd, @FInputMask);
  if nRet <> Z2M_SUCCESS then
  begin
    MsgBoxZ2mErr(Handle, nRet);
    exit;
  end;
  gridControl.Invalidate();
end;

procedure TfmRdControl.gridControlClick(Sender: TObject);
var
  grid: TStringGrid;
  i, y, k, nCol: Integer;
  pt: TPoint;
  rc: TRect;
begin
  grid := (Sender as TStringGrid);
  GetCursorPos(pt);
  pt := grid.ScreenToClient(pt);

  y := 0;
  if pt.Y < y then exit;
  rc := grid.CellRect(1, 0);
  if pt.X < rc.Left then exit;
  if pt.X < rc.Right then
    nCol := 1
  else
  begin
    rc := grid.CellRect(2, 0);
    if pt.X < rc.Right then
      nCol := 2
    else
    begin
      rc := grid.CellRect(3, 0);
      if pt.X < rc.Right then
        nCol := 3
      else
        exit;
    end;
  end;

  k := -1;
  for i := 1 to grid.RowCount - 1 do
  begin
    rc := grid.CellRect(nCol, i);
    if PtInRect(rc, pt) then
    begin
      k := i;
      break;
    end;
  end;
  if k <> -1 then
  begin
    case nCol of
      1: SetBit(Cardinal(FOutMask), k, not IsBit(Cardinal(FOutMask), k));
      2:
      begin
        SetBit(Cardinal(FOutBits), k);
        SetBit(Cardinal(FOutMask), k);
      end;
      3:
      begin
        SetBit(Cardinal(FOutBits), k, False);
        SetBit(Cardinal(FOutMask), k);
      end;
    end;
    gridControl.Invalidate();
  end;
end;

procedure TfmRdControl.gridControlDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  grid: TStringGrid;
  n: Integer;
  fCheck: Boolean;
  rc: TRect;
begin
  grid := (Sender as TStringGrid);
  with grid.Canvas do
  begin
    Font := grid.Font;
    Pen.Color := clBlack;

    if (ACol > 0) and (ARow > 0) then
    begin
      if ACol = 4 then
        n := DFCS_BUTTONCHECK or DFCS_INACTIVE
      else
        n := DFCS_BUTTONRADIO;
      case ACol of
        1: fCheck := not IsBit(FOutMask, ARow);
        2: fCheck := IsBit(FOutMask, ARow) and IsBit(FOutBits, ARow);
        3: fCheck := IsBit(FOutMask, ARow) and (not IsBit(FOutBits, ARow));
        4: fCheck := IsBit(FInputMask, ARow);
        else fCheck := False;
      end;
      if fCheck then n := n or DFCS_CHECKED;
      rc.Left := Rect.Left + ((Rect.Right - Rect.Left) - 16) div 2;
      rc.Top := Rect.Top + ((Rect.Bottom - Rect.Top) - 16) div 2;
      rc.Right := rc.Left + 16;
      rc.Bottom := rc.Top + 16;
      DrawFrameControl(Handle, rc, DFC_BUTTON, n);
    end;
  end;
end;

procedure TfmRdControl.Timer1Timer(Sender: TObject);
var
  nRet: Tz2mStatus;
begin
  try
    nRet := z2mReadInput(FRd, @FInputMask);
    if nRet <> Z2M_SUCCESS then exit;
    gridControl.Invalidate();
    m_fFlash1 := not m_fFlash1;
    if m_fFlash1 then
      Label1.Caption := '[ *]'
    else
      Label1.Caption := '[* ]';
  except
  end;
end;

end.
