unit uUL;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, ExtCtrls, CheckLst, ComCtrls, z2m;

Const
  WM_POSTINITFORM = WM_USER+1;
  WM_ADDCARD      = WM_USER+2;
  WM_RMVCARD      = WM_USER+3;

type
  TFmtEditMode = (femBin, femDec, femHex);
  TCdCellType = (cctFrame, cctSN, cctOtpRW, cctOtpR, cctDataRW, cctDataR,
    cctTextRW, cctTextR);
  TTxtFmtMode = (tfmOem, tfmAnsi);
  TULData = array[0..15, 0..3] of BYTE;

  TfmUL = class(TForm)
    gridData: TStringGrid;
    Panel1: TPanel;
    btnApply: TButton;
    btnRefresh: TButton;
    shLegSN: TShape;
    Label1: TLabel;
    shLegOTP: TShape;
    Label2: TLabel;
    shLegRWData: TShape;
    Label3: TLabel;
    shLegRData: TShape;
    Label4: TLabel;
    Panel2: TPanel;
    Label5: TLabel;
    edtCardName: TEdit;
    Label6: TLabel;
    edtConStat: TEdit;
    pbTransfer: TProgressBar;
    rgTxtFmt: TRadioGroup;
    rgDgtFmt: TRadioGroup;
    procedure FormDestroy(Sender: TObject);
    procedure gridDataClick(Sender: TObject);
    procedure rgTxtFmtClick(Sender: TObject);
    procedure rgDgtFmtClick(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnRefreshClick(Sender: TObject);
    procedure gridDataKeyPress(Sender: TObject; var Key: Char);
    procedure gridDataSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormShow(Sender: TObject);
    procedure gridDataDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure PostInitForm(var msg: TMessage); message WM_POSTINITFORM;
    procedure AddCardEvent(var msg: TMessage); message WM_ADDCARD;
    procedure RmvCardEvent(var msg: TMessage); message WM_RMVCARD;

    procedure DoOpenCard();
    procedure DoCloseCard();
    procedure DoRefresh();
    procedure DoApply();
    procedure SetEditMode(AMode: TFmtEditMode; AUpdControls: Boolean=True; AUpdate: Boolean=False);
    procedure SetTextFmtMode(AMode: TTxtFmtMode; AUpdControls: Boolean=True; AUpdate: Boolean=False);
    procedure DoneCellEdit(ACancel: Boolean);
    procedure UpdateCellFmt(Const ACol, ARow: Integer);
    procedure UpdateCellText(Const ARow: Integer);
    procedure SetCellFmt(Const ACol, ARow: Integer; Const ABt: BYTE);
    procedure SetModifyFlag(AModify: Boolean);
    procedure UpdateCardInfo();
    procedure UpdateButtons();
    procedure NextEditCell();
    function GetCellTip(Const ACol, ARow: Integer; var AData: TULData): TCdCellType;
    function GetOldCellTip(Const ACol, ARow: Integer): TCdCellType;
    function GetNewCellTip(Const ACol, ARow: Integer): TCdCellType;
    procedure CheckRow(ARow: Integer; ACheck: Boolean);
    function IsRowChecked(ARow: Integer): Boolean;
    function IsRowEnabled(ARow: Integer): Boolean;
    function IsValidCellFmtVal(Const ACol, ARow: Integer; ABt: BYTE): Boolean;
  public
    { Public declarations }
    FRd             : THANDLE;
    m_hCd           : THANDLE;
    FCdTip          : Tz2mCardType;
    FCdNum          : Tz2mCardNum;
    m_fDataInit     : Boolean;
    m_aNewData      : TULData;
    m_aOldData      : TULData;
    m_fChanged      : Boolean;
    m_fAfterDis     : Boolean;
    m_fAfterDisShow : Boolean;

    m_nFmtEditMode  : TFmtEditMode;
    m_nCurCol       : Integer;
    m_nCurRow       : Integer;
    m_fCellEdit     : Boolean;
    m_fCtrUpdEnable : Boolean;
    m_nCellSubN     : Integer;
    m_nTextFmtMode  : TTxtFmtMode;
  end;

var
  fmUL: TfmUL;

implementation

uses
  uAppUtils,
  uAppConst;

{$R *.dfm}

procedure TfmUL.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  // Init data
  m_hCd := 0;
  m_fDataInit := False;
  m_nCurCol := -1;
  m_nCurRow := -1;
  m_fCtrUpdEnable := True;
  m_nFmtEditMode := femHex;
  m_nTextFmtMode := tfmAnsi;
  m_nCellSubN := 0;
  m_fCellEdit := False;
  FillChar(m_aNewData[0], sizeof(m_aNewData), 0);
  FillChar(m_aOldData[0], sizeof(m_aOldData), 0);
  m_fChanged := False;
  m_fAfterDisShow := False;

  // Init controls
  for i := 2 to 5 do gridData.Rows[0][i] := IntToStr(i-1);
  gridData.Rows[0][6] := SROW_UL_TEXT;
  for i := 1 to 16 do gridData.Cols[1][i] := IntToStr(i-1);
  gridData.ColWidths[0] := 110;
  gridData.ColWidths[1] := 30;
  gridData.Cols[0][1] := SCOL_UL_SN;
  gridData.Cols[0][2] := SCOL_UL_SN;
  gridData.Cols[0][3] := SCOL_UL_INTERNAL_LOCK;
  gridData.Cols[0][4] := SCOL_UL_OTP;
  for i := 5 to 16 do
    gridData.Cols[0][i] := SCOL_UL_DATARW;

  SetEditMode(femHex, True, True);
  SetTextFmtMode(tfmAnsi, True, True);
end;

procedure TfmUL.FormDestroy(Sender: TObject);
begin
  DoCloseCard();
end;

procedure TfmUL.FormShow(Sender: TObject);
begin
  m_fAfterDis := False;
  PostMessage(Handle, WM_POSTINITFORM, 0, 0);
  UpdateCardInfo();
end;

procedure TfmUL.PostInitForm(var msg: TMessage);
begin
  DoOpenCard();
  if m_hCd <> 0 then DoRefresh();
end;

procedure TfmUL.AddCardEvent(var msg: TMessage);
begin
  if m_hCd = 0 then DoOpenCard();
end;

procedure TfmUL.RmvCardEvent(var msg: TMessage);
begin
  m_fAfterDis := True;
  DoCloseCard();
end;

procedure TfmUL.DoOpenCard();
var
  fRefresh: Boolean;
  nRet: Integer;
begin
  nRet := z2mOpenCard(@m_hCd, FRd, FCdTip, FCdNum);
  if nRet <> 0 then m_hCd := 0;
  m_fDataInit := False;
  UpdateCardInfo();
  UpdateButtons();
  gridData.Enabled := (m_hCd <> 0) and m_fDataInit;
  if m_hCd <> 0 then
  begin
    if m_fAfterDis then
    begin
      if m_fAfterDisShow then exit;
      m_fAfterDisShow := True;
      try
        fRefresh := MessageBox(Handle, Pchar(SMSG_DATARESTORE+#0),
          PChar(''+#0), MB_YESNO or MB_ICONQUESTION) = IDYES;
      finally
        m_fAfterDisShow := False;
      end;
      if fRefresh then DoRefresh();
    end;
  end
  else
  begin
    MsgBoxZ2mErr(Handle, nRet);
  end;
end;

procedure TfmUL.DoCloseCard();
begin
  if m_hCd <> 0 then
  begin
    z2mCloseCard(m_hCd);
    m_hCd := 0;
  end;
  UpdateCardInfo();
  UpdateButtons();
  gridData.Enabled := (m_hCd <> 0) and m_fDataInit;
end;

procedure TfmUL.DoRefresh();
var
  Buf: array[0..15] of Byte;
  i, j, nRet: Integer;
  nPgN: Integer;
  fChg: Boolean;
begin
  if m_hCd=0 then exit;
  DoneCellEdit(True);
  fChg := False;
  pbTransfer.Visible := True;
  btnApply.Enabled := False;
  btnRefresh.Enabled := False;
  gridData.Enabled := False;
  try
    pbTransfer.Position := 0;
    nPgN := 0;
    while nPgN < 15 do
    begin
      nRet := z2mReadULCard4Page(m_hCd, nPgN, @Buf[0]);
      if nRet = Z2M_SUCCESS then
      begin
        Application.ProcessMessages;
        for i := 0 to 3 do
        begin
          for j := 0 to 3 do
          begin
            m_aNewData[nPgN+i, j] := Buf[i*4+j];
            SetCellFmt(j+2, nPgN+i+1, Buf[i*4+j]);
          end;
            if not CompareMem(@m_aNewData[nPgN+i,0], @m_aOldData[nPgN+i,0], 4) then
            begin
              Move(m_aNewData[nPgN+i,0], m_aOldData[nPgN+i,0], 4);
              fChg := True;
            end;
          UpdateCellText(nPgN+i+1);
        end;
      end
      else
      begin
        MsgBoxZ2mErr(Handle, nRet);
        m_fDataInit := False;
        exit;
      end;
      pbTransfer.StepBy(4);
      Inc(nPgN,4);
    end;
    SetModifyFlag(False);
    m_fDataInit := True;
    if m_nCurCol = -1 then
      NextEditCell();
  finally
    pbTransfer.Visible := False;
    UpdateButtons();
    gridData.Enabled := m_fDataInit;
    if gridData.Enabled then
      gridData.SetFocus;
    gridData.Invalidate;
  end;
  if m_fAfterDis then
  begin
    if fChg then MessageBox(Handle, PChar(SMSG_DATACHANGED+#0), PChar(''+#0), MB_OK or MB_ICONWARNING);
    m_fAfterDis := False;
  end;
end;

procedure TfmUL.DoApply();
var
  Buf: array[0..3]of BYTE;
  nPgN, nRet, nMax: Integer;
  s: string;
begin
  if (not m_fDataInit) or (not m_fChanged) then exit;
  DoneCellEdit(False);
  pbTransfer.Visible := True;
  try
    pbTransfer.Position := 0;
    pbTransfer.Step := 1;
    nPgN := 15;
    while nPgN >= 0 do
    begin
      if not CompareMem(@m_aNewData[nPgN,0], @m_aOldData[nPgN,0], 4) then
      begin
        Move(m_aNewData[nPgN,0], Buf[0], 4);
        nRet := z2mWriteULCardPage(m_hCd, nPgN, @Buf[0]);
        if nRet = Z2M_SUCCESS then
        begin
          Move(m_aNewData[nPgN,0], m_aOldData[nPgN,0], 4);
        end
        else
        begin
          nMax := 255;
          SetLength(s, nMax);
          z2mGetErrorText(nRet, PChar(s), nMax);
          MessageDlg(format(SMSG_WRITEPAGEERR, [nPgN+1, s]), mtError, [mbOk], 0);
          exit;
        end;
      end;
      pbTransfer.StepIt;
      Dec(nPgN);
    end;
  finally
    pbTransfer.Visible := False;
  end;
  SetModifyFlag(False)
end;

procedure TfmUL.rgDgtFmtClick(Sender: TObject);
var
  nMode: TFmtEditMode;
begin
  if rgDgtFmt.Buttons[0].Checked then
    nMode := femBin
  else if rgDgtFmt.Buttons[1].Checked then
    nMode := femDec
  else
    nMode := femHex;
  SetEditMode(nMode, False);
  if IsWindowVisible(gridData.Handle) and IsWindowEnabled(gridData.Handle) then
    gridData.SetFocus;
end;

procedure TfmUL.rgTxtFmtClick(Sender: TObject);
var
  tfm: TTxtFmtMode;
begin
  if rgTxtFmt.Buttons[0].Checked then
    tfm := tfmOem
  else
    tfm := tfmAnsi;
  SetTextFmtMode(tfm, False);
end;

procedure TfmUL.SetEditMode(AMode: TFmtEditMode; AUpdControls: Boolean; AUpdate: Boolean);
var
  i, j: Integer;
  bt: BYTE;
begin
  if (AMode = m_nFmtEditMode) and (not AUpdate) then exit;
  DoneCellEdit(False);
  m_nFmtEditMode := AMode;
  for i := 1 to 16 do
  begin
    for j := 2 to 5 do
    begin
      bt := m_aNewData[i-1, j-2];
      SetCellFmt(j, i, bt);
    end;
  end;
  m_fCellEdit := False;
  if AUpdControls then
  begin
    case m_nFmtEditMode of
      femBin: rgDgtFmt.Buttons[0].Checked := True;
      femDec: rgDgtFmt.Buttons[1].Checked := True;
      femHex: rgDgtFmt.Buttons[2].Checked := True;
    end;
  end;
end;

procedure TfmUL.SetTextFmtMode(AMode: TTxtFmtMode; AUpdControls: Boolean; AUpdate: Boolean);
var
  i: Integer;
begin
  if (AMode = m_nTextFmtMode) and (not AUpdate) then exit;
  DoneCellEdit(False);
  m_nTextFmtMode := AMode;
  for i := 5 to 16 do
    UpdateCellText(i);
  if AUpdControls then
  begin
    case m_nTextFmtMode of
      tfmOem: rgTxtFmt.Buttons[0].Checked := True;
      tfmAnsi: rgTxtFmt.Buttons[1].Checked := True;
    end;
  end;
end;

procedure TfmUL.gridDataClick(Sender: TObject);
var
  grid: TStringGrid;
  i, y, k: Integer;
  pt: TPoint;
  rc: TRect;
begin
  grid := (Sender as TStringGrid);
  GetCursorPos(pt);
  pt := grid.ScreenToClient(pt);
  if pt.X > (grid.ColWidths[0]) then exit;

  y := 0;
  if pt.Y < y then exit;

  k := -1;
  for i := 0 to grid.RowCount - 1 do
  begin
    rc := grid.CellRect(0, i);
    if PtInRect(rc, pt) then
    begin
      k := i;
      break;
    end;
  end;
  if k <> -1 then
  begin
    if IsRowEnabled(k) then CheckRow(k, not IsRowChecked(k));
  end;
end;

procedure TfmUL.gridDataDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  grid: TStringGrid;
  dx, dy, a,b,n,i: integer;
  s,ss: string;
  bkcl, tc: TColor;
  rc: TRect;
  cctOld, cctNew: TCdCellType;
  bt1, bt2: BYTE;
  sty: TFontStyles;
begin
  grid := (Sender as TStringGrid);
  with grid.Canvas do
  begin
    Font := grid.Font;
    Pen.Color := clBlack;
    cctOld := GetOldCellTip(ACol, ARow);
    cctNew := GetNewCellTip(ACol, ARow);
    case cctOld of
      cctFrame:
      begin
      {   }
        Brush.Color := grid.FixedColor;
        FillRect(Rect);
        Font.Color := clBlue;
        s := grid.Cells[ACol, ARow];
        if (ACol = 0) and (ARow >= 4) then
        begin
          a := Rect.Left + 16;
          rc := Rect; rc.Right := rc.Left+16; rc.Bottom := rc.Top+16;
          n := 0;
          if IsRowChecked(ARow) then begin n := n or DFCS_CHECKED; s := SROW_UL_DATALOCKED end;
          if not IsRowEnabled(ARow) then n := n or DFCS_INACTIVE;
          DrawFrameControl(Handle, rc, DFC_BUTTON, DFCS_BUTTONCHECK or n);
        end
        else
          a := Rect.Left;
        dx := TextWidth(s) + 2;
        dy := TextHeight(s) + 2;

        TextOut(
          (a+Rect.Right-dx)div 2,
          (Rect.Top+Rect.Bottom-dy)div 2,
          s);
      end;
      cctSN..cctTextR:
      begin
        rc := Rect;
        bkcl := grid.Color;
        sty := Font.Style;
        tc := Font.Color;
        case cctOld of
          cctSN: bkcl := shLegSN.Brush.Color;
          cctOtpRW: bkcl := shLegOTP.Brush.Color;
          cctOtpR: bkcl := clOlive;
          cctDataRW: bkcl := shLegRWData.Brush.Color;
          cctDataR:
          begin
            bkcl := shLegRData.Brush.Color;
            sty := [fsBold];
            tc := clWhite;
          end;
          cctTextR: bkcl := clDkGray;
          cctTextRW: bkcl := clLtGray;
        end;
        Font.Style := sty;
        Font.Color := tc;
        if cctOld in [cctOtpRW,cctDataRW,cctTextRW] then
        begin
          if cctNew in [cctOtpR,cctDataR,cctTextR] then
          begin
            bkcl := TColor($009292F5);
          end;
        end;
        Brush.Color := bkcl;
        Brush.Style := bsSolid;
        FillRect(rc);

        s := grid.Cells[ACol, ARow];
        dx := TextWidth(s) + 2;
        dy := TextHeight(s) + 2;

        if (ACol = m_nCurCol) and (ARow = m_nCurRow) then
        begin
          Pen.Width := 3;
          rc := Rect;
          Brush.Style := bsClear;
          Rectangle(rc);
        end;

        SetBkMode(Handle, TRANSPARENT);
        TextOut(
          (Rect.Left+Rect.Right-dx)div 2,
          (Rect.Top+Rect.Bottom-dy)div 2,
          s);

        if ((ARow = 3) and (ACol >= 4) and (ACol <= 5))
          or (ARow = 4) then
        begin
          bt1 := 0;
          bt2 := 0;
          if ARow = 3 then
          begin
            bt1 := m_aOldData[2, ACol-2];
            bt2 := m_aNewData[2, ACol-2];
          end
          else
          begin
            if (ACol > 2) and (ACol <= 5) then
            begin
            bt1 := m_aOldData[3, ACol-2];
            bt2 := m_aNewData[3, ACol-2];
          end;
          end;
          Font.Color := clRed;
          for i := 1 to 8 do
          begin
              if IsBit(bt1, 9-i) <> IsBit(bt2, 9-i) then
              begin
                ss := Copy(s, 1, i-1);
                if length(ss) = 0 then a := 0 else a := TextWidth(ss);
                TextOut(
                  (Rect.Left+Rect.Right-dx)div 2+a,
                  (Rect.Top+Rect.Bottom-dy)div 2,
                  s[i]);
              end;
            end;
        end;
        if (ACol = m_nCurCol) and (ARow = m_nCurRow)
          and m_fCellEdit and (m_nCellSubN > 0) then
        begin
          a := TextWidth(Copy(s, 1, m_nCellSubN-1));
          b := TextWidth(Copy(s, m_nCellSubN, 1));
          Font.Color := clBlue;
          rc.Left := (Rect.Left+Rect.Right-dx)div 2+a;
          rc.Right := rc.Left+b;
          rc.Top := (Rect.Top+Rect.Bottom-dy)div 2+dy;
          rc.Bottom := rc.Top;
          MoveTo(rc.Left, rc.Top);
          LineTo(rc.Right, rc.Bottom);
        end;
      end;
    end;
  end
end;

procedure TfmUL.gridDataKeyPress(Sender: TObject; var Key: Char);
var
  grid: TStringGrid;
  s,s2: string;
  n1: Integer;
  b: Integer;
begin
  if (m_nCurCol <= 1) or (m_nCurRow <= 0) then exit;
  grid := (Sender as TStringGrid);
  case Key of
    #27: begin DoneCellEdit(True); exit end;
    #13: begin DoneCellEdit(False); NextEditCell(); exit end;
  end;

  if m_nCurCol = 6 then
  begin
    s := grid.Cells[m_nCurCol, m_nCurRow];
    if IsCharAlphaNumeric(Key) or ((Key = #32) and (GetKeyState(VK_CONTROL) and $80 = 0)) then
    begin
      if (not m_fCellEdit) or (Length(s) >= 4) then
        s := '';
      s2 := Key;
      if m_nTextFmtMode = tfmOem then
        s2 := WinToDos(s2);
      if not IsValidCellFmtVal(Length(s)+2, m_nCurRow, Ord(s2[1])) then exit;
      s := s + Key;
      m_nCellSubN := Length(s);
      grid.Cells[m_nCurCol, m_nCurRow] := s;
      SetCellFmt(Length(s)+1, m_nCurRow, Ord(s2[1]));
      m_fCellEdit := True;
    end
    else
    begin
      case Key of
        #32: if GetKeyState(VK_CONTROL) and $80 <> 0 then
        begin
          if (not m_fCellEdit) or (Length(s) >= 4) then
            s := '';
          s2 := Char(m_aNewData[m_nCurRow-1, Length(s)]);
          if m_nTextFmtMode = tfmOem then
            s2 := DosToWin(s2);
          s := s + s2;
          m_nCellSubN := Length(s);
          grid.Cells[m_nCurCol, m_nCurRow] := s;
          m_fCellEdit := True;
        end;
        #8:
        begin 
          if (not m_fCellEdit) then
            s := '';
          Delete(s, Length(s), 1);
          m_nCellSubN := Length(s);
          grid.Cells[m_nCurCol, m_nCurRow] := s;
          UpdateCellFmt(Length(s)+2, m_nCurRow);
          m_fCellEdit := True;
        end;
      end;
    end;
    if m_fCellEdit and (Length(s) >= 4) then
    begin
      DoneCellEdit(False);
      NextEditCell();
    end;
  end
    else if m_nCurRow in [3..4] then
    begin
        s := grid.Cells[m_nCurCol, m_nCurRow];
        case Key of
          '0','1':
          begin
            if (not m_fCellEdit) then
              m_nCellSubN := 1;
            s[m_nCellSubN] := Key;
            b := 0;
            for n1 := 1 to Length(s) do
              if s[n1] = '1' then SetBit(Cardinal(b), 9-n1);
            if not IsValidCellFmtVal(m_nCurCol, m_nCurRow, b) then exit;
            Inc(m_nCellSubN);
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
          #8:
          begin
            if (not m_fCellEdit) or (m_nCellSubN <= 0) then exit;
            if m_nCellSubN > 1 then
              Dec(m_nCellSubN);
            if IsBit(m_aNewData[m_nCurRow-1, m_nCurCol-2], 9-m_nCellSubN) then
              s[m_nCellSubN] := '1'
            else
              s[m_nCellSubN] := '0';
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
          #32: if GetKeyState(VK_CONTROL) and $80 <> 0 then
          begin
            if (not m_fCellEdit) then
              m_nCellSubN := 1;
            if IsBit(m_aNewData[m_nCurRow-1, m_nCurCol-2], 9-m_nCellSubN) then
              s[m_nCellSubN] := '1'
            else
              s[m_nCellSubN] := '0';
            Inc(m_nCellSubN);
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
        end;
        if m_fCellEdit and (m_nCellSubN > 8) then
        begin
          DoneCellEdit(False);
          NextEditCell();
        end;
    end
  else
  begin
    case m_nFmtEditMode of
      femBin:
      begin
        s := grid.Cells[m_nCurCol, m_nCurRow];
        case Key of
          '0','1':
          begin
            if (not m_fCellEdit) then
              m_nCellSubN := 1;
            s[m_nCellSubN] := Key;
            b := 0;
            for n1 := 1 to Length(s) do
              if s[n1] = '1' then SetBit(Cardinal(b), 9-n1);
            if not IsValidCellFmtVal(m_nCurCol, m_nCurRow, b) then exit;
            Inc(m_nCellSubN);
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
          #8:
          begin
            if (not m_fCellEdit) or (m_nCellSubN <= 0) then exit;
            if m_nCellSubN > 1 then
              Dec(m_nCellSubN);
            if IsBit(m_aNewData[m_nCurRow-1, m_nCurCol-2], 9-m_nCellSubN) then
              s[m_nCellSubN] := '1'
            else
              s[m_nCellSubN] := '0';
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
          #32: if GetKeyState(VK_CONTROL) and $80 <> 0 then
          begin
            if (not m_fCellEdit) then
              m_nCellSubN := 1;
            if IsBit(m_aNewData[m_nCurRow-1, m_nCurCol-2], 9-m_nCellSubN) then
              s[m_nCellSubN] := '1'
            else
              s[m_nCellSubN] := '0';
            Inc(m_nCellSubN);
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
        end;
        if m_fCellEdit and (m_nCellSubN > 8) then
        begin
          DoneCellEdit(False);
          NextEditCell();
        end;
      end;
      femHex:
      begin
        s := grid.Cells[m_nCurCol, m_nCurRow];
        case Key of
          '0'..'9','a'..'f','A'..'F':
          begin
            if (not m_fCellEdit) then
              m_nCellSubN := 1;
            if CharInSet(Key, ['a'..'f']) then
              s[m_nCellSubN] := Chr(Ord(Key)+Ord('A')-Ord('a'))
            else
              s[m_nCellSubN] := Key;
            b := HexToInt(s);
            if not IsValidCellFmtVal(m_nCurCol, m_nCurRow, b) then exit;
            Inc(m_nCellSubN);
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
          #8:
          begin
            if (not m_fCellEdit) or (m_nCellSubN <= 0) then exit;
            if m_nCellSubN > 1 then
              Dec(m_nCellSubN);
            s2 := format('%.2X', [m_aNewData[m_nCurRow-1, m_nCurCol-2]]);
            s[m_nCellSubN] := s2[m_nCellSubN];
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
          #32: if GetKeyState(VK_CONTROL) and $80 <> 0 then
          begin
            if (not m_fCellEdit) then
              m_nCellSubN := 1;
            s2 := format('%.2X', [m_aNewData[m_nCurRow-1, m_nCurCol-2]]);
            s[m_nCellSubN] := s2[m_nCellSubN];
            Inc(m_nCellSubN);
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
        end;
        if m_fCellEdit and (m_nCellSubN > 2) then
        begin
          DoneCellEdit(False);
          NextEditCell();
        end;
      end;
      femDec:
      begin
        s := grid.Cells[m_nCurCol, m_nCurRow];
        case Key of
          '0'..'9':
          begin
            if (not m_fCellEdit) then
              m_nCellSubN := 1;
            n1 := Ord(Key)-Ord('0');
            case m_nCellSubN of
              1: if n1 > 2 then exit else if (n1 = 2) and ((Ord(s[2])-Ord('0')) >= 5) then
                begin
                  s[2] := '5';
                  if (Ord(s[3])-Ord('0')) > 5 then
                    s[3] := '5';
                end;
              2: if ((Ord(s[1])-Ord('0')) = 2) and (n1 > 5) then exit;
              3: if ((Ord(s[1])-Ord('0')) = 2) and ((Ord(s[2])-Ord('0')) = 5)
                and (n1 > 5) then exit;
            end;
            b := StrToInt(s);
            if not IsValidCellFmtVal(m_nCurCol, m_nCurRow, b) then exit;
            s[m_nCellSubN] := Key;
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            Inc(m_nCellSubN);
            m_fCellEdit := True;
          end;
          #8:
          begin
            if (not m_fCellEdit) or (m_nCellSubN <= 0) then exit;
            if m_nCellSubN > 1 then
              Dec(m_nCellSubN);
            s2 := format('%.3d', [m_aNewData[m_nCurRow-1, m_nCurCol-2]]);
            n1 := Ord(s2[m_nCellSubN])-Ord('0');
            case m_nCellSubN of
              3:
              begin
                if ((Ord(s[1])-Ord('0')) = 2) and ((Ord(s[1])-Ord('0')) = 5) then
                  if n1 > 5 then n1 := 5;
              end;
              2:
              begin
                if (Ord(s[1])-Ord('0')) = 2 then if n1 > 5 then n1 := 5;
              end;
            end;
            s[m_nCellSubN] := Char(n1 + Ord('0'));
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
          #32: if GetKeyState(VK_CONTROL) and $80 <> 0 then
          begin
            if (not m_fCellEdit) then
              m_nCellSubN := 1;
            s2 := format('%.3d', [m_aNewData[m_nCurRow-1, m_nCurCol-2]]);
            n1 := Ord(s2[m_nCellSubN])-Ord('0');
            case m_nCellSubN of
              3:
              begin
                if ((Ord(s[1])-Ord('0')) = 2) and ((Ord(s[1])-Ord('0')) = 5) then
                  if n1 > 5 then n1 := 5;
              end;
              2:
              begin
                if (Ord(s[1])-Ord('0')) = 2 then if n1 > 5 then n1 := 5;
              end;
            end;
            s[m_nCellSubN] := Char(n1 + Ord('0'));
            Inc(m_nCellSubN);
            grid.Cells[m_nCurCol, m_nCurRow] := s;
            m_fCellEdit := True;
          end;
        end;
        if m_fCellEdit and (m_nCellSubN > 3) then
        begin
          DoneCellEdit(False);
          NextEditCell();
        end;
      end;
    end;
  end;
end;

procedure TfmUL.gridDataSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  if m_fCtrUpdEnable then
  begin
    CanSelect := (ACol >= 2) and (ARow >= 1);
    if CanSelect and ((m_nCurCol <> ACol) or (m_nCurRow <> ARow)) then
    begin
      if not (GetOldCellTip(ACol, ARow) in [cctOtpRW,cctDataRW,cctTextRW]) then
        begin CanSelect := False; exit end;
      DoneCellEdit(False);
      m_nCurCol := ACol;
      m_nCurRow := ARow;
      gridData.Invalidate;
    end
  end
  else
    CanSelect := True;
end;

procedure TfmUL.DoneCellEdit(ACancel: Boolean);
var
  s, s2: string;
  bt: Integer;
  k: Integer;
begin
  if not m_fCellEdit then exit;
  if m_nCurCol = 6 then
  begin
    if ACancel then
    begin
      UpdateCellText(m_nCurRow);
    end
    else
    begin
      s := gridData.Cells[m_nCurCol, m_nCurRow];
      for k := 1 to Length(s) do
      begin
        s2 := s[k];
        if m_nTextFmtMode = tfmOem then
          s2 := WinToDos(s2);
        bt := Ord(s2[1]);
        m_aNewData[m_nCurRow-1, k-1] := bt;
        if m_aOldData[m_nCurRow-1, k-1] <> m_aNewData[m_nCurRow-1, k-1] then
          SetModifyFlag(True);
        UpdateCellFmt(k+1, m_nCurRow);
      end;
    end;
  end
  else if m_nCurRow in [3..4] then
  begin
    if ACancel then
    begin
      UpdateCellFmt(m_nCurCol, m_nCurRow);
    end
    else
    begin
      s := gridData.Cells[m_nCurCol, m_nCurRow];
      bt := 0;
          for k := 1 to Length(s) do
            if s[k] = '1' then SetBit(Cardinal(bt), 9-k);
      m_aNewData[m_nCurRow-1, m_nCurCol-2] := bt;
      if m_aOldData[m_nCurRow-1, m_nCurCol-2] <> m_aNewData[m_nCurRow-1, m_nCurCol-2] then
        SetModifyFlag(True);
      UpdateCellText(m_nCurRow);
    end;
  end
  else
  begin
    if ACancel then
    begin
      UpdateCellFmt(m_nCurCol, m_nCurRow);
    end
    else
    begin
      s := gridData.Cells[m_nCurCol, m_nCurRow];
      bt := 0;
      case m_nFmtEditMode of
        femBin:
        begin
          for k := 1 to Length(s) do
            if s[k] = '1' then SetBit(Cardinal(bt), 9-k);
        end;
        femDec: if Length(s)>0 then bt := StrToInt(s);
        femHex: if Length(s)>0 then bt := HexToInt(s);
      end;
      m_aNewData[m_nCurRow-1, m_nCurCol-2] := bt;
      if m_aOldData[m_nCurRow-1, m_nCurCol-2] <> m_aNewData[m_nCurRow-1, m_nCurCol-2] then
        SetModifyFlag(True);
      UpdateCellText(m_nCurRow);
    end;
  end;
  m_fCellEdit := False;
  m_nCellSubN := 0;
end;

procedure TfmUL.UpdateCellFmt(Const ACol, ARow: Integer);
begin
  SetCellFmt(ACol, ARow, m_aNewData[ARow-1, ACol-2]);
end;

procedure TfmUL.UpdateCellText(Const ARow: Integer);
var
  s: string;
  bt: BYTE;
  k: Integer;
begin
  s := '';
  if ARow >= 5 then
  begin
  for k := 0 to 3 do
  begin
    bt := m_aNewData[ARow-1, k];
    s := s+Char(bt);
  end;
  if m_nTextFmtMode = tfmOem then
    s := DosToWin(s);
  end;
  gridData.Cells[6, ARow] := s;
end;

procedure TfmUL.SetCellFmt(Const ACol, ARow: Integer; Const ABt: BYTE);
var
  s: string;
  k: Integer;
begin
  s := '';
  case ARow of
    1..2: s := format('%.2X', [ABt]);
    3: if ACol <= 3 then s := format('%.2X', [ABt]) else
    begin
      for k := 8 downto 1 do
        if IsBit(ABt, k) then s := s + '1' else s := s + '0';
    end;
    4:
    begin
      for k := 8 downto 1 do
        if IsBit(ABt, k) then s := s + '1' else s := s + '0';
    end;
    else
    begin
  case m_nFmtEditMode of
    femBin:
    begin
      for k := 8 downto 1 do
        if IsBit(ABt, k) then s := s + '1' else s := s + '0';
    end;
    femDec:
      s := format('%.3d', [ABt]);
    femHex:
      s := format('%.2X', [ABt]);
  end;
    end;
  end;
  gridData.Cells[ACol, ARow] := s;
end;

procedure TfmUL.SetModifyFlag(AModify: Boolean);
begin
  if m_fChanged = AModify then exit;
  m_fChanged := AModify;
  UpdateCardInfo();
  UpdateButtons();
end;

procedure TfmUL.UpdateCardInfo();
var
  s: string;
begin
  if m_fChanged then s := '*' else s := '';
  edtCardName.Text := format('%s  %s  %s',
    [CdIdToStr(FCdNum), CdTipToShortStr(FCdTip), s]);
  if m_hCd <> 0 then edtConStat.Text := SMSG_CONNECTED
  else edtConStat.Text := SMSG_DICONNECTED;
end;

procedure TfmUL.UpdateButtons();
begin
  btnApply.Enabled := (m_hCd <> 0) and m_fChanged;
  btnRefresh.Enabled := (m_hCd <> 0);
end;

procedure TfmUL.NextEditCell();
var
  i, j, i0, j0: Integer;
  fTabOk: Boolean;
begin
  DoneCellEdit(False);
  fTabOk := False;
  i := m_nCurRow;

  if m_nCurCol = 6 then
  begin
    j := m_nCurCol;
    if m_nCurRow = -1 then i0 := 1 else i0 := m_nCurRow;
    for i := i0+1 to 16 do
    begin
      if GetOldCellTip(m_nCurCol, i) in [cctOtpRW,cctDataRW,cctTextRW] then
        begin fTabOk := True; break; end;
    end;
    if not fTabOk then
    begin
      for i := 1 to i0-1 do
      begin
        if GetOldCellTip(m_nCurCol, i) in [cctOtpRW,cctDataRW,cctTextRW] then
          begin fTabOk := True; break; end;
      end;
    end;
  end
  else
  begin
    if m_nCurCol = -1 then j0 := 1 else j0 := m_nCurCol;
    if m_nCurRow = -1 then i0 := 1 else i0 := m_nCurRow;

    for j := j0+1 to 5 do
    begin
      if GetOldCellTip(j, i0) in [cctOtpRW,cctDataRW,cctTextRW] then
        begin fTabOk := True; break; end;
    end;
    if not fTabOk then
    begin
      for i := i0+1 to 16 do
      begin
        for j := 2 to 5 do
        begin
          if GetOldCellTip(j, i) in [cctOtpRW,cctDataRW,cctTextRW] then
            begin fTabOk := True; break; end;
        end;
        if fTabOk then break;
      end;
    end;
    if not fTabOk then
    begin
      for i := 1 to i0-1 do
      begin
        for j := 2 to 5 do
        begin
          if GetOldCellTip(j, i) in [cctOtpRW,cctDataRW,cctTextRW] then
            begin fTabOk := True; break; end;
        end;
        if fTabOk then break;
      end;
    end;
    if not fTabOk then
    begin
      for j := 2 to j0-1 do
      begin
        if GetOldCellTip(j, i0) in [cctOtpRW,cctDataRW,cctTextRW] then
          begin fTabOk := True; break; end;
      end;
    end;
  end;
  if fTabOk then
  begin
    m_nCurCol := j;
    m_nCurRow := i;
    m_fCtrUpdEnable := False;
    try
      gridData.Col := m_nCurCol;
      gridData.Row := m_nCurRow;
    finally
      m_fCtrUpdEnable := True;
    end;
    gridData.Invalidate;
  end;
end;

function TfmUL.GetCellTip(Const ACol, ARow: Integer; var AData: TULData): TCdCellType;
begin
  Result := cctFrame;
  if (ACol <= 1) or (ACol >= 7) or (ARow <= 0) or (ARow >= 17) then exit;

  case ARow of
    1..2: if ACol = 6 then Result := cctTextR else Result := cctSN;
    3:
      case ACol of
        2..3: Result := cctSN;
        4:
        begin
          if IsBit(AData[2, 2], 1) and IsBit(AData[2, 2], 2)
           and IsBit(AData[2, 2], 3) then
          Result := cctOtpR else Result := cctOtpRW;
        end;
        5:
        begin
          if IsBit(AData[2, 2], 2) and IsBit(AData[2, 2], 3) then
            Result := cctOtpR else Result := cctOtpRW;
        end;
        6: Result := cctTextR;
      end;
    4: if ACol = 6 then Result := cctTextR else
      begin
        if IsBit(AData[2, 2], 4) then Result := cctOtpR else Result := cctOtpRW;
      end;
    5..8:
      if IsBit(AData[2,2], ARow) then
      begin
        if ACol = 6 then Result := cctTextR else Result := cctDataR;
      end
      else
      begin
        if ACol = 6 then Result := cctTextRW else Result := cctDataRW;
      end;
    9..16:
      if IsBit(AData[2,3], ARow-8) then
      begin
        if ACol = 6 then Result := cctTextR else Result := cctDataR;
      end
      else
      begin
        if ACol = 6 then Result := cctTextRW else Result := cctDataRW;
      end;
  end;
end;

function TfmUL.GetOldCellTip(Const ACol, ARow: Integer): TCdCellType;
begin
  Result := GetCellTip(ACol, ARow, m_aOldData);
end;

function TfmUL.GetNewCellTip(Const ACol, ARow: Integer): TCdCellType;
begin
  Result := GetCellTip(ACol, ARow, m_aNewData);
end;

procedure TfmUL.btnRefreshClick(Sender: TObject);
begin
  DoRefresh();
end;

procedure TfmUL.btnApplyClick(Sender: TObject);
begin
  DoApply();
end;

procedure TfmUL.CheckRow(ARow: Integer; ACheck: Boolean);
var
  fPaint: Boolean;
  b: Integer;
begin
  fPaint := False;
  case ARow of
    4..8:
    begin
      if IsBit(m_aNewData[2,2], ARow) <> ACheck then
      begin
        b := m_aNewData[2,2];
        SetBit(Cardinal(b), ARow, ACheck);
        m_aNewData[2,2] := b;
        if m_aOldData[2,2] <> m_aNewData[2,2] then
          SetModifyFlag(True);
        UpdateCellFmt(4, 3);
        fPaint := True;
      end;
    end;
    9..16:
    begin
      if IsBit(m_aNewData[2,3], ARow-8) <> ACheck then
      begin
        b := m_aNewData[2,3];
        SetBit(Cardinal(b), ARow-8, ACheck);
        m_aNewData[2,3] := b;
        if m_aOldData[2,3] <> m_aNewData[2,3] then
          SetModifyFlag(True);
        UpdateCellFmt(5, 3);
        fPaint := True;
      end;
    end;
  end;
  if fPaint then
    gridData.Invalidate;
end;

function TfmUL.IsRowChecked(ARow: Integer): Boolean;
begin
  Result := False;
  case ARow of
    4..8: if IsBit(m_aNewData[2,2], ARow) then Result := True;
    9..16: if IsBit(m_aNewData[2,3], ARow-8) then Result := True;
  end;
end;

function TfmUL.IsRowEnabled(ARow: Integer): Boolean;
begin
  Result := True;
  case ARow of
    4..8: if IsBit(m_aOldData[2,2], ARow) then Result := False;
    9..16: if IsBit(m_aOldData[2,3], ARow-8) then Result := False;
  end;
end;

function TfmUL.IsValidCellFmtVal(Const ACol, ARow: Integer; ABt: BYTE): Boolean;
var
  i: Integer;
  b: BYTE;
begin
  Result := False;
  if (ACol <= 1) or (ACol >= 7) or (ARow <= 2) or (ARow >= 17) then exit;
  b := m_aOldData[ARow-1, ACol-2];
  if b = ABt then begin Result := True; exit; end;

  case ARow of
    3:
    begin
      case ACol of
        4:
        begin
          Result:= True;
          for i := 1 to 8 do
            if IsBit(b, i) and (not IsBit(ABt, i)) then
              begin Result := False; break end;
          if Result then
          begin
            if IsBit(b, 1) and IsBit(ABt, 4) then
              Result := False
            else if IsBit(b, 2) and
              (IsBit(ABt, 5) or IsBit(ABt, 6) or IsBit(ABt, 7) or IsBit(ABt, 8)) then
              Result := False;
          end;
        end;
        5:
        begin
          Result:= True;
          for i := 1 to 8 do
            if IsBit(b, i) and (not IsBit(ABt, i)) then
              begin Result := False; break end;
          if Result then
          begin
            if IsBit(m_aOldData[2,2], 2) and
              (IsBit(ABt, 1) or IsBit(ABt, 2)) then
              Result := False
            else if IsBit(m_aOldData[2,2], 3) and
              (IsBit(ABt, 3) or IsBit(ABt, 4) or IsBit(ABt, 5) or
              IsBit(ABt, 6) or IsBit(ABt, 7) or IsBit(ABt, 8)) then
              Result := False;
          end;
        end;
      end;
    end;
    4:
    begin
      if not IsBit(m_aOldData[2, 2], 4) then
      begin
        Result:= True;
        for i := 1 to 8 do
          if IsBit(b, i) and (not IsBit(ABt, i)) then
            begin Result := False; break end;
      end;
    end;
    5..8:
    begin
      if not IsBit(m_aOldData[2,2], ARow) then Result := True;
    end;
    9..16:
    begin
      if not IsBit(m_aOldData[2,3], ARow-8) then Result := True;
    end;
  end;
end;

end.
