#ifndef __Z2M_H
#define __Z2M_H

#include "windows.h"
#pragma link "z2m.lib"


#pragma pack(1)   // turn byte alignment on
#pragma option push -b

// (rom) this is the switch to change between static and dynamic linking.
// (rom) it is enabled by default here.
// (rom) To disable simply comment line.
#define Z2M_LINKONREQUEST

#define Z2M_DLLNAME       "z2m.dll"
#define Z2M_MAX_PORTNAME	16
#define Z2M_MAX_MANUFACTOR	250

///////////////////////////////////////////////////////////
//  

typedef INT Tz2mStatus;	//    Z2M.DLL

#define Z2M_SUCCESS						0	//   
#define Z2M_E_UNKNOWN					1	//  
#define Z2M_E_INVALID_HANDLE			2	//  
#define Z2M_E_INVALID_PARAMETER			3	//  
#define Z2M_E_READER_NOT_FOUND			4	//   
#define Z2M_E_CARD_NOT_FOUND			5	//   
#define Z2M_E_READER_ACCESS				6	//    
#define Z2M_E_CARD_ACCESS				7	//    
#define Z2M_E_PAGE_LOCK					8	//  
#define Z2M_E_NO_MORE_ITEMS				9	//     
#define Z2M_E_INSUFFICIENT_BUFFER		10	//    
#define Z2M_E_DLL_NOT_INIT				11	// DLL  
#define Z2M_E_PENDING					12	//    
#define Z2M_E_BREAK						13	//   
#define Z2M_E_UNSUPPORTED				14	//   
#define Z2M_E_AUTHORIZE					15	//     1K/4K
#define Z2M_E_READER_ERROR				16	//   
#define Z2M_E_BUFFER_OVERFLOWN			17	//   
#define Z2M_E_PORT_ERROR				18	//   
#define Z2M_E_CARD_ERROR				19	//   
#define Z2M_E_OUT_CLOSED				20	//   -  


///////////////////////////////////////////////////////////
// 

typedef INT Tz2mReaderSN;	//   
typedef BYTE Tz2mCardNum[16];	//   (  0   
typedef Tz2mCardNum*	Pz2mCardNum;
typedef INT Tz2mCardType;

#define Z2M_C_Unk		0
#define Z2M_C_UL		1 // Mifare UltraLigth
#define Z2M_C_1K		2 // Mifare 1K
#define Z2M_C_4K		3 // Mifare 4K

typedef INT Tz2mReaderType;

#define Z2M_R_Unk		0
#define Z2M_R_Z2M		0x811
#define Z2M_R_M3N		0x0903

typedef INT TZ2mMsgTip;

#define Z2M_M_Reader_Connect		1
#define Z2M_M_Reader_Disconnect		2
#define Z2M_M_Card_Connect			3
#define Z2M_M_Card_Disconnect		4

const INT Z2M_DEF_M3N_PASSW = 0x03020100;

typedef INT64 Tz2mAuthKey;
typedef Tz2mAuthKey* Pz2mAuthKey;

typedef INT Tz2mAKMask;
typedef INT Tz2mAKMode;

#define Z2M_K_KeyA		0 //  A
#define Z2M_K_KeyB		1 //  B

typedef BYTE Tz2mMfAccBits[4];	//       Mifare 1K/4K
typedef Tz2mMfAccBits* Pz2mMfAccBits;

typedef INT Tz2mIndState;

#define z2mi_NoChange	0
#define z2mi_Enable		1
#define z2mi_Disable	2
#define z2mi_Auto		3

typedef struct
{
	BYTE	nWorkMode;
	BYTE	nOutZumm;
	BYTE	nOutTM;
	BYTE	nOutExit;
	BYTE	nOutLock;
	BYTE	nOutDoor;
	BYTE	nProt;
	BYTE	nFlags;
	BYTE	nCardFormat;
	BYTE	nSecurityMode;
	WORD	nReserved;
} Tz2mConfig, *Pz2mConfig;

typedef struct
{
	BYTE	nSecond;
	BYTE	nMinute;
	BYTE	nHour;
	BYTE	nDay;
	BYTE	nMonth;
	BYTE	nReserved;
	WORD	nYear;
} Tz2mTime, *Pz2mTime;

typedef struct
{
	BOOL	fEnable;
	BOOL	fAuto;
} Tz2mIndStateInfo, *Pz2mIndStateInfo;

typedef struct
{
	Tz2mIndState	nRed;
	Tz2mIndState	nGreen;
	Tz2mIndState	nSound;
	DWORD			nTime;		// ms
} Tz2mIndFlash, *Pz2mIndFlash;

const Tz2mAuthKey z2mDefAuthKey = INT64(0xFFFFFFFFFFFF);// FF(x6)

//    
typedef struct
{
	LPSTR pszPort;			// [in/out]  COM-
	INT nPortMax;			// [in/out]    
	Tz2mReaderType nTip;	// [out]  
	Tz2mReaderSN nSN;		// [out]   
	LPSTR pszManufactor;	// [in/out]    
	INT nManufactMax;		// [in/out]    
	DWORD nFlashVer;		// [out]   
} Tz2mFindReaderInfoA, *Pz2mFindReaderInfoA;

typedef struct 
{
	LPWSTR pszPort;			// [in/out]  COM-
	INT nPortMax;			// [in/out]    
	Tz2mReaderType nTip;	// [out]  
	Tz2mReaderSN nSN;		// [out]   
	LPWSTR pszManufactor;	// [in/out]    
	INT nManufactMax;		// [in/out]    
	DWORD nFlashVer;		// [out]   
} Tz2mFindReaderInfoW, *Pz2mFindReaderInfoW;

//   
typedef struct
{
	LPSTR pszPort;			// [in/out]  COM-
	INT nPortMax;			// [in/out]    
	Tz2mReaderType nTip;	// [out]  
	Tz2mReaderSN nSN;		// [out]   
	LPSTR pszManufactor;	// [in/out]    
	INT nManufactMax;		// [in/out]    
	DWORD nFlashVer;		// [out]   
} Tz2mReaderInfoA, *Pz2mReaderInfoA;

typedef struct 
{
	LPWSTR pszPort;			// [in/out]  COM-
	INT nPortMax;			// [in/out]    
	Tz2mReaderType nTip;	// [out]  
	Tz2mReaderSN nSN;		// [out]   
	LPWSTR pszManufactor;	// [in/out]    
	INT nManufactMax;		// [in/out]    
	DWORD nFlashVer;		// [out]   
} Tz2mReaderInfoW, *Pz2mReaderInfoW;

//   
typedef struct 
{
	Tz2mCardNum nNum;		// [out]  
	Tz2mCardType nTip;		// [out]  
} Tz2mFindCardInfo, *Pz2mFindCardInfo;

//   
typedef struct 
{
	Tz2mCardNum nNum;		// [out]  
	Tz2mCardType nTip;		// [out]  
	LPSTR pszRdPort;		// [in/out]  COM-
	INT nRdPortMax;			// [in/out]    
	Tz2mReaderType nRdTip;	// [out]  
	Tz2mReaderSN nRdSN;		// [out]   
} Tz2mCardInfoA, *Pz2mCardInfoA;

typedef struct 
{
	Tz2mCardNum nNum;		// [out]  
	Tz2mCardType nTip;		// [out]  
	LPWSTR pszRdPort;		// [in/out]  COM-
	INT nRdPortMax;			// [in/out]    
	Tz2mReaderType nRdTip;	// [out]  
	Tz2mReaderSN nRdSN;		// [out]   
} Tz2mCardInfoW, *Pz2mCardInfoW;

//     Mifare 1K/4K
typedef struct
{
	HANDLE hCard;			// [in/out]  
	INT nSectN;             // [in/out]  
	BOOL fOpenByMask;       // [in/out] True,    

    //    
	Tz2mAKMode nKeyMode;   	// [in/out] 1 : =0  A, =1  B
	Tz2mAKMask nKeyMask;    // [in/out]    
	INT nFoundKeyN;         // [out]      

    //    
	Tz2mAuthKey nKey;       // [in/out]   
	Tz2mCardNum nCardNum;	// [out]  
	Tz2mCardType nCardType;	// [out]  

    //    
	Tz2mMfAccBits aAreaAcc;	// [out]  3     
} Tz2mMfSectorInfo, *Pz2mMfSectorInfo;

typedef struct
{
	HANDLE hEvent;			// [in]    Windows (CreateEvent)
        //  =0,     Windows-
    //    Windows-
	DWORD nMsgId;			// [in] Id     (, WM_USER+1)
	DWORD nThreadId;		// [in] Id ,  
        //  =0,   hWndHandle
	HWND hWndHandle;		// [in]    
} Tz2mNotifyParam, *Pz2mNotifyParam;


typedef struct 
{
	TZ2mMsgTip nTip;		// [out]  
	LPSTR pszRdPort;		// [in/out]  COM-
	INT nRdPortMax;			// [in/out]    
	Tz2mReaderType nRdTip;	// [out]  
	Tz2mReaderSN nRdSN;		// [out]   
	Tz2mCardType nCdTip;	// [out]  
	Tz2mCardNum nCdNum;		// [out]  
} Tz2mNotifyMsgA, *Pz2mNotifyMsgA;

typedef struct 
{
	TZ2mMsgTip nTip;		// [out]  
	LPWSTR pszRdPort;		// [in/out]  COM-
	INT nRdPortMax;			// [in/out]    
	Tz2mReaderType nRdTip;	// [out]  
	Tz2mReaderSN	nRdSN;	// [out]   
	Tz2mCardType nCdTip;	// [out]  
	Tz2mCardNum nCdNum;		// [out]  
} Tz2mNotifyMsgW, *Pz2mNotifyMsgW;


typedef struct 
{
	HANDLE hEvent;			// [in]    Windows (CreateEvent)
        //  =0,     Windows-
    //    Windows-
	DWORD nMsgId;			// [in] Id     (, WM_USER+1)
	DWORD nThreadId;		// [in] Id ,  
        //  =0,   hWndHandle
	HWND hWndHandle;		// [in]    
} Tz2mAsyncParam, *Pz2mAsyncParam;

typedef struct
{
	Tz2mStatus nStatus;
	INT	nCurrent;
	INT nTotal;
} Tz2mAsyncState, *Pz2mAsyncState;

#ifdef UNICODE

typedef Tz2mFindReaderInfoW Tz2mFindReaderInfo;
typedef Pz2mFindReaderInfoW Pz2mFindReaderInfo;
typedef Tz2mReaderInfoW Tz2mReaderInfo;
typedef Pz2mReaderInfoW Pz2mReaderInfo;
typedef Tz2mCardInfoW Tz2mCardInfo;
typedef Pz2mCardInfoW Pz2mCardInfo;
typedef Tz2mNotifyMsgW Tz2mNotifyMsg;
typedef Pz2mNotifyMsgW Pz2mNotifyMsg;

#else

typedef Tz2mFindReaderInfoA Tz2mFindReaderInfo;
typedef Pz2mFindReaderInfoA Pz2mFindReaderInfo;
typedef Tz2mReaderInfoA Tz2mReaderInfo;
typedef Pz2mReaderInfoA Pz2mReaderInfo;
typedef Tz2mCardInfoA Tz2mCardInfo;
typedef Pz2mCardInfoA Pz2mCardInfo;
typedef Tz2mNotifyMsgA Tz2mNotifyMsg;
typedef Pz2mNotifyMsgA Pz2mNotifyMsg;

#endif // UNICODE


//////////////////////////////////////////////////////////////////////////
// 

#ifndef Z2M_LINKONREQUEST

#define EXPORT(type) extern "C" __declspec(dllexport) type __stdcall


//    
EXPORT(DWORD) z2mGetVersion();

// / 
EXPORT(Tz2mStatus) z2mInitialize();
EXPORT(VOID) z2mFinalize();

//    
EXPORT(VOID) z2mBreakCurrAsync();
//     
EXPORT(Tz2mStatus) z2mGetAsyncResult();
EXPORT(Tz2mStatus) z2mGetAsyncState(Pz2mAsyncState VState);

//  
EXPORT(Tz2mStatus) z2mFindFirstReaderA(PHANDLE VHandle,
										LPCSTR APort,
										Pz2mFindReaderInfoA VInfo,
										Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mFindFirstReaderW(PHANDLE VHandle,
										LPCSTR APort,
										Pz2mFindReaderInfoW VInfo,
										Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mFindNextReaderA(HANDLE AHandle,
										Pz2mFindReaderInfoA VInfo,
										Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mFindNextReaderW(HANDLE AHandle,
										Pz2mFindReaderInfoW VInfo,
										Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mCloseFindReader(HANDLE AHandle);
EXPORT(Tz2mStatus) z2mOpenReaderA(PHANDLE VHandle, LPCSTR APort,
								  Tz2mReaderSN ASN, Tz2mReaderType AType, LPINT APassw = NULL,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mOpenReaderW(PHANDLE VHandle, LPCWSTR APort,
								  Tz2mReaderSN ASN, Tz2mReaderType AType, LPINT APassw = NULL,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mCloseReader(HANDLE AHandle);
EXPORT(Tz2mStatus) z2mGetReaderInfoA(HANDLE AHandle, Pz2mReaderInfoA VInfo);
EXPORT(Tz2mStatus) z2mGetReaderInfoW(HANDLE AHandle, Pz2mReaderInfoW VInfo);
EXPORT(Tz2mStatus) z2mFindCard(HANDLE ARd, Pz2mFindCardInfo VInfo,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mOpenCard(PHANDLE VHandle, HANDLE ARd,
								Tz2mCardType ACdType, const Tz2mCardNum ACdNum,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mCloseCard(HANDLE AHandle);
EXPORT(Tz2mStatus) z2mGetCardInfoA(HANDLE AHandle, Pz2mCardInfoA VInfo);
EXPORT(Tz2mStatus) z2mGetCardInfoW(HANDLE AHandle, Pz2mCardInfoW VInfo);
EXPORT(Tz2mStatus) z2mReadCardData(HANDLE AHandle, DWORD AAddr, LPVOID VBuf,
								   DWORD ACount, LPDWORD VRead,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mWriteCardData(HANDLE AHandle, DWORD AAddr, LPVOID ABuf,
								   DWORD ACount, LPDWORD VWrite,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mReadULCard4Page(HANDLE AHandle, INT APageN,
								   LPVOID VBuf16,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mWriteULCardPage(HANDLE AHandle, INT APageN,
								   LPCVOID ABuf4,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mCreateNotifier(PHANDLE VHandle, const Tz2mAsyncParam AParam);
EXPORT(Tz2mStatus) z2mDestroyNotifier(HANDLE AHandle);
EXPORT(Tz2mStatus) z2mWaitNotifyMsg(HANDLE AHandle);
EXPORT(Tz2mStatus) z2mGetNextNotifyMsgA(HANDLE AHandle, Pz2mNotifyMsgA VMsg);
EXPORT(Tz2mStatus) z2mGetNextNotifyMsgW(HANDLE AHandle, Pz2mNotifyMsgW VMsg);

EXPORT(BOOL) z2mSameCardNumber(const Tz2mCardNum ANum1, const Tz2mCardNum ANum2);
EXPORT(BOOL) z2mIsEmptyCardNumber(const Tz2mCardNum ANum);

//   
EXPORT(Tz2mStatus) z2mGetErrorTextA(INT AErrN, LPSTR VTextBuf, INT &ATextMax);
EXPORT(Tz2mStatus) z2mGetErrorTextW(INT AErrN, LPWSTR VTextBuf, INT &ATextMax);

EXPORT(Tz2mStatus) z2mEncodeMfAccBits(INT AAreaN, LPBYTE VAccData4, BYTE ABits);
EXPORT(Tz2mStatus) z2mDecodeMfAccBits(INT AAreaN, LPCBYTE VAccData4, LPBYTE VResult);

EXPORT(Tz2mStatus) z2mReadMfBlock(HANDLE AHandle, INT ABlockN, LPBYTE VBuf16,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mWriteMfBlock(HANDLE AHandle, INT ABlockN, LPCBYTE ABuf16,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mGetMfSectorInfo(HANDLE AHandle, Pz2mMfSectorInfo VInfo,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mOpenMfSector(PHANDLE VHandle, Pz2mMfSectorInfo AInfo,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mCloseMfSector(HANDLE AHandle);
EXPORT(Tz2mStatus) z2mGetIndicatorState(HANDLE AHandle,
									Pz2mIndStateInfo VRed,
									Pz2mIndStateInfo VGreen,
									Pz2mIndStateInfo VSound,
									Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mSetIndicatorState(HANDLE AHandle,
								Tz2mIndState ARed,
								Tz2mIndState AGreen,
								Tz2mIndState ASound,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mAddIndicatorFlash(HANDLE AHandle,
								Pz2mIndFlash AFlashArr, INT AArrCount,
								BOOL AReset, LPINT VRCount = NULL,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mBreakIndicatorFlash(HANDLE AHandle,
								BOOL ASetAuto = TRUE,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mGetIndicatorFreeFlashCount(HANDLE AHandle,
								LPINT VCount,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mSetAuthKey(HANDLE AHandle,
								Tz2mAKMode AKeyMode, INT ACellN,
								const Tz2mAuthKey AKey,
								Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mActivatePowerKey(HANDLE AHandle,
								  BOOL AForce, DWORD ATimeMs,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mOutputControl(HANDLE AHandle,
								  DWORD AMask, DWORD AOutput,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mReadInput(HANDLE AHandle,
								  LPDWORD VFlags,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mSetConfig(HANDLE AHandle,
								  Pz2mConfig AConfig,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mGetConfig(HANDLE AHandle,
								  Pz2mConfig VConfig,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mSetReaderSecurity(HANDLE AHandle,
								  INT ABlockN, DWORD AKeyMask, BOOL AKeyB,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mGetReaderSecurity(HANDLE AHandle,
								  LPINT VBlockN, LPDWORD VKeyMask, LPBOOL VKeyB,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mGetReaderTime(HANDLE AHandle,
								  Pz2mTime VTime,
								  Pz2mAsyncParam AAsync = NULL);
EXPORT(Tz2mStatus) z2mSetReaderTime(HANDLE AHandle,
								  Pz2mTime ATime,
								  Pz2mAsyncParam AAsync = NULL);

#ifdef UNICODE

#define z2mGetErrorText z2mGetErrorTextA
#define z2mFindFirstReader z2mFindFirstReaderW
#define z2mFindNextReader z2mFindNextReaderW
#define z2mOpenReader z2mOpenReaderW
#define z2mGetReaderInfo z2mGetReaderInfoW
#define z2mGetCardInfo z2mGetCardInfoW
#define z2mGetNextNotifyMsg z2mGetNextNotifyMsgW

#else

#define z2mGetErrorText z2mGetErrorTextA
#define z2mFindFirstReader z2mFindFirstReaderA
#define z2mFindNextReader z2mFindNextReaderA
#define z2mOpenReader z2mOpenReaderA
#define z2mGetReaderInfo z2mGetReaderInfoA
#define z2mGetCardInfo z2mGetCardInfoA
#define z2mGetNextNotifyMsg z2mGetNextNotifyMsgA

#endif // UNICODE

#else // #ifndef Z2M_LINKONREQUEST

//    
typedef DWORD (__stdcall* Tz2mGetVersion) ();

// / 
typedef Tz2mStatus (__stdcall* Tz2mInitialize) ();
typedef VOID (__stdcall* Tz2mFinalize) ();

//    
typedef VOID (__stdcall* Tz2mBreakCurrAsync) ();

//     
typedef Tz2mStatus (__stdcall* Tz2mGetAsyncResult) ();

typedef Tz2mStatus (__stdcall* Tz2mGetAsyncState) (Pz2mAsyncState VState);

//  
typedef Tz2mStatus (__stdcall* Tz2mFindFirstReaderA) (PHANDLE VHandle,
									  LPCSTR APort,
									  Pz2mFindReaderInfoA VInfo,
									  Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mFindFirstReaderW) (PHANDLE VHandle,
									  LPCWSTR APort,
									  Pz2mFindReaderInfoW VInfo,
									  Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mFindNextReaderA) (HANDLE AHandle,
													 Pz2mFindReaderInfoA VInfo,
													 Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mFindNextReaderW) (HANDLE AHandle,
													 Pz2mFindReaderInfoW VInfo,
													 Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mCloseFindReader) (HANDLE AHandle);

typedef Tz2mStatus (__stdcall* Tz2mOpenReaderA) (PHANDLE VHandle,
									LPCSTR APort,
									Tz2mReaderSN ASN, Tz2mReaderType AType, LPINT APassw,
									Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mOpenReaderW) (PHANDLE VHandle,
									LPCWSTR APort,
									Tz2mReaderSN ASN, Tz2mReaderType AType, LPINT APassw,
									Pz2mAsyncParam AAsync);

typedef Tz2mStatus (__stdcall* Tz2mCloseReader) (HANDLE AHandle);

typedef Tz2mStatus (__stdcall* Tz2mGetReaderInfoA) (HANDLE AHandle, Pz2mReaderInfoA VInfo);
typedef Tz2mStatus (__stdcall* Tz2mGetReaderInfoW) (HANDLE AHandle, Pz2mReaderInfoW VInfo);

typedef Tz2mStatus (__stdcall* Tz2mFindCard) (HANDLE ARd, Pz2mFindCardInfo VInfo,
												   Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mOpenCard) (PHANDLE VHandle, HANDLE ARd,
								Tz2mCardType ACdType, const Tz2mCardNum ACdNum,
								Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mCloseCard) (HANDLE AHandle);

typedef Tz2mStatus (__stdcall* Tz2mGetCardInfoA) (HANDLE AHandle, Pz2mCardInfoA VInfo);
typedef Tz2mStatus (__stdcall* Tz2mGetCardInfoW) (HANDLE AHandle, Pz2mCardInfoW VInfo);

typedef Tz2mStatus (__stdcall* Tz2mReadCardData) (HANDLE AHandle, DWORD AAddr, PVOID VBuf,
												  DWORD ACount, LPDWORD VRead,
									   Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mWriteCardData) (HANDLE AHandle, DWORD AAddr, PVOID ABuf,
												   DWORD ACount, LPDWORD VWrite,
									   Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mReadULCard4Page) (HANDLE AHandle, INT APageN, LPVOID VBuf16,
									   Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mWriteULCardPage) (HANDLE AHandle, INT APageN, LPCVOID ABuf4,
									   Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mCreateNotifier) (PHANDLE VHandle, const Tz2mNotifyParam AParam);
typedef Tz2mStatus (__stdcall* Tz2mDestroyNotifier) (HANDLE AHandle);
typedef Tz2mStatus (__stdcall* Tz2mWaitNotifyMsg) (HANDLE AHandle);
typedef Tz2mStatus (__stdcall* Tz2mGetNextNotifyMsgA) (HANDLE AHandle, Pz2mNotifyMsgA VMsg);
typedef Tz2mStatus (__stdcall* Tz2mGetNextNotifyMsgW) (HANDLE AHandle, Pz2mNotifyMsgW VMsg);

typedef BOOL (__stdcall* Tz2mSameCardNumber) (const Tz2mCardNum ANum1, const Tz2mCardNum ANum2);
typedef BOOL (__stdcall* Tz2mIsEmptyCardNumber) (const Tz2mCardNum ANum);

//   
typedef Tz2mStatus (__stdcall* Tz2mGetErrorTextA) (INT AErrN, LPSTR VTextBuf, INT &ATextMax);
typedef Tz2mStatus (__stdcall* Tz2mGetErrorTextW) (INT AErrN, LPWSTR VTextBuf, INT &ATextMax);

typedef Tz2mStatus (__stdcall* Tz2mEncodeMfAccBits) (INT AAreaN, LPBYTE VAccData4, BYTE ABits);
typedef Tz2mStatus (__stdcall* Tz2mDecodeMfAccBits) (INT AAreaN, LPCBYTE AAccData4, LPBYTE VResult);
typedef Tz2mStatus (__stdcall* Tz2mReadMfBlock) (HANDLE AHandle, INT ABlockN,
											LPBYTE VBuf16, Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mWriteMfBlock) (HANDLE AHandle, INT ABlockN,
											LPCBYTE ABuf16, Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mGetMfSectorInfo) (HANDLE AHandle,
											Pz2mMfSectorInfo VInfo, Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mOpenMfSector) (HANDLE AHandle,
											Pz2mMfSectorInfo AInfo, Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mCloseMfSector) (HANDLE AHandle);
typedef Tz2mStatus (__stdcall* Tz2mGetIndicatorState) (HANDLE AHandle,
											Pz2mIndStateInfo VRed,
											Pz2mIndStateInfo VGreen,
											Pz2mIndStateInfo VSound,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mSetIndicatorState) (HANDLE AHandle,
											Tz2mIndState ARed,
											Tz2mIndState AGreen,
											Tz2mIndState ASound,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mAddIndicatorFlash) (HANDLE AHandle,
											Pz2mIndFlash AFlashArr, INT AArrCount,
											BOOL AReset, Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mBreakIndicatorFlash) (HANDLE AHandle,
											BOOL ASetAuto, Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mGetIndicatorFreeFlashCount) (HANDLE AHandle,
											LPINT VCount, Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mSetAuthKey) (HANDLE AHandle,
											Tz2mAKMode AKeyMode, INT ACellN,
                                            const Tz2mAuthKey AKey,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mActivatePowerKey) (HANDLE AHandle,
											BOOL AForce, DWORD ATimeMs,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mOutputControl) (HANDLE AHandle,
											DWORD AMask, DWORD AOutput,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mReadInput) (HANDLE AHandle,
											LPDWORD VFlags,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mSetConfig) (HANDLE AHandle,
											Pz2mConfig AConfig,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mGetConfig) (HANDLE AHandle,
											Pz2mConfig VConfig,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mSetReaderSecurity) (HANDLE AHandle,
											INT ABlockN, DWORD AKeyMask, BOOL AKeyB,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mGetReaderSecurity) (HANDLE AHandle,
											LPINT VBlockN, LPDWORD VKeyMask, LPBOOL VKeyB,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mGetReaderTime) (HANDLE AHandle,
											Pz2mTime VTime,
											Pz2mAsyncParam AAsync);
typedef Tz2mStatus (__stdcall* Tz2mSetReaderTime) (HANDLE AHandle,
											Pz2mTime ATime,
											Pz2mAsyncParam AAsync);

#ifdef UNICODE

typedef Tz2mGetErrorTextW Tz2mGetErrorText;
typedef Tz2mFindFirstReaderW Tz2mFindFirstReader;
typedef Tz2mFindNextReaderW Tz2mFindNextReader;
typedef Tz2mOpenReaderW Tz2mOpenReader;
typedef Tz2mGetReaderInfoW Tz2mGetReaderInfo;
typedef Tz2mGetCardInfoW Tz2mGetCardInfo;
typedef Tz2mGetNextNotifyMsgW Tz2mGetNextNotifyMsg;

#else // #ifdef UNICODE

typedef Tz2mGetErrorTextA Tz2mGetErrorText;
typedef Tz2mFindFirstReaderA Tz2mFindFirstReader;
typedef Tz2mFindNextReaderA Tz2mFindNextReader;
typedef Tz2mOpenReaderA Tz2mOpenReader;
typedef Tz2mGetReaderInfoA Tz2mGetReaderInfo;
typedef Tz2mGetCardInfoA Tz2mGetCardInfo;
typedef Tz2mGetNextNotifyMsgA Tz2mGetNextNotifyMsg;

#endif // #ifdef UNICODE #else

//    
Tz2mGetVersion				z2mGetVersion;
// / 
Tz2mInitialize					z2mInitialize;
Tz2mFinalize					z2mFinalize;
//    
Tz2mBreakCurrAsync			z2mBreakCurrAsync;
Tz2mGetAsyncResult			z2mGetAsyncResult;
Tz2mGetAsyncState			z2mGetAsyncState;
//  
Tz2mFindFirstReaderA		z2mFindFirstReaderA;
Tz2mFindFirstReaderW		z2mFindFirstReaderW;
Tz2mFindFirstReader			z2mFindFirstReader;
Tz2mFindNextReaderA			z2mFindNextReaderA;
Tz2mFindNextReaderW			z2mFindNextReaderW;
Tz2mFindNextReader			z2mFindNextReader;
Tz2mCloseFindReader			z2mCloseFindReader;

Tz2mOpenReaderA				z2mOpenReaderA;
Tz2mOpenReaderW				z2mOpenReaderW;
Tz2mOpenReader				z2mOpenReader;
Tz2mCloseReader				z2mCloseReader;
Tz2mGetReaderInfoA			z2mGetReaderInfoA;
Tz2mGetReaderInfoW			z2mGetReaderInfoW;
Tz2mGetReaderInfo			z2mGetReaderInfo;
Tz2mFindCard				z2mFindCard;
Tz2mOpenCard				z2mOpenCard;
Tz2mCloseCard				z2mCloseCard;
Tz2mGetCardInfoA			z2mGetCardInfoA;
Tz2mGetCardInfoW			z2mGetCardInfoW;
Tz2mGetCardInfo				z2mGetCardInfo;
Tz2mReadCardData			z2mReadCardData;
Tz2mWriteCardData			z2mWriteCardData;
Tz2mReadULCard4Page			z2mReadULCard4Page;
Tz2mWriteULCardPage			z2mWriteULCardPage;
Tz2mCreateNotifier			z2mCreateNotifier;
Tz2mDestroyNotifier			z2mDestroyNotifier;
Tz2mWaitNotifyMsg			z2mWaitNotifyMsg;
Tz2mGetNextNotifyMsgA		z2mGetNextNotifyMsgA;
Tz2mGetNextNotifyMsgW		z2mGetNextNotifyMsgW;
Tz2mGetNextNotifyMsg		z2mGetNextNotifyMsg;
Tz2mSameCardNumber			z2mSameCardNumber;
Tz2mIsEmptyCardNumber		z2mIsEmptyCardNumber;
//   
Tz2mGetErrorTextA			z2mGetErrorTextA;
Tz2mGetErrorTextW     		z2mGetErrorTextW;
Tz2mGetErrorText			z2mGetErrorText;

Tz2mEncodeMfAccBits			z2mEncodeMfAccBits;
Tz2mDecodeMfAccBits			z2mDecodeMfAccBits;
Tz2mReadMfBlock				z2mReadMfBlock;
Tz2mWriteMfBlock			z2mWriteMfBlock;
Tz2mGetMfSectorInfo			z2mGetMfSectorInfo;
Tz2mOpenMfSector			z2mOpenMfSector;
Tz2mCloseMfSector			z2mCloseMfSector;
Tz2mGetIndicatorState		z2mGetIndicatorState;
Tz2mSetIndicatorState		z2mSetIndicatorState;
Tz2mAddIndicatorFlash		z2mAddIndicatorFlash;
Tz2mBreakIndicatorFlash		z2mBreakIndicatorFlash;
Tz2mGetIndicatorFreeFlashCount	z2mGetIndicatorFreeFlashCount;
Tz2mSetAuthKey				z2mSetAuthKey;
Tz2mActivatePowerKey		z2mActivatePowerKey;
Tz2mOutputControl			z2mOutputControl;
Tz2mReadInput				z2mReadInput;
Tz2mSetConfig				z2mSetConfig;
Tz2mGetConfig				z2mGetConfig;
Tz2mSetReaderSecurity		z2mSetReaderSecurity;
Tz2mGetReaderSecurity		z2mGetReaderSecurity;
Tz2mGetReaderTime			z2mGetReaderTime;
Tz2mSetReaderTime			z2mSetReaderTime;

#endif // #ifndef Z2M_LINKONREQUEST #else

#pragma option pop
#pragma pack() // turn byte alignment off

HMODULE	z2mLib = 0;
INT		z2mLoadCount = 0;

BOOL IsZ2mLoaded()
{
#ifdef Z2M_LINKONREQUEST
	return z2mLib != 0;
#else
	return TRUE;
#endif // Z2M_LINKONREQUEST
}

VOID UnloadZ2m()
{
#ifdef Z2M_LINKONREQUEST
	z2mLoadCount--;
	if (z2mLoadCount > 0)
		return;
	FreeLibrary(z2mLib);
	z2mGetVersion = NULL;
	z2mInitialize = NULL;
	z2mFinalize = NULL;
	z2mBreakCurrAsync = NULL;
	z2mGetAsyncResult = NULL;
	z2mGetAsyncState = NULL;
	z2mFindFirstReaderA = NULL;
	z2mFindFirstReaderW = NULL;
	z2mFindFirstReader = NULL;
	z2mFindNextReaderA = NULL;
	z2mFindNextReaderW = NULL;
	z2mFindNextReader = NULL;
	z2mCloseFindReader = NULL;
	z2mOpenReaderA = NULL;
	z2mOpenReaderW = NULL;
	z2mOpenReader = NULL;
	z2mCloseReader = NULL;
	z2mGetReaderInfoA = NULL;
	z2mGetReaderInfoW = NULL;
	z2mGetReaderInfo = NULL;
	z2mFindCard = NULL;
	z2mOpenCard = NULL;
	z2mCloseCard = NULL;
	z2mGetCardInfoA = NULL;
	z2mGetCardInfoW = NULL;
	z2mGetCardInfo = NULL;
	z2mReadCardData = NULL;
	z2mWriteCardData = NULL;
	z2mReadULCard4Page = NULL;
	z2mWriteULCardPage = NULL;
	z2mCreateNotifier = NULL;
	z2mDestroyNotifier = NULL;
	z2mWaitNotifyMsg = NULL;
	z2mGetNextNotifyMsgA = NULL;
	z2mGetNextNotifyMsgW = NULL;
	z2mGetNextNotifyMsg = NULL;
	z2mSameCardNumber = NULL;
	z2mIsEmptyCardNumber = NULL;
	z2mGetErrorTextA = NULL;
	z2mGetErrorTextW = NULL;
	z2mGetErrorText = NULL;

	z2mEncodeMfAccBits = NULL;
	z2mDecodeMfAccBits = NULL;
	z2mReadMfBlock = NULL;
	z2mWriteMfBlock = NULL;
	z2mGetMfSectorInfo = NULL;
	z2mOpenMfSector = NULL;
	z2mCloseMfSector = NULL;
	z2mGetIndicatorState = NULL;
	z2mSetIndicatorState = NULL;
	z2mAddIndicatorFlash = NULL;
	z2mBreakIndicatorFlash = NULL;
	z2mGetIndicatorFreeFlashCount = NULL;
	z2mSetAuthKey = NULL;

	z2mActivatePowerKey = NULL;
	z2mOutputControl = NULL;
	z2mReadInput = NULL;
	z2mSetConfig = NULL;
	z2mGetConfig = NULL;
	z2mSetReaderSecurity = NULL;
	z2mGetReaderSecurity = NULL;
	z2mGetReaderTime = NULL;
	z2mSetReaderTime = NULL;

#endif // Z2M_LINKONREQUEST
}

PVOID GetModuleSymbolEx(HMODULE ADllHandle, LPCSTR ASymbolName, BOOL& VAccu)
{
	PVOID pRes = GetProcAddress(ADllHandle, ASymbolName);
	VAccu = VAccu && (pRes != NULL);
	return pRes;
}

#define GET_CTR_FN(type, lib, name) ((type)(reinterpret_cast<type>(GetModuleSymbolEx(lib, name, fRes))))

BOOL LoadZ2m()
{
#ifdef Z2M_LINKONREQUEST
	BOOL fRes = TRUE;
	z2mLoadCount++;
	if (z2mLoadCount > 1)
		return fRes;
	z2mLib = LoadLibrary(Z2M_DLLNAME);
	if (z2mLib == 0) return FALSE;

	z2mGetVersion			= GET_CTR_FN(Tz2mGetVersion, z2mLib, "z2mGetVersion@0");
	z2mInitialize			= GET_CTR_FN(Tz2mInitialize, z2mLib, "z2mInitialize@0");
	z2mFinalize				= GET_CTR_FN(Tz2mFinalize, z2mLib, "z2mFinalize@0");
	z2mBreakCurrAsync		= GET_CTR_FN(Tz2mBreakCurrAsync, z2mLib, "z2mBreakCurrAsync@0");
	z2mGetAsyncResult		= GET_CTR_FN(Tz2mGetAsyncResult, z2mLib, "z2mGetAsyncResult@0");
	z2mGetAsyncState		= GET_CTR_FN(Tz2mGetAsyncState, z2mLib, "z2mGetAsyncState@4");
	z2mFindFirstReaderA		= GET_CTR_FN(Tz2mFindFirstReaderA, z2mLib, "z2mFindFirstReaderA@16");
	z2mFindFirstReaderW		= GET_CTR_FN(Tz2mFindFirstReaderW, z2mLib, "z2mFindFirstReaderW@16");
	z2mFindNextReaderA		= GET_CTR_FN(Tz2mFindNextReaderA, z2mLib, "z2mFindNextReaderA@12");
	z2mFindNextReaderW		= GET_CTR_FN(Tz2mFindNextReaderW, z2mLib, "z2mFindNextReaderW@12");
	z2mCloseFindReader		= GET_CTR_FN(Tz2mCloseFindReader, z2mLib, "z2mCloseFindReader@4");
	z2mOpenReaderA			= GET_CTR_FN(Tz2mOpenReaderA, z2mLib, "z2mOpenReaderA@24");
	z2mOpenReaderW			= GET_CTR_FN(Tz2mOpenReaderW, z2mLib, "z2mOpenReaderW@24");
	z2mCloseReader			= GET_CTR_FN(Tz2mCloseReader, z2mLib, "z2mCloseReader@4");
	z2mGetReaderInfoA		= GET_CTR_FN(Tz2mGetReaderInfoA, z2mLib, "z2mGetReaderInfoA@8");
	z2mGetReaderInfoW		= GET_CTR_FN(Tz2mGetReaderInfoW, z2mLib, "z2mGetReaderInfoW@8");

	z2mFindCard				= GET_CTR_FN(Tz2mFindCard, z2mLib, "z2mFindCard@12");
	z2mOpenCard				= GET_CTR_FN(Tz2mOpenCard, z2mLib, "z2mOpenCard@20");
	z2mCloseCard			= GET_CTR_FN(Tz2mCloseCard, z2mLib, "z2mCloseCard@4");
	z2mGetCardInfoA			= GET_CTR_FN(Tz2mGetCardInfoA, z2mLib, "z2mGetCardInfoA@8");
	z2mGetCardInfoW			= GET_CTR_FN(Tz2mGetCardInfoW, z2mLib, "z2mGetCardInfoW@8");
	z2mReadCardData			= GET_CTR_FN(Tz2mReadCardData, z2mLib, "z2mReadCardData@24");
	z2mWriteCardData		= GET_CTR_FN(Tz2mWriteCardData, z2mLib, "z2mWriteCardData@24");
	z2mReadULCard4Page		= GET_CTR_FN(Tz2mReadULCard4Page, z2mLib, "z2mReadULCard4Page@16");
	z2mWriteULCardPage		= GET_CTR_FN(Tz2mWriteULCardPage, z2mLib, "z2mWriteULCardPage@16");
	z2mCreateNotifier		= GET_CTR_FN(Tz2mCreateNotifier, z2mLib, "z2mCreateNotifier@20");
	z2mDestroyNotifier		= GET_CTR_FN(Tz2mDestroyNotifier, z2mLib, "z2mDestroyNotifier@4");
	z2mWaitNotifyMsg		= GET_CTR_FN(Tz2mWaitNotifyMsg, z2mLib, "z2mWaitNotifyMsg@4");
	z2mGetNextNotifyMsgA	= GET_CTR_FN(Tz2mGetNextNotifyMsgA, z2mLib, "z2mGetNextNotifyMsgA@8");
	z2mGetNextNotifyMsgW	= GET_CTR_FN(Tz2mGetNextNotifyMsgW, z2mLib, "z2mGetNextNotifyMsgW@8");

	z2mSameCardNumber		= GET_CTR_FN(Tz2mSameCardNumber, z2mLib, "z2mSameCardNumber@8");
	z2mIsEmptyCardNumber	= GET_CTR_FN(Tz2mIsEmptyCardNumber, z2mLib, "z2mIsEmptyCardNumber@4");

	z2mGetErrorTextA		= GET_CTR_FN(Tz2mGetErrorTextA, z2mLib, "z2mGetErrorTextA@12");
	z2mGetErrorTextW		= GET_CTR_FN(Tz2mGetErrorTextW, z2mLib, "z2mGetErrorTextW@12");

	z2mEncodeMfAccBits		= GET_CTR_FN(Tz2mEncodeMfAccBits, z2mLib, "z2mEncodeMfAccBits@12");
	z2mDecodeMfAccBits		= GET_CTR_FN(Tz2mDecodeMfAccBits, z2mLib, "z2mDecodeMfAccBits@12");
	z2mReadMfBlock			= GET_CTR_FN(Tz2mReadMfBlock, z2mLib, "z2mReadMfBlock@16");
	z2mWriteMfBlock			= GET_CTR_FN(Tz2mWriteMfBlock, z2mLib, "z2mWriteMfBlock@16");
	z2mGetMfSectorInfo		= GET_CTR_FN(Tz2mGetMfSectorInfo, z2mLib, "z2mGetMfSectorInfo@12");
	z2mOpenMfSector			= GET_CTR_FN(Tz2mOpenMfSector, z2mLib, "z2mOpenMfSector@12");
	z2mCloseMfSector		= GET_CTR_FN(Tz2mCloseMfSector, z2mLib, "z2mCloseMfSector@4");
	z2mGetIndicatorState	= GET_CTR_FN(Tz2mGetIndicatorState, z2mLib, "z2mGetIndicatorState@20");
	z2mSetIndicatorState	= GET_CTR_FN(Tz2mSetIndicatorState, z2mLib, "z2mSetIndicatorState@20");
	z2mAddIndicatorFlash	= GET_CTR_FN(Tz2mAddIndicatorFlash, z2mLib, "z2mAddIndicatorFlash@24");
	z2mBreakIndicatorFlash	= GET_CTR_FN(Tz2mBreakIndicatorFlash, z2mLib, "z2mBreakIndicatorFlash@12");
	z2mGetIndicatorFreeFlashCount = GET_CTR_FN(Tz2mGetIndicatorFreeFlashCount, z2mLib, "z2mGetIndicatorFreeFlashCount@12");
	z2mSetAuthKey			= GET_CTR_FN(Tz2mSetAuthKey, z2mLib, "z2mSetAuthKey@24");

	z2mActivatePowerKey		= GET_CTR_FN(Tz2mActivatePowerKey, z2mLib, "z2mActivatePowerKey@16");
	z2mOutputControl		= GET_CTR_FN(Tz2mOutputControl, z2mLib, "z2mOutputControl@16");
	z2mReadInput			= GET_CTR_FN(Tz2mReadInput, z2mLib, "z2mReadInput@12");
	z2mSetConfig			= GET_CTR_FN(Tz2mSetConfig, z2mLib, "z2mSetConfig@12");
	z2mGetConfig			= GET_CTR_FN(Tz2mGetConfig, z2mLib, "z2mGetConfig@12");
	z2mSetReaderSecurity	= GET_CTR_FN(Tz2mSetReaderSecurity, z2mLib, "z2mSetReaderSecurity@20");
	z2mGetReaderSecurity	= GET_CTR_FN(Tz2mGetReaderSecurity, z2mLib, "z2mGetReaderSecurity@20");
	z2mGetReaderTime		= GET_CTR_FN(Tz2mGetReaderTime, z2mLib, "z2mGetReaderTime@12");
	z2mSetReaderTime		= GET_CTR_FN(Tz2mSetReaderTime, z2mLib, "z2mSetReaderTime@12");

#ifdef UNICODE
	z2mGetErrorText = z2mGetErrorTextW;
	z2mFindFirstReader = z2mFindFirstReaderW;
	z2mFindNextReader = z2mFindNextReaderW;
	z2mOpenReader = z2mOpenReaderW;
	z2mGetReaderInfo = z2mGetReaderInfoW;
	z2mGetCardInfo = z2mGetCardInfoW;
	z2mGetNextNotifyMsg = z2mGetNextNotifyMsgW;
#else
	z2mGetErrorText = z2mGetErrorTextA;
	z2mFindFirstReader = z2mFindFirstReaderA;
	z2mFindNextReader = z2mFindNextReaderA;
	z2mOpenReader = z2mOpenReaderA;
	z2mGetReaderInfo = z2mGetReaderInfoA;
	z2mGetCardInfo = z2mGetCardInfoA;
	z2mGetNextNotifyMsg = z2mGetNextNotifyMsgA;
#endif // UNICODE

	if (!fRes)
		UnloadZ2m();
	return fRes;
#else
	return TRUE;
#endif // Z2M_LINKONREQUEST
}

#endif /* __Z2M_H */

