unit Z2m;

interface

// (rom) this is the switch to change between static and dynamic linking.
// (rom) it is enabled by default here.
// (rom) To disable simply change the '$' to a '.'.
//{$DEFINE Z2M_LINKONREQUEST}

{$A-} // packed types
{$MINENUMSIZE 4}

uses
  Windows;

Const
  Z2M_DLLNAME           = 'z2m.dll';
Const
  Z2M_MAX_PORTNAME      = 16;
  Z2M_MAX_MANUFACTOR    = 250;

type
  Tz2mStatus = Integer;
Const
  Z2M_SUCCESS                   = 0; //   
  Z2M_E_UNKNOWN                 = 1; //  
  Z2M_E_INVALID_HANDLE          = 2; //  
  Z2M_E_INVALID_PARAMETER       = 3; //  
  Z2M_E_READER_NOT_FOUND        = 4; //   
  Z2M_E_CARD_NOT_FOUND          = 5; //   
  Z2M_E_READER_ACCESS           = 6; //    
  Z2M_E_CARD_ACCESS             = 7; //    
  Z2M_E_PAGE_LOCK               = 8; //  
  Z2M_E_NO_MORE_ITEMS           = 9; //    ( )
  Z2M_E_INSUFFICIENT_BUFFER     = 10; //    
  Z2M_E_DLL_NOT_INIT            = 11; // DLL  
  Z2M_E_PENDING                 = 12; //    
  Z2M_E_BREAK                   = 13; //   
  Z2M_E_UNSUPPORTED             = 14; //   
  Z2M_E_AUTHORIZE               = 15; //     1K/4K
                                      //     
  Z2M_E_READER_ERROR            = 16; //   
  Z2M_E_BUFFER_OVERFLOWN        = 17; //   
  Z2M_E_PORT_ERROR              = 18; //   
  Z2M_E_CARD_ERROR              = 19; //   
  Z2M_E_OUT_CLOSED              = 20; //   -  

type
  Tz2mReaderSN    = Integer;  //   
  Tz2mCardNum     = array[0..15] of Byte;  //  
                                           // (  0    )
  Pz2mCardNum     = ^Tz2mCardNum;
  Tz2mCardType    = Integer;
Const
  Z2M_C_Unk = 0;
  Z2M_C_UL  = 1;  // Mifare UltraLigth
  Z2M_C_1K  = 2;  // Mifare 1K
  Z2M_C_4K  = 3;  // Mifare 4K
type
  Tz2mReaderType  = Integer;
Const
  Z2M_R_Unk = 0;
  Z2M_R_Z2M = $0811;
  Z2M_R_M3N = $0903;
type
  Tz2mMsgTip      = Integer;
Const
  Z2M_M_Reader_Connect      = 1;
  Z2M_M_Reader_Disconnect   = 2;
  Z2M_M_Card_Connect        = 3;
  Z2M_M_Card_Disconnect     = 4;

Const
  Z2M_DEF_M3N_PASSW: Integer  = $03020100;
type
  Tz2mAuthKey     = Int64;                //    Mifare 1K/4K (6 )
  Pz2mAuthKey = ^Tz2mAuthKey;
  Tz2mAKMask      = Integer;              //      Mifare 1K/4K
  Tz2mAKMode      = (                     //    
    Z2M_K_KeyA,  //  A
    Z2M_K_KeyB   //  B
  );

  Tz2mMfAccBits   = array[0..3]of Byte;   //     
  Pz2mMfAccBits =^ Tz2mMfAccBits;         //  Mifare 1K/4K

  Tz2mIndState = (
    z2mi_NoChange,
    z2mi_Enable,
    z2mi_Disable,
    z2mi_Auto
  );

  Tz2mIndStateInfo = record
    fEnable         : LongBool;
    fAuto           : LongBool;
  end;
  Pz2mIndStateInfo = ^Tz2mIndStateInfo;
  Tz2mIndFlash = record
    nRed            : Tz2mIndState;
    nGreen          : Tz2mIndState;
    nSound          : Tz2mIndState;
    nTime           : Cardinal; // ms
  end;
  Pz2mIndFlash = ^Tz2mIndFlash;

  Tz2mConfig = record
    nWorkMode       : Byte;
    nOutZumm        : Byte;
    nOutTM          : Byte;
    nOutExit        : Byte;
    nOutLock        : Byte;
    nOutDoor        : Byte;
    nProt           : Byte;
    nFlags          : Byte; // 0 Impulse, 1 No card, 2 card num
    nCardFormat     : Byte;
    nSecurityMode   : Byte;
    nReserved       : Word;
  end;
  Pz2mConfig = ^Tz2mConfig;

  Tz2mTime = record
    nSecond         : Byte;
    nMinute         : Byte;
    nHour           : Byte;
    nDay            : Byte;
    nMonth          : Byte;
    nReserved       : Byte;
    nYear           : Word;
  end;
  Pz2mTime = ^Tz2mTime;

Const
  z2mDefAuthKey: Tz2mAuthKey = $FFFFFFFFFFFF; // FF(x6)

type
  // for Ansi
  Tz2mFindReaderInfoA = record  //    
    pszPort	        : PAnsiChar;	    //  COM-
    nPortMax        : Integer;
    nTip            : Tz2mReaderType; //  
    nSN		          : Tz2mReaderSN;	  //   
    pszManufactor   : PAnsiChar;      //    
    nManufactorMax  : Integer;
    nFlashVer       : Cardinal;
  end;
  Pz2mFindReaderInfoA = ^Tz2mFindReaderInfoA;

  // for Unicode
  Tz2mFindReaderInfoW = record
    pszPort	        : PWideChar;	    //  COM-
    nPortMax        : Integer;
    nTip            : Tz2mReaderType; //  
    nSN		          : Tz2mReaderSN;	  //   
    pszManufactor   : PWideChar;      //    
    nManufactorMax  : Integer;
    nFlashVer       : Cardinal;
  end;
  Pz2mFindReaderInfoW = ^Tz2mFindReaderInfoW;


  Tz2mReaderInfoA = record  //   
    pszPort	        : PAnsiChar;	    //  COM-
    nPortMax        : Integer;
    nTip            : Tz2mReaderType; //  
    nSN		          : Tz2mReaderSN;	  //  
    pszManufactor   : PAnsiChar;      //    
    nManufactorMax  : Integer;
    nFlashVer       : Cardinal;
  end;
  Pz2mReaderInfoA = ^Tz2mReaderInfoA;

  Tz2mReaderInfoW = record  //   
    pszPort	        : PWideChar;	    //  COM-
    nPortMax        : Integer;
    nTip            : Tz2mReaderType; //  
    nSN		          : Tz2mReaderSN;	  //  
    pszManufactor   : PWideChar;      //    
    nManufactorMax  : Integer;
    nFlashVer       : Cardinal;
  end;
  Pz2mReaderInfoW = ^Tz2mReaderInfoW;


  Tz2mFindCardInfo    = record  //   
    nNum	          : Tz2mCardNum;	  //  
    nTip            : Tz2mCardType;
  end;
  Pz2mFindCardInfo = ^Tz2mFindCardInfo;


  Tz2mCardInfoA = record  //   
    nNum	          : Tz2mCardNum;	  //  
    nTip            : Tz2mCardType;   //  
    pszRdPort	      : PAnsiChar;	    //  COM-
    nRdPortMax      : Integer;
    nRdTip          : Tz2mReaderType; //  
    nRdSN		        : Tz2mReaderSN;	  //  
  end;
  Pz2mCardInfoA = ^Tz2mCardInfoA;

  Tz2mCardInfoW = record  //   
    nNum	          : Tz2mCardNum;	  //  
    nTip            : Tz2mCardType;
    pszRdPort	      : PWideChar;	    //  COM-
    nRdPortMax      : Integer;
    nRdTip          : Tz2mReaderType; //  
    nRdSN		        : Tz2mReaderSN;	  //  
  end;
  Pz2mCardInfoW = ^Tz2mCardInfoW;

  Tz2mMfSectorInfo    = record            //     Mifare 1K/4K
    hCard       : THandle;          // [in/out]  
    nSectN      : Integer;          // [in/out]  
    fOpenByMask : LongBool;         // [in/out] True,    

    //    
    nKeyMode    : Tz2mAKMode;       // [in/out] 1 : =0  A, =1  B
    nKeyMask    : Tz2mAKMask;       // [in/out]    
    nFoundKeyN  : Integer;          // [out]      

    //    
    nKey        : Tz2mAuthKey;      // [in/out]   

    nCardNum    : Tz2mCardNum;      // [out]  
    nCardType   : Tz2mCardType;     // [out]  
    //    
    aAreaAcc    : Tz2mMfAccBits;    // [out]  3     
  end;
  Pz2mMfSectorInfo = ^Tz2mMfSectorInfo;

  TZ2mNotifyParam = record
    hEvent          : THANDLE;      // [in]    Windows (CreateEvent)
        //  =0,     Windows-

    //    Windows-
    nMsgId          : Cardinal;     // [in] Id     (, WM_USER+1)
    nThreadId       : Cardinal;     // [in] Id ,  
        //  =0,   hWndHandle
    hWndHandle      : HWND;         // [in]    
  end;


  TZ2mNotifyMsgA = record
    nTip            : TZ2mMsgTip;     //  

    pszRdPort	      : PAnsiChar;	    //  COM-
    nRdPortMax      : Integer;
    nRdTip          : Tz2mReaderType; //  
    nRdSN		        : Tz2mReaderSN;	  //  

    nCdTip          : Tz2mCardType;
    nCdNum          : Tz2mCardNum;	  //  
  end;
  PZ2mNotifyMsgA = ^TZ2mNotifyMsgA;

  TZ2mNotifyMsgW  = record
    nTip            : TZ2mMsgTip;     //  

    pszRdPort	      : PWideChar;	    //  COM-
    nRdPortMax      : Integer;
    nRdTip          : Tz2mReaderType; //  
    nRdSN		        : Tz2mReaderSN;	  //  

    nCdTip          : Tz2mCardType;
    nCdNum          : Tz2mCardNum;	  //  
  end;
  PZ2mNotifyMsgW = ^TZ2mNotifyMsgW;

  TZ2mAsyncParam = record
    hEvent          : THANDLE;      // [in]    Windows (CreateEvent)
        //  =0,     Windows-

    //    Windows-
    nMsgId          : Cardinal;     // [in] Id     (, WM_USER+1)
    nThreadId       : Cardinal;     // [in] Id ,  
        //  =0,   hWndHandle
    hWndHandle      : HWND;         // [in]    
  end;
  PZ2mAsyncParam = ^TZ2mAsyncParam;

  TZ2mAsyncState = record
    nStatus         : TZ2mStatus;
    nCurrent        : Integer;
    nTotal          : Integer;
  end;
  PZ2mAsyncState = ^TZ2mAsyncState;

{$A+}

{$IFDEF UNICODE}
  Tz2mFindReaderInfo  = Tz2mFindReaderInfoW;
  Pz2mFindReaderInfo  = Pz2mFindReaderInfoW;
  Tz2mReaderInfo      = Tz2mReaderInfoW;
  Pz2mReaderInfo      = Pz2mReaderInfoW;
  Tz2mCardInfo        = Tz2mCardInfoW;
  Pz2mCardInfo        = Pz2mCardInfoW;
  Tz2mNotifyMsg       = Tz2mNotifyMsgW;
  Pz2mNotifyMsg       = Pz2mNotifyMsgW;
{$ELSE}
  Tz2mFindReaderInfo  = Tz2mFindReaderInfoA;
  Pz2mFindReaderInfo  = Pz2mFindReaderInfoA;
  Tz2mReaderInfo      = Tz2mReaderInfoA;
  Pz2mReaderInfo      = Pz2mReaderInfoA;
  Tz2mCardInfo        = Tz2mCardInfoA;
  Pz2mCardInfo        = Pz2mCardInfoA;
  Tz2mNotifyMsg       = Tz2mNotifyMsgA;
  Pz2mNotifyMsg       = Pz2mNotifyMsgA;
{$ENDIF}

{$IFNDEF Z2M_LINKONREQUEST}

// 
function z2mGetVersion(): Cardinal; stdcall;

function z2mInitialize(): Tz2mStatus; stdcall;

procedure z2mFinalize(); stdcall;

procedure z2mBreakCurrAsync(); stdcall;

function z2mGetAsyncResult(): Tz2mStatus; stdcall;

function z2mGetAsyncState(VState: Pz2mAsyncState): Tz2mStatus; stdcall;


//  
function z2mFindFirstReaderA(VHandle: PHandle; APort: PAnsiChar;
    VInfo: Pz2mFindReaderInfoA;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mFindFirstReaderW(VHandle: PHandle; APort: PWideChar;
    VInfo: Pz2mFindReaderInfoW;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mFindFirstReader(VHandle: PHandle; APort: PChar;
    VInfo: Pz2mFindReaderInfo;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mFindNextReaderA(AHandle: THandle; VInfo: Pz2mFindReaderInfoA;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mFindNextReaderW(AHandle: THandle; VInfo: Pz2mFindReaderInfoW;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mFindNextReader(AHandle: THandle; VInfo: Pz2mFindReaderInfo;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mCloseFindReader(AHandle: THandle): Tz2mStatus; stdcall;


//   
function z2mOpenReaderA(VHandle: PHandle; APort: PAnsiChar;
    ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger=nil;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mOpenReaderW(VHandle: PHandle; APort: PWideChar;
    ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger=nil;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mOpenReader(VHandle: PHandle; APort: PChar;
    ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger=nil;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mCloseReader(AHandle: THandle): Tz2mStatus; stdcall;


//      
function z2mGetReaderInfoA(AHandle: THandle;
    VInfo: Pz2mReaderInfoA): Tz2mStatus; stdcall;

function z2mGetReaderInfoW(AHandle: THandle;
    VInfo: Pz2mReaderInfoW): Tz2mStatus; stdcall;

function z2mGetReaderInfo(AHandle: THandle;
    VInfo: Pz2mReaderInfo): Tz2mStatus; stdcall;


//  
function z2mFindCard(ARd: THandle; VInfo: Pz2mFindCardInfo;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

//   
function z2mOpenCard(VHandle: PHandle; ARd: THandle;
      ACdType: Tz2mCardType; Const ACdNum: Tz2mCardNum;
      AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mCloseCard(AHandle: THandle): Tz2mStatus; stdcall;


//   
function z2mGetCardInfoA(AHandle: THandle;
    VInfo: Pz2mCardInfoA): Tz2mStatus; stdcall;

function z2mGetCardInfoW(AHandle: THandle;
    VInfo: Pz2mCardInfoW): Tz2mStatus; stdcall;

function z2mGetCardInfo(AHandle: THandle;
    VInfo: Pz2mCardInfo): Tz2mStatus; stdcall;

function z2mReadCardData(AHandle: THandle; AAddr: Cardinal;
    VBuf: Pointer; ACount: Cardinal; VRead: PCardinal;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mWriteCardData(AHandle: THandle; AAddr: Cardinal;
    ABuf: Pointer; ACount: Cardinal; VWrite: PCardinal;
    AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;


// /  Mifare UL
function z2mReadULCard4Page(AHandle: THandle; APageN: Integer;
    VBuf16: Pointer; AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

function z2mWriteULCardPage(AHandle: THandle; APageN: Integer;
    ABuf4: Pointer; AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

//      Dll
function z2mCreateNotifier(VHandle: PHandle;
    Const AParam: Tz2mNotifyParam): Tz2mStatus; stdcall;

function z2mDestroyNotifier(AHandle: THandle): Tz2mStatus; stdcall;

function z2mWaitNotifyMsg(AHandle: THandle): Tz2mStatus; stdcall;

function z2mGetNextNotifyMsgA(AHandle: THandle;
    VMsg: Pz2mNotifyMsgA): Tz2mStatus; stdcall;

function z2mGetNextNotifyMsgW(AHandle: THandle;
    VMsg: Pz2mNotifyMsgW): Tz2mStatus; stdcall;

function z2mGetNextNotifyMsg(AHandle: THandle;
    VMsg: Pz2mNotifyMsg): Tz2mStatus; stdcall;

function z2mSameCardNumber(Const ANum1, ANum2: Tz2mCardNum): LongBool; stdcall;
function z2mIsEmptyCardNumber(Const ANum: Tz2mCardNum): LongBool; stdcall;

//   
function z2mGetErrorTextA(AErr: Integer; VTextBuf: PAnsiChar;
    var AMaxBuf: Integer): Tz2mStatus; stdcall;

function z2mGetErrorTextW(AErr: Integer; VTextBuf: PWideChar;
    var AMaxBuf: Integer): Tz2mStatus; stdcall;

function z2mGetErrorText(AErr: Integer; VTextBuf: PChar;
      var AMaxBuf: Integer): Tz2mStatus; stdcall;

function z2mEncodeMfAccBits(AAreaN: Integer; VAccData4: PByte; ABits: Byte): Tz2mStatus; stdcall;
function z2mDecodeMfAccBits(AAreaN: Integer; AAccData4: PByte; VResult: PByte): Tz2mStatus; stdcall;
function z2mReadMfBlock(AHandle: THandle; ABlockN: Integer;
    VBuf16: PByte; AAsync: PZ2mAsyncParam = nil): TZ2mStatus; stdcall;
function z2mWriteMfBlock(AHandle: THandle; ABlockN: Integer;
    ABuf16: PByte; AAsync: PZ2mAsyncParam = nil): TZ2mStatus; stdcall;
function z2mGetMfSectorInfo(AHandle: THandle; VInfo: Pz2mMfSectorInfo;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mOpenMfSector(VHandle: PHandle; AInfo: Pz2mMfSectorInfo;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mCloseMfSector(AHandle: THandle): TZ2mStatus; stdcall;
function z2mGetIndicatorState(AHandle: THandle;
    VRed, VGreen, VSound: Pz2mIndStateInfo;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mSetIndicatorState(AHandle: THandle;
    ARed, AGreen, ASound: Tz2mIndState;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mAddIndicatorFlash(AHandle: THandle;
    AFlashArr: Pz2mIndFlash; AArrCount: Integer; AReset: LongBool;
    VRCount: PInteger=nil; AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mBreakIndicatorFlash(AHandle: THandle; ASetAuto: LongBool=True;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mGetIndicatorFreeFlashCount(AHandle: THandle; VCount: PInteger;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mSetAuthKey(AHandle: THandle; AKeyMode: Tz2mAKMode; ACellN: Integer;
    Const AKey: Tz2mAuthKey; AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;

function z2mActivatePowerKey(AHandle: THandle; AForce: LongBool; ATimeMs: Cardinal;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mOutputControl(AHandle: THandle; AMask, AOutput: Cardinal;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mReadInput(AHandle: THandle; VFlags: PCardinal;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mSetConfig(AHandle: THandle; AConfig: Pz2mConfig;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mGetConfig(AHandle: THandle; VConfig: Pz2mConfig;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mSetReaderSecurity(AHandle: THandle;
    ASectN, ABlockN: Integer; AKeyMask: Cardinal; AKeyB: Bool;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mGetReaderSecurity(AHandle: THandle;
    VSectN, VBlockN: PInteger; VKeyMask: PCardinal; VKeyB: PBool;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mGetReaderTime(AHandle: THandle; VTime: Pz2mTime;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
function z2mSetReaderTime(AHandle: THandle; ATime: Pz2mTime;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;


{$ELSE}

type
  Tz2mGetVersion = function(): Cardinal; stdcall;
  Tz2mInitialize = function(): Tz2mStatus; stdcall;
  Tz2mFinalize = procedure(); stdcall;
  Tz2mBreakCurrAsync = procedure(); stdcall;
  Tz2mGetAsyncResult = function(): Tz2mStatus; stdcall;
  Tz2mGetAsyncState = function(VState: Pz2mAsyncState): Tz2mStatus; stdcall;

  Tz2mFindFirstReaderA = function(VHandle: PHandle; APort: PAnsiChar;
      VInfo: Pz2mFindReaderInfoA; AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;
  Tz2mFindFirstReaderW = function (VHandle: PHandle; APort: PWideChar;
      VInfo: Pz2mFindReaderInfoW; AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

  Tz2mFindNextReaderA = function(AHandle: THandle;
      VInfo: Pz2mFindReaderInfoA; AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;
  Tz2mFindNextReaderW = function(AHandle: THandle;
      VInfo: Pz2mFindReaderInfoW; AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

  Tz2mCloseFindReader = function(AHandle: THandle): Tz2mStatus; stdcall;


  Tz2mOpenReaderA = function(VHandle: PHandle; APort: PAnsiChar;
      ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger=nil;
      AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;
  Tz2mOpenReaderW = function(VHandle: PHandle; APort: PWideChar;
      ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger=nil;
      AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

  Tz2mCloseReader = function(AHandle: THandle): Tz2mStatus; stdcall;

  Tz2mGetReaderInfoA = function(AHandle: THandle;
      VInfo: Pz2mReaderInfoA): Tz2mStatus; stdcall;
  Tz2mGetReaderInfoW = function(AHandle: THandle;
      VInfo: Pz2mReaderInfoW): Tz2mStatus; stdcall;

  Tz2mFindCard = function(ARd: THandle; VInfo: Pz2mFindCardInfo;
      AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

  Tz2mOpenCard = function(VHandle: PHandle; ARd: THandle;
      ACdType: Tz2mCardType; Const ACdNum: Tz2mCardNum;
      AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;
  Tz2mCloseCard = function (AHandle: THandle): Tz2mStatus; stdcall;

  Tz2mGetCardInfoA = function(AHandle: THandle;
      VInfo: Pz2mCardInfoA): Tz2mStatus; stdcall;
  Tz2mGetCardInfoW = function(AHandle: THandle;
      VInfo: Pz2mCardInfoW): Tz2mStatus; stdcall;

  Tz2mReadCardData = function(AHandle: THandle; AAddr: Cardinal;
      VBuf: Pointer; ACount: Cardinal; VRead: PCardinal;
      AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;
  Tz2mWriteCardData = function(AHandle: THandle; AAddr: Cardinal;
      ABuf: Pointer; ACount: Cardinal; VWrite: PCardinal;
      AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

  Tz2mReadULCard4Page = function(AHandle: THandle; APageN: Integer;
      VBuf16: Pointer; AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;
  Tz2mWriteULCardPage = function(AHandle: THandle; APageN: Integer;
      ABuf4: Pointer; AAsync: Pz2mAsyncParam = nil): Tz2mStatus; stdcall;

  Tz2mCreateNotifier = function(VHandle: PHandle;
      Const AParam: Tz2mNotifyParam): Tz2mStatus; stdcall;
  Tz2mDestroyNotifier = function(AHandle: THandle): Tz2mStatus; stdcall;
  Tz2mWaitNotifyMsg = function(AHandle: THandle): Tz2mStatus; stdcall;
  Tz2mGetNextNotifyMsgA = function(AHandle: THandle;
      VMsg: Pz2mNotifyMsgA): Tz2mStatus; stdcall;
  Tz2mGetNextNotifyMsgW = function(AHandle: THandle;
      VMsg: Pz2mNotifyMsgW): Tz2mStatus; stdcall;

  Tz2mSameCardNumber = function(Const ANum1, ANum2: Tz2mCardNum): LongBool; stdcall;
  Tz2mIsEmptyCardNumber = function (Const ANum: Tz2mCardNum): LongBool; stdcall;

  Tz2mGetErrorTextA = function(AErr: Integer; VTextBuf: PAnsiChar;
      var AMaxBuf: Integer): Tz2mStatus; stdcall;
  Tz2mGetErrorTextW = function(AErr: Integer; VTextBuf: PWideChar;
      var AMaxBuf: Integer): Tz2mStatus; stdcall;

  Tz2mEncodeMfAccBits = function(AAreaN: Integer; VAccData4: PByte;
      ABits: Byte): Tz2mStatus; stdcall;
  Tz2mDecodeMfAccBits = function(AAreaN: Integer; AAccData4: PByte;
      VResult: PByte): Tz2mStatus; stdcall;
  Tz2mReadMfBlock = function(AHandle: THandle; ABlockN: Integer;
      VBuf16: PByte; AAsync: PZ2mAsyncParam = nil): TZ2mStatus; stdcall;
  Tz2mWriteMfBlock = function(AHandle: THandle; ABlockN: Integer;
      ABuf16: PByte; AAsync: PZ2mAsyncParam = nil): TZ2mStatus; stdcall;
  Tz2mGetMfSectorInfo = function(AHandle: THandle; VInfo: Pz2mMfSectorInfo;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mOpenMfSector = function(VHandle: PHandle; AInfo: Pz2mMfSectorInfo;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mCloseMfSector = function(AHandle: THandle): TZ2mStatus; stdcall;
  Tz2mGetIndicatorState = function(AHandle: THandle;
      VRed, VGreen, VSound: Pz2mIndStateInfo;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mSetIndicatorState = function(AHandle: THandle;
      ARed, AGreen, ASound: Tz2mIndState;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mAddIndicatorFlash = function(AHandle: THandle;
      AFlashArr: Pz2mIndFlash; AArrCount: Integer; AReset: LongBool;
      VRCount: PInteger=nil; AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mBreakIndicatorFlash = function(AHandle: THandle; ASetAuto: LongBool=True;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mGetIndicatorFreeFlashCount = function(AHandle: THandle; VCount: PInteger;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mSetAuthKey = function(AHandle: THandle; AKeyMode: Tz2mAKMode; ACellN: Integer;
      Const AKey: Tz2mAuthKey; AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mActivatePowerKey = function(AHandle: THandle; AForce: LongBool; ATimeMs: Cardinal;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mOutputControl = function(AHandle: THandle; AMask, AOutput: Cardinal;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mReadInput = function(AHandle: THandle; VFlags: PCardinal;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mSetConfig = function(AHandle: THandle; AConfig: Pz2mConfig;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mGetConfig = function(AHandle: THandle; VConfig: Pz2mConfig;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mSetReaderSecurity = function(AHandle: THandle;
      ASectN, ABlockN: Integer; AKeyMask: Cardinal; AKeyB: Bool;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mGetReaderSecurity = function(AHandle: THandle;
      VSectN, VBlockN: PInteger; VKeyMask: PCardinal; VKeyB: PBool;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mGetReaderTime = function(AHandle: THandle; VTime: Pz2mTime;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
  Tz2mSetReaderTime = function(AHandle: THandle; ATime: Pz2mTime;
      AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;

{$IFDEF UNICODE}
  Tz2mFindFirstReader   = Tz2mFindFirstReaderW;
  Tz2mFindNextReader    = Tz2mFindNextReaderW;
  Tz2mOpenReader        = Tz2mOpenReaderW;
  Tz2mGetReaderInfo     = Tz2mGetReaderInfoW;
  Tz2mGetCardInfo       = Tz2mGetCardInfoW;
  Tz2mGetNextNotifyMsg  = Tz2mGetNextNotifyMsgW;
{$ELSE}
  Tz2mFindFirstReader   = Tz2mFindFirstReaderA;
  Tz2mFindNextReader    = Tz2mFindNextReaderA;
  Tz2mOpenReader        = Tz2mOpenReaderA;
  Tz2mGetReaderInfo     = Tz2mGetReaderInfoA;
  Tz2mGetCardInfo       = Tz2mGetCardInfoA;
  Tz2mGetNextNotifyMsg  = Tz2mGetNextNotifyMsgA;
{$ENDIF}

var
//    
  z2mGetVersion               : Tz2mGetVersion;
  z2mInitialize               : Tz2mInitialize;
  z2mFinalize                 : Tz2mFinalize;
  z2mBreakCurrAsync           : Tz2mBreakCurrAsync;
  z2mGetAsyncResult           : Tz2mGetAsyncResult;
  z2mGetAsyncState            : Tz2mGetAsyncState;
  z2mFindFirstReaderA         : Tz2mFindFirstReaderA;
  z2mFindFirstReaderW         : Tz2mFindFirstReaderW;
  z2mFindNextReaderA          : Tz2mFindNextReaderA;
  z2mFindNextReaderW          : Tz2mFindNextReaderW;
  z2mCloseFindReader          : Tz2mCloseFindReader;
  z2mOpenReaderA              : Tz2mOpenReaderA;
  z2mOpenReaderW              : Tz2mOpenReaderW;
  z2mCloseReader              : Tz2mCloseReader;
  z2mGetReaderInfoA           : Tz2mGetReaderInfoA;
  z2mGetReaderInfoW           : Tz2mGetReaderInfoW;
  z2mFindCard                 : Tz2mFindCard;
  z2mOpenCard                 : Tz2mOpenCard;
  z2mCloseCard                : Tz2mCloseCard;
  z2mGetCardInfoA             : Tz2mGetCardInfoA;
  z2mGetCardInfoW             : Tz2mGetCardInfoW;
  z2mReadCardData             : Tz2mReadCardData;
  z2mWriteCardData            : Tz2mWriteCardData;
  z2mReadULCard4Page          : Tz2mReadULCard4Page;
  z2mWriteULCardPage          : Tz2mWriteULCardPage;
  z2mCreateNotifier           : Tz2mCreateNotifier;
  z2mDestroyNotifier          : Tz2mDestroyNotifier;
  z2mWaitNotifyMsg            : Tz2mWaitNotifyMsg;
  z2mGetNextNotifyMsgW        : Tz2mGetNextNotifyMsgW;
  z2mGetNextNotifyMsgA        : Tz2mGetNextNotifyMsgA;
  z2mSameCardNumber           : Tz2mSameCardNumber;
  z2mIsEmptyCardNumber        : Tz2mIsEmptyCardNumber;
  z2mGetErrorTextA            : Tz2mGetErrorTextA;
  z2mGetErrorTextW            : Tz2mGetErrorTextW;
  z2mEncodeMfAccBits          : Tz2mEncodeMfAccBits;
  z2mDecodeMfAccBits          : Tz2mDecodeMfAccBits;
  z2mReadMfBlock              : Tz2mReadMfBlock;
  z2mWriteMfBlock             : Tz2mWriteMfBlock;
  z2mGetMfSectorInfo          : Tz2mGetMfSectorInfo;
  z2mOpenMfSector             : Tz2mOpenMfSector;
  z2mCloseMfSector            : Tz2mCloseMfSector;
  z2mGetIndicatorState        : Tz2mGetIndicatorState;
  z2mSetIndicatorState        : Tz2mSetIndicatorState;
  z2mAddIndicatorFlash        : Tz2mAddIndicatorFlash;
  z2mBreakIndicatorFlash      : Tz2mBreakIndicatorFlash;
  z2mGetIndicatorFreeFlashCount: Tz2mGetIndicatorFreeFlashCount;
  z2mSetAuthKey               : Tz2mSetAuthKey;
  z2mActivatePowerKey         : Tz2mActivatePowerKey;
  z2mOutputControl            : Tz2mOutputControl;
  z2mReadInput                : Tz2mReadInput;
  z2mSetConfig                : Tz2mSetConfig;
  z2mGetConfig                : Tz2mGetConfig;
  z2mSetReaderSecurity        : Tz2mSetReaderSecurity;
  z2mGetReaderSecurity        : Tz2mGetReaderSecurity;
  z2mGetReaderTime            : Tz2mGetReaderTime;
  z2mSetReaderTime            : Tz2mSetReaderTime;
{$IFDEF UNICODE}
  z2mFindFirstReader          : Tz2mFindFirstReaderW;
  z2mFindNextReader           : Tz2mFindNextReaderW;
  z2mOpenReader               : Tz2mOpenReaderW;
  z2mGetReaderInfo            : Tz2mGetReaderInfoW;
  z2mGetCardInfo              : Tz2mGetCardInfoW;
  z2mGetNextNotifyMsg         : Tz2mGetNextNotifyMsgW;
  z2mGetErrorText             : Tz2mGetErrorTextW;
{$ELSE}
  z2mFindFirstReader          : Tz2mFindFirstReaderA;
  z2mFindNextReader           : Tz2mFindNextReaderA;
  z2mOpenReader               : Tz2mOpenReaderA;
  z2mGetReaderInfo            : Tz2mGetReaderInfoA;
  z2mGetCardInfo              : Tz2mGetCardInfoA;
  z2mGetNextNotifyMsg         : Tz2mGetNextNotifyMsgA;
  z2mGetErrorText             : Tz2mGetErrorTextA;
{$ENDIF}

{$ENDIF !Z2M_LINKONREQUEST}

function Isz2mLoaded(): Boolean;
function Getz2mModuleHandle(): THandle;
function Loadz2m(): Boolean;
procedure Unloadz2m();

implementation

{$IFDEF Z2M_LINKONREQUEST}
var
  gLib: THandle = 0;
  gLoadCount: Integer = 0;
{$ENDIF Z2M_LINKONREQUEST}

function Isz2mLoaded(): Boolean;
begin
  {$IFDEF Z2M_LINKONREQUEST}
  Result := gLib <> 0;
  {$ELSE}
  Result := True;
  {$ENDIF Z2M_LINKONREQUEST}
end;

function Getz2mModuleHandle(): THandle;
begin
  {$IFDEF Z2M_LINKONREQUEST}
  Result := gLib;
  {$ELSE}
  Result := 0;
  {$ENDIF Z2M_LINKONREQUEST}
end;

function Loadz2m(): Boolean;
  {$IFDEF Z2M_LINKONREQUEST}
  function GetModuleSymbolEx(ASymbolName: LPCSTR; var VAccu: Boolean): Pointer;
  begin
    Result := GetProcAddress(gLib, ASymbolName);
    VAccu := VAccu and (Result <> nil);
  end;
  {$ENDIF Z2M_LINKONREQUEST}
begin
  {$IFDEF Z2M_LINKONREQUEST}
  Result := True;
  Inc(gLoadCount);
  if gLoadCount > 1 then
    Exit;
  gLib := LoadLibrary(PChar(Z2M_DLLNAME));
  if gLib=0 then begin Result := False; exit; end;

  @z2mGetVersion := GetModuleSymbolEx('z2mGetVersion@0', Result);
  @z2mInitialize := GetModuleSymbolEx('z2mInitialize@0', Result);
  @z2mFinalize := GetModuleSymbolEx('z2mFinalize@0', Result);
  @z2mBreakCurrAsync := GetModuleSymbolEx('z2mBreakCurrAsync@0', Result);
  @z2mGetAsyncResult := GetModuleSymbolEx('z2mGetAsyncResult@0', Result);
  @z2mGetAsyncState := GetModuleSymbolEx('z2mGetAsyncState@4', Result);
  @z2mFindFirstReaderA := GetModuleSymbolEx('z2mFindFirstReaderA@16', Result);
  @z2mFindFirstReaderW := GetModuleSymbolEx('z2mFindFirstReaderW@16', Result);
  @z2mFindNextReaderA := GetModuleSymbolEx('z2mFindNextReaderA@12', Result);
  @z2mFindNextReaderW := GetModuleSymbolEx('z2mFindNextReaderW@12', Result);
  @z2mCloseFindReader := GetModuleSymbolEx('z2mCloseFindReader@4', Result);
  @z2mOpenReaderA := GetModuleSymbolEx('z2mOpenReaderA@24', Result);
  @z2mOpenReaderW := GetModuleSymbolEx('z2mOpenReaderW@24', Result);
  @z2mCloseReader := GetModuleSymbolEx('z2mCloseReader@4', Result);
  @z2mGetReaderInfoA := GetModuleSymbolEx('z2mGetReaderInfoA@8', Result);
  @z2mGetReaderInfoW := GetModuleSymbolEx('z2mGetReaderInfoW@8', Result);

  @z2mFindCard := GetModuleSymbolEx('z2mFindCard@12', Result);
  @z2mOpenCard := GetModuleSymbolEx('z2mOpenCard@20', Result);
  @z2mCloseCard := GetModuleSymbolEx('z2mCloseCard@4', Result);
  @z2mGetCardInfoA := GetModuleSymbolEx('z2mGetCardInfoA@8', Result);
  @z2mGetCardInfoW := GetModuleSymbolEx('z2mGetCardInfoW@8', Result);
  @z2mReadCardData := GetModuleSymbolEx('z2mReadCardData@24', Result);
  @z2mWriteCardData := GetModuleSymbolEx('z2mWriteCardData@24', Result);
  @z2mReadULCard4Page := GetModuleSymbolEx('z2mReadULCard4Page@16', Result);
  @z2mWriteULCardPage := GetModuleSymbolEx('z2mWriteULCardPage@16', Result);
  @z2mCreateNotifier := GetModuleSymbolEx('z2mCreateNotifier@20', Result);
  @z2mDestroyNotifier := GetModuleSymbolEx('z2mDestroyNotifier@4', Result);
  @z2mWaitNotifyMsg := GetModuleSymbolEx('z2mWaitNotifyMsg@4', Result);
  @z2mGetNextNotifyMsgW := GetModuleSymbolEx('z2mGetNextNotifyMsgW@8', Result);
  @z2mGetNextNotifyMsgA := GetModuleSymbolEx('z2mGetNextNotifyMsgA@8', Result);

  @z2mSameCardNumber := GetModuleSymbolEx('z2mSameCardNumber@8', Result);
  @z2mIsEmptyCardNumber := GetModuleSymbolEx('z2mIsEmptyCardNumber@4', Result);

  @z2mGetErrorTextA := GetModuleSymbolEx('z2mGetErrorTextA@12', Result);
  @z2mGetErrorTextW := GetModuleSymbolEx('z2mGetErrorTextW@12', Result);

  @z2mEncodeMfAccBits := GetModuleSymbolEx('z2mEncodeMfAccBits@12', Result);
  @z2mDecodeMfAccBits := GetModuleSymbolEx('z2mDecodeMfAccBits@12', Result);
  @z2mReadMfBlock := GetModuleSymbolEx('z2mReadMfBlock@16', Result);
  @z2mWriteMfBlock := GetModuleSymbolEx('z2mWriteMfBlock@16', Result);
  @z2mGetMfSectorInfo := GetModuleSymbolEx('z2mGetMfSectorInfo@12', Result);
  @z2mOpenMfSector := GetModuleSymbolEx('z2mOpenMfSector@12', Result);
  @z2mCloseMfSector := GetModuleSymbolEx('z2mCloseMfSector@4', Result);
  @z2mGetIndicatorState := GetModuleSymbolEx('z2mGetIndicatorState@20', Result);
  @z2mSetIndicatorState := GetModuleSymbolEx('z2mSetIndicatorState@20', Result);
  @z2mAddIndicatorFlash := GetModuleSymbolEx('z2mAddIndicatorFlash@24', Result);
  @z2mBreakIndicatorFlash := GetModuleSymbolEx('z2mBreakIndicatorFlash@12', Result);
  @z2mGetIndicatorFreeFlashCount := GetModuleSymbolEx('z2mGetIndicatorFreeFlashCount@12', Result);
  @z2mSetAuthKey := GetModuleSymbolEx('z2mSetAuthKey@24', Result);
  @z2mActivatePowerKey := GetModuleSymbolEx('z2mActivatePowerKey@16', Result);
  @z2mOutputControl := GetModuleSymbolEx('z2mOutputControl@16', Result);
  @z2mReadInput := GetModuleSymbolEx('z2mReadInput@12', Result);
  @z2mSetConfig := GetModuleSymbolEx('z2mSetConfig@12', Result);
  @z2mGetConfig := GetModuleSymbolEx('z2mGetConfig@12', Result);
  @z2mSetReaderSecurity := GetModuleSymbolEx('z2mSetReaderSecurity@24', Result);
  @z2mGetReaderSecurity := GetModuleSymbolEx('z2mGetReaderSecurity@24', Result);
  @z2mGetReaderTime := GetModuleSymbolEx('z2mGetReaderTime@12', Result);
  @z2mSetReaderTime := GetModuleSymbolEx('z2mSetReaderTime@12', Result);

{$IFDEF UNICODE}
  @z2mFindFirstReader := @z2mFindFirstReaderW;
  @z2mFindNextReader := @z2mFindNextReaderW;
  @z2mOpenReader := @z2mOpenReaderW;
  @z2mGetReaderInfo := @z2mGetReaderInfoW;
  @z2mGetCardInfo := @z2mGetCardInfoW;
  @z2mGetNextNotifyMsg := @z2mGetNextNotifyMsgW;
  @z2mGetErrorText := @z2mGetErrorTextW;
{$ELSE}
  @z2mFindFirstReader := @z2mFindFirstReaderA;
  @z2mFindNextReader := @z2mFindNextReaderA;
  @z2mOpenReader := @z2mOpenReaderA;
  @z2mGetReaderInfo := @z2mGetReaderInfoA;
  @z2mGetCardInfo := @z2mGetCardInfoA;
  @z2mGetNextNotifyMsg := @z2mGetNextNotifyMsgA;
  @z2mGetErrorText := @z2mGetErrorTextA;
{$ENDIF}
  if not Result then
    Unloadz2m();
  {$ELSE}
  Result := True;
  {$ENDIF Z2M_LINKONREQUEST}
end;

procedure Unloadz2m();
begin
  {$IFDEF Z2M_LINKONREQUEST}
  Dec(gLoadCount);
  if gLoadCount > 0 then
    exit;
  FreeLibrary(gLib);
  z2mGetVersion := nil;
  z2mInitialize := nil;
  z2mFinalize := nil;
  z2mBreakCurrAsync := nil;
  z2mGetAsyncResult := nil;
  z2mGetAsyncState := nil;
  z2mFindFirstReaderA := nil;
  z2mFindFirstReaderW := nil;
  z2mFindFirstReader := nil;
  z2mFindNextReaderA := nil;
  z2mFindNextReaderW := nil;
  z2mFindNextReader := nil;
  z2mCloseFindReader := nil;
  z2mOpenReaderA := nil;
  z2mOpenReaderW := nil;
  z2mGetReaderInfo := nil;
  z2mOpenReader := nil;
  z2mCloseReader := nil;
  z2mGetReaderInfoA := nil;
  z2mGetReaderInfoW := nil;

  z2mFindCard := nil;
  z2mOpenCard := nil;
  z2mCloseCard := nil;
  z2mGetCardInfoA := nil;
  z2mGetCardInfoW := nil;
  z2mGetCardInfo := nil;
  z2mReadCardData := nil;
  z2mWriteCardData := nil;
  z2mReadULCard4Page := nil;
  z2mWriteULCardPage := nil;

  z2mCreateNotifier := nil;
  z2mDestroyNotifier := nil;
  z2mWaitNotifyMsg := nil;
  z2mGetNextNotifyMsgW := nil;
  z2mGetNextNotifyMsgA := nil;
  z2mGetNextNotifyMsg := nil;

  z2mSameCardNumber := nil;
  z2mIsEmptyCardNumber := nil;

  z2mGetErrorTextA := nil;
  z2mGetErrorTextW := nil;
  z2mGetErrorText := nil;

  z2mEncodeMfAccBits := nil;
  z2mDecodeMfAccBits := nil;
  z2mReadMfBlock := nil;
  z2mWriteMfBlock := nil;
  z2mGetMfSectorInfo := nil;
  z2mOpenMfSector := nil;
  z2mCloseMfSector := nil;
  z2mGetIndicatorState := nil;
  z2mSetIndicatorState := nil;
  z2mAddIndicatorFlash := nil;
  z2mBreakIndicatorFlash := nil;
  z2mGetIndicatorFreeFlashCount := nil;
  z2mSetAuthKey := nil;
  z2mActivatePowerKey := nil;
  z2mOutputControl := nil;
  z2mReadInput := nil;
  z2mSetConfig := nil;
  z2mGetConfig := nil;
  z2mSetReaderSecurity := nil;
  z2mGetReaderSecurity := nil;
  z2mGetReaderTime := nil;
  z2mSetReaderTime := nil;

  {$ENDIF Z2M_LINKONREQUEST}
end;

{$IFNDEF Z2M_LINKONREQUEST}

function z2mGetVersion(): Cardinal; stdcall;
                            external Z2M_DLLNAME name 'z2mGetVersion@0';

function z2mInitialize(): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mInitialize@0';
procedure z2mFinalize(); stdcall;
                            external Z2M_DLLNAME name 'z2mFinalize@0';

procedure z2mBreakCurrAsync(); stdcall;
                            external Z2M_DLLNAME name 'z2mBreakCurrAsync@0';
function z2mGetAsyncResult(): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetAsyncResult@0';
function z2mGetAsyncState(VState: Pz2mAsyncState): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetAsyncState@4';

function z2mFindFirstReaderA(VHandle: PHandle; APort: PAnsiChar;
    VInfo: Pz2mFindReaderInfoA; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindFirstReaderA@16';
function z2mFindFirstReaderW(VHandle: PHandle; APort: PWideChar;
    VInfo: Pz2mFindReaderInfoW; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindFirstReaderW@16';
function z2mFindNextReaderA(AHandle: THandle;
    VInfo: Pz2mFindReaderInfoA; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindNextReaderA@12';
function z2mFindNextReaderW(AHandle: THandle;
    VInfo: Pz2mFindReaderInfoW; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindNextReaderW@12';
function z2mCloseFindReader(AHandle: THandle): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mCloseFindReader@4';

function z2mOpenReaderA(VHandle: PHandle; APort: PAnsiChar;
    ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger;
    AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mOpenReaderA@24';
function z2mOpenReaderW(VHandle: PHandle; APort: PWideChar;
    ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger;
    AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mOpenReaderW@24';
function z2mCloseReader(AHandle: THandle): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mCloseReader@4';

function z2mGetReaderInfoA(AHandle: THandle;
    VInfo: Pz2mReaderInfoA): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetReaderInfoA@8';
function z2mGetReaderInfoW(AHandle: THandle;
    VInfo: Pz2mReaderInfoW): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetReaderInfoW@8';

function z2mFindCard(ARd: THandle; VInfo: Pz2mFindCardInfo;
    AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindCard@12';

function z2mOpenCard(VHandle: PHandle; ARd: THandle;
      ACdType: Tz2mCardType; Const ACdNum: Tz2mCardNum;
      AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mOpenCard@20';
function z2mCloseCard(AHandle: THandle): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mCloseCard@4';

function z2mGetCardInfoA(AHandle: THandle;
    VInfo: Pz2mCardInfoA): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetCardInfoA@8';
function z2mGetCardInfoW(AHandle: THandle;
    VInfo: Pz2mCardInfoW): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetCardInfoW@8';

function z2mReadCardData(AHandle: THandle; AAddr: Cardinal;
    VBuf: Pointer; ACount: Cardinal; VRead: PCardinal;
    AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mReadCardData@24';
function z2mWriteCardData(AHandle: THandle; AAddr: Cardinal;
    ABuf: Pointer; ACount: Cardinal; VWrite: PCardinal;
    AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mWriteCardData@24';
function z2mReadULCard4Page(AHandle: THandle; APageN: Integer;
    VBuf16: Pointer; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mReadULCard4Page@16';
function z2mWriteULCardPage(AHandle: THandle; APageN: Integer;
    ABuf4: Pointer; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mWriteULCardPage@16';

function z2mCreateNotifier(VHandle: PHandle;
    Const AParam: Tz2mNotifyParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mCreateNotifier@20';
function z2mDestroyNotifier(AHandle: THandle): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mDestroyNotifier@4';
function z2mWaitNotifyMsg(AHandle: THandle): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mWaitNotifyMsg@4';
function z2mGetNextNotifyMsgA(AHandle: THandle;
    VMsg: Pz2mNotifyMsgA): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetNextNotifyMsgA@8';
function z2mGetNextNotifyMsgW(AHandle: THandle;
    VMsg: Pz2mNotifyMsgW): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetNextNotifyMsgW@8';


function z2mSameCardNumber(Const ANum1, ANum2: Tz2mCardNum): LongBool; stdcall;
                            external Z2M_DLLNAME name 'z2mSameCardNumber@8';
function z2mIsEmptyCardNumber(Const ANum: Tz2mCardNum): LongBool; stdcall;
                            external Z2M_DLLNAME name 'z2mIsEmptyCardNumber@4';

function z2mGetErrorTextA(AErr: Integer; VTextBuf: PAnsiChar;
      var AMaxBuf: Integer): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetErrorTextA@12';
function z2mGetErrorTextW(AErr: Integer; VTextBuf: PWideChar;
      var AMaxBuf: Integer): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetErrorTextW@12';
function z2mEncodeMfAccBits(AAreaN: Integer; VAccData4: PByte; ABits: Byte): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mEncodeMfAccBits@12';
function z2mDecodeMfAccBits(AAreaN: Integer; AAccData4: PByte; VResult: PByte): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mDecodeMfAccBits@12';
function z2mReadMfBlock(AHandle: THandle; ABlockN: Integer;
    VBuf16: PByte; AAsync: PZ2mAsyncParam = nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mReadMfBlock@16';
function z2mWriteMfBlock(AHandle: THandle; ABlockN: Integer;
    ABuf16: PByte; AAsync: PZ2mAsyncParam = nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mWriteMfBlock@16';
function z2mGetMfSectorInfo(AHandle: THandle; VInfo: Pz2mMfSectorInfo;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetMfSectorInfo@12';
function z2mOpenMfSector(VHandle: PHandle; AInfo: Pz2mMfSectorInfo;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mOpenMfSector@12';
function z2mCloseMfSector(AHandle: THandle): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mCloseMfSector@4';
function z2mGetIndicatorState(AHandle: THandle;
    VRed, VGreen, VSound: Pz2mIndStateInfo;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetIndicatorState@20';
function z2mSetIndicatorState(AHandle: THandle;
    ARed, AGreen, ASound: Tz2mIndState;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mSetIndicatorState@20';
function z2mAddIndicatorFlash(AHandle: THandle;
    AFlashArr: Pz2mIndFlash; AArrCount: Integer; AReset: LongBool;
    VRCount: PInteger=nil; AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mAddIndicatorFlash@24';
function z2mBreakIndicatorFlash(AHandle: THandle; ASetAuto: LongBool=True;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mBreakIndicatorFlash@12';
function z2mGetIndicatorFreeFlashCount(AHandle: THandle; VCount: PInteger;
    AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetIndicatorFreeFlashCount@12';
function z2mSetAuthKey(AHandle: THandle; AKeyMode: Tz2mAKMode; ACellN: Integer;
    Const AKey: Tz2mAuthKey; AAsync: PZ2mAsyncParam=nil): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mSetAuthKey@24';
function z2mActivatePowerKey(AHandle: THandle; AForce: LongBool; ATimeMs: Cardinal;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mActivatePowerKey@16';
function z2mOutputControl(AHandle: THandle; AMask, AOutput: Cardinal;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mOutputControl@16';
function z2mReadInput(AHandle: THandle; VFlags: PCardinal;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mReadInput@12';
function z2mSetConfig(AHandle: THandle; AConfig: Pz2mConfig;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mSetConfig@12';
function z2mGetConfig(AHandle: THandle; VConfig: Pz2mConfig;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetConfig@12';
function z2mSetReaderSecurity(AHandle: THandle;
    ASectN, ABlockN: Integer; AKeyMask: Cardinal; AKeyB: Bool;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mSetReaderSecurity@24';
function z2mGetReaderSecurity(AHandle: THandle;
    VSectN, VBlockN: PInteger; VKeyMask: PCardinal; VKeyB: PBool;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetReaderSecurity@24';
function z2mGetReaderTime(AHandle: THandle; VTime: Pz2mTime;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetReaderTime@12';
function z2mSetReaderTime(AHandle: THandle; ATime: Pz2mTime;
    AAsync: PZ2mAsyncParam): TZ2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mSetReaderTime@12';

{$IFDEF UNICODE}
function z2mFindFirstReader(VHandle: PHandle; APort: PChar;
    VInfo: Pz2mFindReaderInfo; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindFirstReaderW@16';
function z2mFindNextReader(AHandle: THandle;
    VInfo: Pz2mFindReaderInfo; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindNextReaderW@12';

function z2mOpenReader(VHandle: PHandle; APort: PChar;
    ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger;
    AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mOpenReaderW@24';

function z2mGetReaderInfo(AHandle: THandle;
    VInfo: Pz2mReaderInfo): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetReaderInfoW@8';

function z2mGetCardInfo(AHandle: THandle;
    VInfo: Pz2mCardInfo): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetCardInfoW@8';

function z2mGetNextNotifyMsg(AHandle: THandle;
    VMsg: Pz2mNotifyMsg): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetNextNotifyMsgW@8';

function z2mGetErrorText(AErr: Integer; VTextBuf: PChar;
      var AMaxBuf: Integer): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetErrorTextW@12';
{$ELSE}
function z2mFindFirstReader(VHandle: PHandle; APort: PChar;
    VInfo: Pz2mFindReaderInfo; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindFirstReaderA@16';
function z2mFindNextReader(AHandle: THandle;
    VInfo: Pz2mFindReaderInfo; AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mFindNextReaderA@12';

function z2mOpenReader(VHandle: PHandle; APort: PChar;
    ASN: Tz2mReaderSN; ARdType: Tz2mReaderType; APassw: PInteger;
    AAsync: Pz2mAsyncParam): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mOpenReaderA@24';

function z2mGetReaderInfo(AHandle: THandle;
    VInfo: Pz2mReaderInfo): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetReaderInfoA@8';

function z2mGetCardInfo(AHandle: THandle;
    VInfo: Pz2mCardInfo): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetCardInfoA@8';

function z2mGetNextNotifyMsg(AHandle: THandle;
    VMsg: Pz2mNotifyMsg): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetNextNotifyMsgA@8';


function z2mGetErrorText(AErr: Integer; VTextBuf: PChar;
      var AMaxBuf: Integer): Tz2mStatus; stdcall;
                            external Z2M_DLLNAME name 'z2mGetErrorTextA@12';
{$ENDIF}

{$ENDIF !Z2M_LINKONREQUEST}

end.

